@@ -1,7 +1,7 @@
 /*
  * Calcurse - text-based organizer
  *
- * Copyright (c) 2004-2015 calcurse Development Team <misc@calcurse.org>
+ * Copyright (c) 2004-2016 calcurse Development Team <misc@calcurse.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -47,6 +47,7 @@
 /* Long options */
 enum {
 	OPT_FILTER_TYPE = 1000,
+	OPT_FILTER_HASH,
 	OPT_FILTER_PATTERN,
 	OPT_FILTER_START_FROM,
 	OPT_FILTER_START_TO,
@@ -69,8 +70,11 @@ enum {
 	OPT_FMT_EV,
 	OPT_FMT_REV,
 	OPT_FMT_TODO,
+	OPT_DUMP_IMPORTED,
+	OPT_EXPORT_UID,
 	OPT_READ_ONLY,
-	OPT_STATUS
+	OPT_STATUS,
+	OPT_DAEMON
 };
 
 /*
@@ -78,19 +82,14 @@ enum {
  */
 static void usage(void)
 {
-	const char *arg_usage =
-	    _("Usage: calcurse [-g|-h|-v] [-an] [-t[num]] [-i<file>] [-x[format]]\n"
-	     "                [-d <date>|<num>] [-s[date]] [-r[range]]\n"
-	     "                [-c<file>] [-D<dir>] [-S<regex>] [--status]\n"
-	     "                [--read-only]\n");
-	fputs(arg_usage, stdout);
+	printf("%s\n", _("usage: calcurse [--daemon|-F|-G|-g|-i<file>|-Q|--status|-x[<format>]]\n"
+			 "                [-c<file>] [-D<path>] [-h] [-q] [--read-only] [-v]\n"
+			 "                [--filter-*] [--format-*]"));
 }
 
 static void usage_try(void)
 {
-	const char *arg_usage_try =
-	    _("Try 'calcurse -h' for more information.\n");
-	fputs(arg_usage_try, stdout);
+	printf("%s\n", _("Try `calcurse -h` for more information."));
 }
 
 /*
@@ -98,22 +97,10 @@ static void usage_try(void)
  */
 static void version_arg(void)
 {
-	const char *vtext =
-	    _("\nCopyright (c) 2004-2015 calcurse Development Team.\n"
-	      "This is free software; see the source for copying conditions.\n");
-
-	fprintf(stdout, _("Calcurse %s - text-based organizer\n"),
-		VERSION);
-	fputs(vtext, stdout);
-}
-
-static void more_info(void)
-{
-	fputs(_("\nFor more information, type '?' from within Calcurse, "
-		"or read the manpage.\n"), stdout);
-	fputs(_("Mail feature requests and suggestions to <misc@calcurse.org>.\n"),
-	      stdout);
-	fputs(_("Mail bug reports to <bugs@calcurse.org>.\n"), stdout);
+	printf(_("calcurse %s -- text-based organizer\n"), VERSION);
+	putchar('\n');
+	printf("%s\n", _("Copyright (c) 2004-2016 calcurse Development Team."));
+	printf("%s\n", _("This is free software; see the source for copying conditions."));
 }
 
 /*
@@ -121,68 +108,32 @@ static void more_info(void)
  */
 static void help_arg(void)
 {
-	const char *htext =
-	    _("\nMiscellaneous:\n"
-	      "  -h, --help\n"
-	      "	print this help and exit.\n"
-	      "\n  -v, --version\n"
-	      "	print calcurse version and exit.\n"
-	      "\n  --status\n"
-	      "	display the status of running instances of calcurse.\n"
-	      "\n  --read-only\n"
-	      "	Don't save configuration nor appointments/todos. Use with care.\n"
-	      "\nFiles:\n"
-	      "  -c <file>, --calendar <file>\n"
-	      "	specify the calendar <file> to use (has precedence over '-D').\n"
-	      "\n  -D <dir>, --directory <dir>\n"
-	      "	specify the data directory to use.\n"
-	      "\tIf not specified, the default directory is ~/.calcurse\n"
-	      "\nNon-interactive:\n"
-	      "  -a, --appointment\n"
-	      " 	print events and appointments for current day and exit.\n"
-	      "\n  -d <date|num>, --day <date|num>\n"
-	      "	print events and appointments for <date> or <num> upcoming days and"
-	      "\n\texit. To specify both a starting date and a range, use the\n"
-	      "\t'--startday' and the '--range' option.\n"
-	      "\n  -g, --gc\n"
-	      "	run the garbage collector for note files and exit. \n"
-	      "\n  -i <file>, --import <file>\n"
-	      "	import the icalendar data contained in <file>. \n"
-	      "\n  -n, --next\n"
-	      "	print next appointment within upcoming 24 hours "
-	      "and exit. Also given\n\tis the remaining time before this "
-	      "next appointment.\n"
-	      "\n  -l <num>, --limit <num>\n"
-	      "	only print information regarding the next <num> items. \n"
-	      "\n  -r[num], --range[=num]\n"
-	      "	print events and appointments for the [num] number of days"
-	      "\n\tand exit. If no [num] is given, a range of 1 day is considered.\n"
-	      "\n  -s[date], --startday[=date]\n"
-	      "	print events and appointments from [date] and exit.\n"
-	      "\tIf no [date] is given, the current day is considered.\n"
-	      "\n  -S<regex>, --search=<regex>\n"
-	      "	search for the given regular expression within events, appointments,\n"
-	      "\tand todos description.\n"
-	      "\n  -t[num], --todo[=num]\n"
-	      "	print todo list and exit. If the optional number [num] is given,\n"
-	      "\tthen only todos having a priority equal to [num] will be returned.\n"
-	      "\tThe priority number must be between 1 (highest) and 9 (lowest).\n"
-	      "\tIt is also possible to specify '0' for the priority, in which case\n"
-	      "\tonly completed tasks will be shown.\n"
-	      "\n  -x[format], --export[=format]\n"
-	      "	export user data to the specified format. Events, appointments and\n"
-	      "\ttodos are converted and echoed to stdout.\n"
-	      "\tTwo possible formats are available: 'ical' and 'pcal'.\n"
-	      "\tIf the optional argument format is not given, ical format is\n"
-	      "\tselected by default.\n"
-	      "\tnote: redirect standard output to export data to a file,\n"
-	      "\tby issuing a command such as: calcurse --export > calcurse.dat\n");
-
-	fprintf(stdout, _("Calcurse %s - text-based organizer\n"),
-		VERSION);
 	usage();
-	fputs(htext, stdout);
-	more_info();
+	putchar('\n');
+	printf("%s\n", _("Operations in non-interactive mode:"));
+	printf("%s\n", _("  -F, --filter  Read items, filter them, and write them back"));
+	printf("%s\n", _("  -G, --grep    Grep items from the data files"));
+	printf("%s\n", _("  -Q, --query   Print items in a given query range"));
+	putchar('\n');
+	printf("%s\n", _("Note that -F, -G and -Q can be combined with filter options --filter-*\n"
+			 "and formatting options --format-*. Consult the man page for details."));
+	putchar('\n');
+	printf("%s\n", _("Miscellaneous:"));
+	printf("%s\n", _("  -c, --calendar <file>   Specify the calendar data file to use"));
+	printf("%s\n", _("  --daemon                Run notification daemon in the background"));
+	printf("%s\n", _("  -D, --directory <path>  Specify the data directory to use"));
+	printf("%s\n", _("  -g, --gc                Run the garbage collector and exit"));
+	printf("%s\n", _("  -h, --help              Show this help text and exit"));
+	printf("%s\n", _("  -i, --import <file>     Import iCal data from a file"));
+	printf("%s\n", _("  -q, --quiet             Do not show system dialogs"));
+	printf("%s\n", _("  --read-only             Do not save configuration or data files"));
+	printf("%s\n", _("  --status                Display the status of running instances"));
+	printf("%s\n", _("  -v, --version           Show version information and exit"));
+	printf("%s\n", _("  -x, --export [<format>] Export items, valid formats: ical, pcal"));
+	putchar('\n');
+	printf("%s\n", _("For more information, type '?' from within calcurse, or read the manpage."));
+	printf("%s\n", _("Submit feature requests and suggestions to <misc@calcurse.org>."));
+	printf("%s\n", _("Submit bug reports to <bugs@calcurse.org>."));
 }
 
 /*
@@ -310,7 +261,7 @@ date_arg_from_to(long from, long to, int add_line, const char *fmt_apt,
 	}
 }
 
-static long parse_datearg(const char *str)
+static time_t parse_datearg(const char *str)
 {
 	struct date day;
 
@@ -329,12 +280,12 @@ static long parse_datearg(const char *str)
 	return LONG_MAX;
 }
 
-static long parse_datetimearg(const char *str)
+static time_t parse_datetimearg(const char *str)
 {
 	char *date = mem_strdup(str);
 	char *time;
 	unsigned hour, min;
-	long ret;
+	time_t ret;
 
 	time = strchr(date, ' ');
 	if (time) {
@@ -363,10 +314,10 @@ static long parse_datetimearg(const char *str)
 	return ret;
 }
 
-static int parse_daterange(const char *str, long *date_from, long *date_to)
+static int parse_daterange(const char *str, time_t *date_from, time_t *date_to)
 {
 	int ret = 0;
-	char *s = xstrdup(str);
+	char *s = mem_strdup(str);
 	char *p = strchr(s, ',');
 
 	if (!p)
@@ -393,7 +344,7 @@ static int parse_daterange(const char *str, long *date_from, long *date_to)
 
 	ret = 1;
 cleanup:
-	free(s);
+	mem_free(s);
 	return ret;
 }
 
@@ -435,21 +386,23 @@ cleanup:
 int parse_args(int argc, char **argv)
 {
 	/* Command-line flags */
-	int grep = 0, query = 0, next = 0;
-	int status = 0, gc = 0, import = 0, export = 0;
+	int grep = 0, grep_filter = 0, query = 0, next = 0;
+	int status = 0, gc = 0, import = 0, export = 0, daemon = 0;
 	/* Query ranges */
-	long from = -1, range = -1, to = -1;
+	time_t from = -1, to = -1;
+	int range = -1;
 	int limit = INT_MAX;
 	/* Filters */
-	struct item_filter filter = { 0, NULL, -1, -1, -1, -1, 0, 0, 0 };
+	struct item_filter filter = { 0, NULL, NULL, -1, -1, -1, -1, 0, 0, 0 };
 	/* Format strings */
-	const char *fmt_apt = " - %S -> %E\n\t%m\n";
-	const char *fmt_rapt = " - %S -> %E\n\t%m\n";
-	const char *fmt_ev = " * %m\n";
-	const char *fmt_rev = " * %m\n";
-	const char *fmt_todo = "%p. %m\n";
+	const char *fmt_apt = NULL;
+	const char *fmt_rapt = NULL;
+	const char *fmt_ev = NULL;
+	const char *fmt_rev = NULL;
+	const char *fmt_todo = NULL;
 	/* Import and export parameters */
 	int xfmt = IO_EXPORT_ICAL;
+	int dump_imported = 0, export_uid = 0;
 	/* Data file locations */
 	const char *cfile = NULL, *datadir = NULL, *ifile = NULL;
 
@@ -457,13 +410,14 @@ int parse_args(int argc, char **argv)
 	int ch;
 	regex_t reg;
 
-	static const char *optstr = "gGhvnNax::t::d:c:r::s::S:D:i:l:Q";
+	static const char *optstr = "FgGhvnNax::t::d:c:r::s::S:D:i:l:qQ";
 
 	struct option longopts[] = {
 		{"appointment", no_argument, NULL, 'a'},
 		{"calendar", required_argument, NULL, 'c'},
 		{"day", required_argument, NULL, 'd'},
 		{"directory", required_argument, NULL, 'D'},
+		{"filter", no_argument, NULL, 'F'},
 		{"gc", no_argument, NULL, 'g'},
 		{"grep", no_argument, NULL, 'G'},
 		{"help", no_argument, NULL, 'h'},
@@ -477,9 +431,11 @@ int parse_args(int argc, char **argv)
 		{"todo", optional_argument, NULL, 't'},
 		{"version", no_argument, NULL, 'v'},
 		{"export", optional_argument, NULL, 'x'},
+		{"quiet", no_argument, NULL, 'q'},
 		{"query", optional_argument, NULL, 'Q'},
 
 		{"filter-type", required_argument, NULL, OPT_FILTER_TYPE},
+		{"filter-hash", required_argument, NULL, OPT_FILTER_HASH},
 		{"filter-pattern", required_argument, NULL, OPT_FILTER_PATTERN},
 		{"filter-start-from", required_argument, NULL, OPT_FILTER_START_FROM},
 		{"filter-start-to", required_argument, NULL, OPT_FILTER_START_TO},
@@ -502,8 +458,11 @@ int parse_args(int argc, char **argv)
 		{"format-event", required_argument, NULL, OPT_FMT_EV},
 		{"format-recur-event", required_argument, NULL, OPT_FMT_REV},
 		{"format-todo", required_argument, NULL, OPT_FMT_TODO},
+		{"export-uid", no_argument, NULL, OPT_EXPORT_UID},
+		{"dump-imported", no_argument, NULL, OPT_DUMP_IMPORTED},
 		{"read-only", no_argument, NULL, OPT_READ_ONLY},
 		{"status", no_argument, NULL, OPT_STATUS},
+		{"daemon", no_argument, NULL, OPT_DAEMON},
 		{NULL, no_argument, NULL, 0}
 	};
 
@@ -531,6 +490,9 @@ int parse_args(int argc, char **argv)
 		case 'D':
 			datadir = optarg;
 			break;
+		case 'F':
+			grep_filter = grep = 1;
+			break;
 		case 'h':
 			help_arg();
 			goto cleanup;
@@ -572,6 +534,8 @@ int parse_args(int argc, char **argv)
 				filter.priority = atoi(optarg);
 				if (filter.priority == 0)
 					filter.completed = 1;
+				else
+					filter.uncompleted = 1;
 				EXIT_IF(filter.priority > 9,
 					_("invalid priority: %s"), optarg);
 			} else {
@@ -595,6 +559,9 @@ int parse_args(int argc, char **argv)
 					     optarg);
 			}
 			break;
+		case 'q':
+			quiet = 1;
+			break;
 		case 'Q':
 			query = 1;
 			break;
@@ -603,6 +570,9 @@ int parse_args(int argc, char **argv)
 			EXIT_IF(filter.type_mask == 0,
 				_("invalid filter mask"));
 			break;
+		case OPT_FILTER_HASH:
+			filter.hash = mem_strdup(optarg);
+			break;
 		case 'S':
 		case OPT_FILTER_PATTERN:
 			EXIT_IF(filter.regex,
@@ -699,12 +669,21 @@ int parse_args(int argc, char **argv)
 		case OPT_FMT_TODO:
 			fmt_todo = optarg;
 			break;
+		case OPT_DUMP_IMPORTED:
+			dump_imported = 1;
+			break;
+		case OPT_EXPORT_UID:
+			export_uid = 1;
+			break;
 		case OPT_READ_ONLY:
 			read_only = 1;
 			break;
 		case OPT_STATUS:
 			status = 1;
 			break;
+		case OPT_DAEMON:
+			daemon = 1;
+			break;
 		default:
 			usage();
 			usage_try();
@@ -716,7 +695,7 @@ int parse_args(int argc, char **argv)
 	if (filter.type_mask == 0)
 		filter.type_mask = TYPE_MASK_ALL;
 
-	if (status + grep + query + next + gc + import + export > 1) {
+	if (status + grep + query + next + gc + import + export + daemon > 1) {
 		ERROR_MSG(_("invalid argument combination"));
 		usage();
 		usage_try();
@@ -748,8 +727,22 @@ int parse_args(int argc, char **argv)
 		vars_init();
 		config_load();	/* To get output date format. */
 		io_load_data(&filter);
-		io_save_todo(NULL);
-		io_save_apts(NULL);
+		if (grep_filter) {
+			io_save_todo(path_todo);
+			io_save_apts(path_apts);
+		} else {
+			/*
+			 * Use default values for non-specified format strings.
+			 */
+			fmt_apt = fmt_apt ? fmt_apt : "%(raw)";
+			fmt_rapt = fmt_rapt ? fmt_rapt : "%(raw)";
+			fmt_ev = fmt_ev ? fmt_ev : "%(raw)";
+			fmt_rev = fmt_rev ? fmt_rev : "%(raw)";
+			fmt_todo = fmt_todo ? fmt_todo : "%(raw)";
+
+			io_dump_todo(fmt_todo);
+			io_dump_apts(fmt_apt, fmt_rapt, fmt_ev, fmt_rev);
+		}
 	} else if (query) {
 		io_check_file(path_apts);
 		io_check_file(path_todo);
@@ -757,6 +750,14 @@ int parse_args(int argc, char **argv)
 		vars_init();
 		config_load();	/* To get output date format. */
 		io_load_data(&filter);
+
+		/* Use default values for non-specified format strings. */
+		fmt_apt = fmt_apt ? fmt_apt : " - %S -> %E\n\t%m\n";
+		fmt_rapt = fmt_rapt ? fmt_rapt : " - %S -> %E\n\t%m\n";
+		fmt_ev = fmt_ev ? fmt_ev : " * %m\n";
+		fmt_rev = fmt_rev ? fmt_rev : " * %m\n";
+		fmt_todo = fmt_todo ? fmt_todo : "%p. %m\n";
+
 		int add_line = todo_arg(fmt_todo, &limit, &filter);
 		date_arg_from_to(from, to, add_line, fmt_apt, fmt_rapt, fmt_ev,
 				 fmt_rev, &limit);
@@ -775,14 +776,35 @@ int parse_args(int argc, char **argv)
 		/* Get default pager in case we need to show a log file. */
 		vars_init();
 		io_load_data(NULL);
-		io_import_data(IO_IMPORT_ICAL, ifile);
+		if (dump_imported) {
+			/*
+			 * Use default values for non-specified format strings.
+			 */
+			fmt_apt = fmt_apt ? fmt_apt : "%(raw)";
+			fmt_rapt = fmt_rapt ? fmt_rapt : "%(raw)";
+			fmt_ev = fmt_ev ? fmt_ev : "%(raw)";
+			fmt_rev = fmt_rev ? fmt_rev : "%(raw)";
+			fmt_todo = fmt_todo ? fmt_todo : "%(raw)";
+		} else {
+			/*
+			 * Do not dump items, unset format strings explicitly.
+			 */
+			fmt_apt = fmt_rapt = fmt_ev = fmt_rev = NULL;
+			fmt_todo = NULL;
+		}
+		io_import_data(IO_IMPORT_ICAL, ifile, fmt_ev, fmt_rev, fmt_apt,
+			       fmt_rapt, fmt_todo);
 		io_save_apts(path_apts);
 		io_save_todo(path_todo);
 	} else if (export) {
 		io_check_file(path_apts);
 		io_check_file(path_todo);
 		io_load_data(&filter);
-		io_export_data(xfmt);
+		io_export_data(xfmt, export_uid);
+	} else if (daemon) {
+		notify_init_vars();
+		dmon_stop();
+		dmon_start(0);
 	} else {
 		/* interactive mode */
 		non_interactive = 0;
