@@ -1,7 +1,7 @@
 /*
  * Calcurse - text-based organizer
  *
- * Copyright (c) 2004-2015 calcurse Development Team <misc@calcurse.org>
+ * Copyright (c) 2004-2016 calcurse Development Team <misc@calcurse.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -48,11 +48,24 @@ int count, reg;
  */
 static void do_storage(int day_changed)
 {
+	struct day_item *day = ui_day_selitem();
+	union aptev_ptr item;
+
+	if (day) {
+		/*
+		 * day_process_storage() rebuilds the vector of day items, so
+		 * we need to save the reference to the actual item here.
+		 */
+		item = day->item;
+	}
+
 	day_process_storage(ui_calendar_get_slctd_day(), day_changed);
 	ui_day_load_items();
 
 	if (day_changed)
 		ui_day_sel_reset();
+	else if (day)
+		ui_day_set_selitem_by_aptev_ptr(item);
 }
 
 static inline void key_generic_change_view(void)
@@ -98,6 +111,7 @@ static inline void key_view_item(void)
 static inline void key_generic_config_menu(void)
 {
 	wins_erase_status_bar();
+	wins_reset_status_page();
 	custom_config_main();
 	do_storage(0);
 	wins_update(FLAG_ALL);
@@ -106,7 +120,6 @@ static inline void key_generic_config_menu(void)
 static inline void key_generic_add_appt(void)
 {
 	ui_day_item_add();
-	do_storage(1);
 	wins_update(FLAG_CAL | FLAG_APP | FLAG_STA);
 }
 
@@ -120,8 +133,8 @@ static inline void key_add_item(void)
 {
 	switch (wins_slctd()) {
 	case APP:
+	case CAL:
 		ui_day_item_add();
-		do_storage(0);
 		wins_update(FLAG_CAL | FLAG_APP | FLAG_STA);
 		break;
 	case TOD:
@@ -242,9 +255,17 @@ static inline void key_view_note(void)
 	wins_update(FLAG_ALL);
 }
 
+static inline void key_generic_credits(void)
+{
+	if (!display_help("credits"))
+		warnbox(_("Unable to find documentation."));
+	wins_update(FLAG_ALL);
+}
+
 static inline void key_generic_help(void)
 {
-	display_help(NULL);
+	if (!display_help(NULL))
+		warnbox(_("Unable to find documentation."));
 	wins_update(FLAG_ALL);
 }
 
@@ -266,7 +287,7 @@ static inline void key_generic_reload(void)
 static inline void key_generic_import(void)
 {
 	wins_erase_status_bar();
-	io_import_data(IO_IMPORT_ICAL, NULL);
+	io_import_data(IO_IMPORT_ICAL, NULL, NULL, NULL, NULL, NULL, NULL);
 	ui_calendar_monthly_view_cache_set_invalid();
 	do_storage(0);
 	wins_update(FLAG_ALL);
@@ -283,10 +304,10 @@ static inline void key_generic_export()
 	switch (status_ask_choice
 		(export_msg, export_choices, nb_export_choices)) {
 	case 1:
-		io_export_data(IO_EXPORT_ICAL);
+		io_export_data(IO_EXPORT_ICAL, 0);
 		break;
 	case 2:
-		io_export_data(IO_EXPORT_PCAL);
+		io_export_data(IO_EXPORT_PCAL, 0);
 		break;
 	default:		/* User escaped */
 		break;
@@ -413,6 +434,9 @@ static inline void key_generic_scroll_up(void)
 	if (wins_slctd() == CAL) {
 		ui_calendar_view_prev();
 		wins_update(FLAG_CAL | FLAG_APP);
+	} else if (wins_slctd() == TOD) {
+		ui_todo_view_prev();
+		wins_update(FLAG_TOD | FLAG_APP);
 	}
 }
 
@@ -421,6 +445,9 @@ static inline void key_generic_scroll_down(void)
 	if (wins_slctd() == CAL) {
 		ui_calendar_view_next();
 		wins_update(FLAG_CAL | FLAG_APP);
+	} else if (wins_slctd() == TOD) {
+		ui_todo_view_next();
+		wins_update(FLAG_TOD | FLAG_APP);
 	}
 }
 
@@ -600,10 +627,11 @@ int main(int argc, char **argv)
 	 * implicitly calling wrefresh() later (causing ncurses race conditions).
 	 */
 	wins_wrefresh(win[KEY].p);
-	if (conf.system_dialogs) {
+	if (show_dialogs()) {
 		wins_update(FLAG_ALL);
 		io_startup_screen(no_data_file);
 	}
+	ui_calendar_monthly_view_cache_set_invalid();
 	do_storage(1);
 	ui_todo_load_items();
 	ui_todo_sel_reset();
@@ -656,6 +684,7 @@ int main(int argc, char **argv)
 			HANDLE_KEY(KEY_LOWER_PRIORITY, key_lower_priority);
 			HANDLE_KEY(KEY_EDIT_NOTE, key_edit_note);
 			HANDLE_KEY(KEY_VIEW_NOTE, key_view_note);
+			HANDLE_KEY(KEY_GENERIC_CREDITS, key_generic_credits);
 			HANDLE_KEY(KEY_GENERIC_HELP, key_generic_help);
 			HANDLE_KEY(KEY_GENERIC_SAVE, key_generic_save);
 			HANDLE_KEY(KEY_GENERIC_RELOAD, key_generic_reload);
