@@ -1,7 +1,7 @@
 /*
  * Calcurse - text-based organizer
  *
- * Copyright (c) 2004-2015 calcurse Development Team <misc@calcurse.org>
+ * Copyright (c) 2004-2016 calcurse Development Team <misc@calcurse.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -93,6 +93,7 @@
 #define DPID_PATH_NAME   ".daemon.pid"
 #define DLOG_PATH_NAME   "daemon.log"
 #define NOTES_DIR_NAME   "notes/"
+#define HOOKS_DIR_NAME   "hooks/"
 
 #define TODO_PATH        DIR_NAME TODO_PATH_NAME
 #define APTS_PATH        DIR_NAME APTS_PATH_NAME
@@ -102,6 +103,7 @@
 #define DLOG_PATH        DIR_NAME DLOG_PATH_NAME
 #define DPID_PATH        DIR_NAME DPID_PATH_NAME
 #define NOTES_DIR        DIR_NAME NOTES_DIR_NAME
+#define HOOKS_DIR        DIR_NAME HOOKS_DIR_NAME
 
 #define DEFAULT_EDITOR     "vi"
 #define DEFAULT_PAGER      "less"
@@ -321,6 +323,7 @@ struct event {
 struct todo {
 	char *mesg;
 	int id;
+	int completed;
 	char *note;
 };
 
@@ -405,11 +408,12 @@ enum item_type {
 /* Filter settings. */
 struct item_filter {
 	int type_mask;
+	char *hash;
 	regex_t *regex;
-	long start_from;
-	long start_to;
-	long end_from;
-	long end_to;
+	time_t start_from;
+	time_t start_to;
+	time_t end_from;
+	time_t end_to;
 	int priority;
 	int completed;
 	int uncompleted;
@@ -422,13 +426,20 @@ struct day_item {
 	union aptev_ptr item;
 };
 
-/* Available view for the calendar panel. */
+/* Available views for the calendar panel. */
 enum {
 	CAL_MONTH_VIEW,
 	CAL_WEEK_VIEW,
 	CAL_VIEWS
 };
 
+/* Available views for the todo panel. */
+enum {
+	TODO_SHOW_COMPLETED_VIEW,
+	TODO_HIDE_COMPLETED_VIEW,
+	TODO_VIEWS
+};
+
 struct notify_app {
 	long time;
 	int got_app;
@@ -587,6 +598,10 @@ struct pad {
 };
 
 /* Notification bar definition. */
+#define NOTIFY_FLAGGED_ONLY    0
+#define NOTIFY_UNFLAGGED_ONLY  1
+#define NOTIFY_ALL             2
+
 struct nbar {
 	unsigned show;		/* display or hide the notify-bar */
 	int cntdwn;		/* warn when time left before next app
@@ -599,6 +614,13 @@ struct nbar {
 	pthread_mutex_t mutex;
 };
 
+/* Dynamic strings. */
+struct string {
+	char *buf;
+	int bufsize;
+	int len;
+};
+
 /* Return codes for the getstring() function. */
 enum getstr {
 	GETSTRING_VALID,
@@ -606,6 +628,10 @@ enum getstr {
 	GETSTRING_RET		/* return was pressed without entering any text. */
 };
 
+/* Return codes for parse_datetime(). */
+#define PARSE_DATETIME_HAS_DATE (1 << 0)
+#define PARSE_DATETIME_HAS_TIME (1 << 1)
+
 /* Week days. */
 enum wday {
 	SUNDAY,
@@ -674,6 +700,8 @@ void apoint_llist_free(void);
 struct apoint *apoint_new(char *, char *, long, long, char);
 unsigned apoint_inday(struct apoint *, long *);
 void apoint_sec2str(struct apoint *, long, char *, char *);
+char *apoint_tostr(struct apoint *);
+char *apoint_hash(struct apoint *);
 void apoint_write(struct apoint *, FILE *);
 struct apoint *apoint_scan(FILE *, struct tm, struct tm, char, char *,
 			   struct item_filter *);
@@ -699,7 +727,7 @@ unsigned ui_calendar_week_begins_on_monday(void);
 void ui_calendar_store_current_date(struct date *);
 void ui_calendar_init_slctd_day(void);
 struct date *ui_calendar_get_slctd_day(void);
-long ui_calendar_get_slctd_day_sec(void);
+time_t ui_calendar_get_slctd_day_sec(void);
 void ui_calendar_monthly_view_cache_set_invalid(void);
 void ui_calendar_update_panel(void);
 void ui_calendar_goto_today(void);
@@ -748,6 +776,8 @@ int day_check_if_item(struct date);
 unsigned day_chk_busy_slices(struct date, int, int *);
 struct day_item *day_cut_item(long, int);
 int day_paste_item(struct day_item *, long);
+int day_get_position_by_aptev_ptr(union aptev_ptr);
+int day_get_position(struct day_item *);
 struct day_item *day_get_item(int);
 unsigned day_item_count(int);
 void day_edit_note(struct day_item *, const char *);
@@ -767,6 +797,8 @@ void event_llist_init(void);
 void event_llist_free(void);
 struct event *event_new(char *, char *, long, int);
 unsigned event_inday(struct event *, long *);
+char *event_tostr(struct event *);
+char *event_hash(struct event *);
 void event_write(struct event *, FILE *);
 struct event *event_scan(FILE *, struct tm, int, char *, struct item_filter *);
 void event_delete(struct event *);
@@ -779,10 +811,14 @@ int updatestring(WINDOW *, char **, int, int);
 /* help.c */
 int display_help(const char *);
 
+/* hooks.c */
+int run_hook(const char *);
+
 /* ical.c */
 void ical_import_data(FILE *, FILE *, unsigned *, unsigned *, unsigned *,
-		      unsigned *, unsigned *);
-void ical_export_data(FILE *);
+		      unsigned *, unsigned *, const char *, const char *,
+		      const char *, const char *, const char *);
+void ical_export_data(FILE *, int);
 
 /* io.c */
 unsigned io_fprintln(const char *, const char *, ...);
@@ -790,7 +826,9 @@ void io_init(const char *, const char *);
 void io_extract_data(char *, const char *, int);
 void io_save_mutex_lock(void);
 void io_save_mutex_unlock(void);
+void io_dump_apts(const char *, const char *, const char *, const char *);
 unsigned io_save_apts(const char *);
+void io_dump_todo(const char *);
 unsigned io_save_todo(const char *);
 unsigned io_save_keys(void);
 void io_save_cal(enum save_display);
@@ -805,8 +843,9 @@ unsigned io_file_exists(const char *);
 int io_check_file(const char *);
 int io_check_data_files(void);
 void io_startup_screen(int);
-void io_export_data(enum export_type);
-void io_import_data(enum import_type, const char *);
+void io_export_data(enum export_type, int);
+void io_import_data(enum import_type, const char *, const char *, const char *,
+		    const char *, const char *, const char *);
 struct io_file *io_log_init(void);
 void io_log_print(struct io_file *, int, const char *);
 void io_log_display(struct io_file *, const char *, const char *);
@@ -950,15 +989,17 @@ struct recur_apoint *recur_apoint_scan(FILE *, struct tm, struct tm,
 struct recur_event *recur_event_scan(FILE *, struct tm, int, char,
 				     int, struct tm, char *, llist_t *,
 				     struct item_filter *);
+char *recur_apoint_tostr(struct recur_apoint *);
+char *recur_apoint_hash(struct recur_apoint *);
 void recur_apoint_write(struct recur_apoint *, FILE *);
+char *recur_event_tostr(struct recur_event *);
+char *recur_event_hash(struct recur_event *);
 void recur_event_write(struct recur_event *, FILE *);
 void recur_save_data(FILE *);
 unsigned recur_item_find_occurrence(long, long, llist_t *, int,
-				    int, long, long, unsigned *);
-unsigned recur_apoint_find_occurrence(struct recur_apoint *, long,
-				      unsigned *);
-unsigned recur_event_find_occurrence(struct recur_event *, long,
-				     unsigned *);
+				    int, long, long, time_t *);
+unsigned recur_apoint_find_occurrence(struct recur_apoint *, long, time_t *);
+unsigned recur_event_find_occurrence(struct recur_event *, long, time_t *);
 unsigned recur_item_inday(long, long, llist_t *, int, int, long, long);
 unsigned recur_apoint_inday(struct recur_apoint *, long *);
 unsigned recur_event_inday(struct recur_event *, long *);
@@ -979,15 +1020,27 @@ unsigned sigs_set_hdlr(int, void (*)(int));
 void sigs_ignore(void);
 void sigs_unignore(void);
 
+/* strings.c */
+void string_init(struct string *);
+void string_reset(struct string *);
+int string_grow(struct string *, int);
+char *string_buf(struct string *);
+int string_catf(struct string *, const char *, ...);
+int string_vcatf(struct string *, const char *, va_list);
+int string_printf(struct string *, const char *, ...);
+
 /* todo.c */
 extern llist_t todolist;
-struct todo *todo_get_item(int);
-struct todo *todo_add(char *, int, char *);
+struct todo *todo_get_item(int, int);
+struct todo *todo_add(char *, int, int, char *);
+char *todo_tostr(struct todo *);
+char *todo_hash(struct todo *);
 void todo_write(struct todo *, FILE *);
 void todo_delete_note(struct todo *);
 void todo_delete(struct todo *);
+void todo_resort(struct todo *);
 void todo_flag(struct todo *);
-int todo_get_position(struct todo *);
+int todo_get_position(struct todo *, int);
 void todo_edit_note(struct todo *, const char *);
 void todo_view_note(struct todo *, const char *);
 void todo_free(struct todo *);
@@ -995,6 +1048,9 @@ void todo_init_list(void);
 void todo_free_list(void);
 
 /* ui-day.c */
+struct day_item *ui_day_selitem(void);
+void ui_day_set_selitem_by_aptev_ptr(union aptev_ptr);
+void ui_day_set_selitem(struct day_item *);
 void ui_day_item_add(void);
 void ui_day_item_delete(unsigned);
 void ui_day_item_edit(void);
@@ -1032,10 +1088,15 @@ void ui_todo_popup_item(void);
 void ui_todo_flag(void);
 void ui_todo_view_note(void);
 void ui_todo_edit_note(void);
+void ui_todo_view_prev(void);
+void ui_todo_view_next(void);
+int ui_todo_get_view(void);
+void ui_todo_set_view(int);
 
 /* utf8.c */
 int utf8_width(char *);
 int utf8_strwidth(char *);
+int utf8_chop(char *, int);
 
 /* utils.c */
 void exit_calcurse(int) __attribute__ ((__noreturn__));
@@ -1053,16 +1114,20 @@ int is_all_digit(const char *);
 long get_item_time(long);
 int get_item_hour(long);
 int get_item_min(long);
-long date2sec(struct date, unsigned, unsigned);
+struct tm date2tm(struct date, unsigned, unsigned);
+time_t date2sec(struct date, unsigned, unsigned);
+time_t utcdate2sec(struct date, unsigned, unsigned);
+int date_cmp_day(time_t, time_t);
 char *date_sec2date_str(long, const char *);
 void date_sec2date_fmt(long, const char *, char *);
+int date_change(struct tm *, int, int);
 long date_sec_change(long, int, int);
 long update_time_in_date(long, unsigned, unsigned);
-long get_sec_date(struct date);
+time_t get_sec_date(struct date);
 long min2sec(unsigned);
 void draw_scrollbar(WINDOW *, int, int, int, int, int, unsigned);
 void item_in_popup(const char *, const char *, const char *, const char *);
-long get_today(void);
+time_t get_today(void);
 long now(void);
 char *nowstr(void);
 void print_bool_option_incolor(WINDOW *, unsigned, int, int);
@@ -1074,6 +1139,8 @@ int parse_date(const char *, enum datefmt, int *, int *, int *,
 int check_time(unsigned, unsigned);
 int parse_time(const char *, unsigned *, unsigned *);
 int parse_duration(const char *, unsigned *);
+int parse_date_duration(const char *, unsigned *);
+int parse_datetime(const char *, long *);
 void file_close(FILE *, const char *);
 void psleep(unsigned);
 int fork_exec(int *, int *, const char *, const char *const *);
@@ -1082,12 +1149,15 @@ int child_wait(int *, int *, int);
 void press_any_key(void);
 void print_apoint(const char *, long, struct apoint *);
 void print_event(const char *, long, struct event *);
-void print_recur_apoint(const char *, long, unsigned,
-			struct recur_apoint *);
+void print_recur_apoint(const char *, long, time_t, struct recur_apoint *);
 void print_recur_event(const char *, long, struct recur_event *);
 void print_todo(const char *, struct todo *);
 int vasprintf(char **, const char *, va_list);
 int asprintf(char **, const char *, ...);
+int starts_with(const char *, const char *);
+int starts_with_ci(const char *, const char *);
+int hash_matches(const char *, const char *);
+int show_dialogs(void);
 
 /* vars.c */
 extern int col, row;
@@ -1096,6 +1166,7 @@ extern unsigned colorize;
 extern int foreground, background;
 extern enum ui_mode ui_mode;
 extern int read_only;
+extern int quiet;
 extern int want_reload;
 extern const char *datefmt_str[DATE_FORMATS];
 extern int days[12];
@@ -1110,6 +1181,7 @@ extern char path_notes[BUFSIZ];
 extern char path_cpid[BUFSIZ];
 extern char path_dpid[BUFSIZ];
 extern char path_dmon_log[BUFSIZ];
+extern char path_hooks[BUFSIZ];
 extern struct conf conf;
 extern struct pad apad;
 extern struct nbar nbar;
