@@ -1,7 +1,7 @@
 /*
  * Calcurse - text-based organizer
  *
- * Copyright (c) 2004-2015 calcurse Development Team <misc@calcurse.org>
+ * Copyright (c) 2004-2016 calcurse Development Team <misc@calcurse.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -59,6 +59,8 @@ static int config_parse_str(char *, const char *);
 static int config_serialize_str(char **, const char *);
 static int config_parse_calendar_view(void *, const char *);
 static int config_serialize_calendar_view(char **, void *);
+static int config_parse_todo_view(void *, const char *);
+static int config_serialize_todo_view(char **, void *);
 static int config_parse_default_panel(void *, const char *);
 static int config_serialize_default_panel(char **, void *);
 static int config_parse_first_day_of_week(void *, const char *);
@@ -73,6 +75,8 @@ static int config_parse_output_datefmt(void *, const char *);
 static int config_serialize_output_datefmt(char **, void *);
 static int config_parse_input_datefmt(void *, const char *);
 static int config_serialize_input_datefmt(char **, void *);
+static int config_parse_notifyall(void *, const char *);
+static int config_serialize_notifyall(char **, void *);
 
 #define CONFIG_HANDLER_BOOL(var) (config_fn_parse_t) config_parse_bool, \
   (config_fn_serialize_t) config_serialize_bool, &(var)
@@ -91,6 +95,7 @@ static const struct confvar confmap[] = {
 	{"appearance.notifybar", CONFIG_HANDLER_BOOL(nbar.show)},
 	{"appearance.sidebarwidth", config_parse_sidebar_width, config_serialize_sidebar_width, NULL},
 	{"appearance.theme", config_parse_color_theme, config_serialize_color_theme, NULL},
+	{"appearance.todoview", config_parse_todo_view, config_serialize_todo_view, NULL},
 	{"daemon.enable", CONFIG_HANDLER_BOOL(dmon.enable)},
 	{"daemon.log", CONFIG_HANDLER_BOOL(dmon.log)},
 	{"format.inputdate", config_parse_input_datefmt, config_serialize_input_datefmt, NULL},
@@ -106,7 +111,7 @@ static const struct confvar confmap[] = {
 	{"general.progressbar", CONFIG_HANDLER_BOOL(conf.progress_bar)},
 	{"general.systemdialogs", CONFIG_HANDLER_BOOL(conf.system_dialogs)},
 	{"notification.command", CONFIG_HANDLER_STR(nbar.cmd)},
-	{"notification.notifyall", CONFIG_HANDLER_BOOL(nbar.notify_all)},
+	{"notification.notifyall", config_parse_notifyall, config_serialize_notifyall, NULL},
 	{"notification.warning", CONFIG_HANDLER_INT(nbar.cntdwn)}
 };
 
@@ -206,6 +211,18 @@ static int config_parse_calendar_view(void *dummy, const char *val)
 	return 1;
 }
 
+static int config_parse_todo_view(void *dummy, const char *val)
+{
+	if (!strcmp(val, "show-completed"))
+		ui_todo_set_view(TODO_SHOW_COMPLETED_VIEW);
+	else if (!strcmp(val, "hide-completed"))
+		ui_todo_set_view(TODO_HIDE_COMPLETED_VIEW);
+	else
+		return 0;
+
+	return 1;
+}
+
 static int config_parse_default_panel(void *dummy, const char *val)
 {
 	if (!strcmp(val, "calendar"))
@@ -276,6 +293,19 @@ static int config_parse_input_datefmt(void *dummy, const char *val)
 	}
 }
 
+static int config_parse_notifyall(void *dummy, const char *val)
+{
+	if (strcmp(val, "flagged-only") == 0)
+		nbar.notify_all = NOTIFY_FLAGGED_ONLY;
+	else if (strcmp(val, "unflagged-only") == 0)
+		nbar.notify_all = NOTIFY_UNFLAGGED_ONLY;
+	else if (strcmp(val, "all") == 0)
+		nbar.notify_all = NOTIFY_ALL;
+	else
+		return config_parse_bool(&nbar.notify_all, val);
+	return 1;
+}
+
 /* Set a configuration variable. */
 static int config_set_conf(const char *key, const char *value)
 {
@@ -378,6 +408,16 @@ static int config_serialize_calendar_view(char **buf, void *dummy)
 	return 1;
 }
 
+static int config_serialize_todo_view(char **buf, void *dummy)
+{
+	if (ui_todo_get_view() == TODO_SHOW_COMPLETED_VIEW)
+		*buf = mem_strdup("show-completed");
+	else
+		*buf = mem_strdup("hide-completed");
+
+	return 1;
+}
+
 static int config_serialize_default_panel(char **buf, void *dummy)
 {
 	if (conf.default_panel == CAL)
@@ -428,6 +468,19 @@ static int config_serialize_input_datefmt(char **buf, void *dummy)
 	return config_serialize_int(buf, &conf.input_datefmt);
 }
 
+static int config_serialize_notifyall(char **buf, void *dummy)
+{
+	if (nbar.notify_all == NOTIFY_FLAGGED_ONLY)
+		*buf = mem_strdup("flagged-only");
+	else if (nbar.notify_all == NOTIFY_UNFLAGGED_ONLY)
+		*buf = mem_strdup("unflagged-only");
+	else if (nbar.notify_all == NOTIFY_ALL)
+		*buf = mem_strdup("all");
+	else
+		return 0;
+	return 1;
+}
+
 /* Serialize the value of a configuration variable. */
 static int
 config_serialize_conf(char **buf, const char *key,
