@@ -1,7 +1,7 @@
 /*
  * Calcurse - text-based organizer
  *
- * Copyright (c) 2004-2015 calcurse Development Team <misc@calcurse.org>
+ * Copyright (c) 2004-2016 calcurse Development Team <misc@calcurse.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -667,7 +667,7 @@ static enum listbox_row_type general_option_row_type(int i, void *cb_data)
 
 static int general_option_height(int i, void *cb_data)
 {
-	if (i == 9)
+	if (i == 11)
 		return 4;
 	else
 		return 3;
@@ -754,7 +754,7 @@ static void general_option_edit(int i)
 		break;
 	}
 
-	free(buf);
+	mem_free(buf);
 }
 
 /* General configuration. */
@@ -770,7 +770,7 @@ void custom_general_config(void)
 	listbox_init(&lb, 0, 0, notify_bar() ? row - 3 : row - 2, col,
 		     _("general options"), general_option_row_type,
 		     general_option_height, print_general_option);
-	listbox_load_items(&lb, 10);
+	listbox_load_items(&lb, NB_OPTIONS);
 	listbox_draw_deco(&lb, 0);
 	listbox_display(&lb);
 	wins_set_bindings(bindings, ARRAY_SIZE(bindings));
@@ -1032,6 +1032,26 @@ void custom_keys_config(void)
 			wins_scrollwin_delete(&kwin);
 			return;
 		}
+
+		if (resize) {
+			resize = 0;
+			wins_reset_noupdate();
+			nbdisplayed = ((notify_bar() ? row - 3 : row - 2) -
+					LABELLINES) / LINESPERKEY;
+			lastrow = firstrow + nbdisplayed - 1;
+			wins_scrollwin_resize(&kwin, 0, 0,
+					notify_bar() ? row - 3 : row - 2, col);
+			wins_scrollwin_draw_deco(&kwin, 0);
+			delwin(win[STA].p);
+			win[STA].p = newwin(win[STA].h, win[STA].w, win[STA].y,
+					win[STA].x);
+			keypad(win[STA].p, TRUE);
+			if (notify_bar()) {
+				notify_reinit_bar();
+				notify_update_bar();
+			}
+		}
+
 		custom_keys_config_bar();
 		werase(kwin.inner);
 		nbrowelm =
@@ -1104,6 +1124,11 @@ void custom_config_main(void)
 			break;
 		}
 
+		if (resize) {
+			resize = 0;
+			wins_reset();
+		}
+
 		wins_set_bindings(bindings, ARRAY_SIZE(bindings));
 		wins_update_border(FLAG_ALL);
 		wins_update_panels(FLAG_ALL);
