@@ -1,7 +1,7 @@
 /*
  * Calcurse - text-based organizer
  *
- * Copyright (c) 2004-2015 calcurse Development Team <misc@calcurse.org>
+ * Copyright (c) 2004-2016 calcurse Development Team <misc@calcurse.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -66,14 +66,30 @@ void day_free_vector(void)
 	VECTOR_FREE(&day_items);
 }
 
-static int day_cmp_start(struct day_item **pa, struct day_item **pb)
+static int day_cmp(struct day_item **pa, struct day_item **pb)
 {
 	struct day_item *a = *pa;
 	struct day_item *b = *pb;
+	int a_state, b_state;
 
 	if ((a->type == APPT || a->type == RECUR_APPT) &&
 	    (b->type == APPT || b->type == RECUR_APPT)) {
-		return a->start - b->start;
+		if (a->start < b->start)
+			return -1;
+		if (a->start > b->start)
+			return 1;
+
+		a_state = day_item_get_state(a);
+		b_state = day_item_get_state(b);
+		if ((a_state & APOINT_NOTIFY) && !(b_state & APOINT_NOTIFY))
+			return -1;
+		if (!(a_state & APOINT_NOTIFY) && (b_state & APOINT_NOTIFY))
+			return 1;
+
+		return strcmp(day_item_get_mesg(a), day_item_get_mesg(b));
+	} else if ((a->type == EVNT || a->type == RECUR_EVNT) &&
+		   (b->type == EVNT || b->type == RECUR_EVNT)) {
+		return strcmp(day_item_get_mesg(a), day_item_get_mesg(b));
 	}
 
 	return a->type - b->type;
@@ -308,7 +324,7 @@ static int day_store_recur_apoints(long date)
 
 		p.rapt = rapt;
 
-		unsigned real_start;
+		time_t real_start;
 		if (recur_apoint_find_occurrence(rapt, date, &real_start)) {
 			day_add_item(RECUR_APPT, real_start, p);
 			a_nb++;
@@ -347,7 +363,7 @@ day_store_items(long date, int include_captions)
 	if (include_captions && events > 0 && apts > 0)
 		day_add_item(DAY_SEPARATOR, 0, p);
 
-	VECTOR_SORT(&day_items, day_cmp_start);
+	VECTOR_SORT(&day_items, day_cmp);
 	day_items_nb = events + apts;
 }
 
@@ -358,7 +374,6 @@ day_store_items(long date, int include_captions)
  */
 void day_process_storage(struct date *slctd_date, unsigned day_changed)
 {
-	long date;
 	struct date day;
 
 	if (slctd_date)
@@ -366,14 +381,12 @@ void day_process_storage(struct date *slctd_date, unsigned day_changed)
 	else
 		ui_calendar_store_current_date(&day);
 
-	date = date2sec(day, 0, 0);
-
 	/* Inits */
 	if (apad.length != 0)
 		delwin(apad.ptrwin);
 
 	/* Store the events and appointments (recursive and normal items). */
-	day_store_items(date, 1);
+	day_store_items(date2sec(day, 0, 0), 1);
 }
 
 /*
@@ -414,7 +427,6 @@ day_display_item(struct day_item *day, WINDOW *win, int incolor, int width,
 {
 	int ch_recur, ch_note;
 	char buf[width * UTF8_MAXLEN];
-	int i;
 
 	if (width <= 0)
 		return;
@@ -424,23 +436,15 @@ day_display_item(struct day_item *day, WINDOW *win, int incolor, int width,
 	ch_recur = (day->type == RECUR_EVNT
 		    || day->type == RECUR_APPT) ? '*' : ' ';
 	ch_note = day_item_get_note(day) ? '>' : ' ';
-	if (incolor == 0)
+
+	strncpy(buf, mesg, width * UTF8_MAXLEN);
+	buf[sizeof(buf) - 1] = '\0';
+	utf8_chop(buf, width - 3);
+
+	if (!incolor)
 		custom_apply_attr(win, ATTR_HIGHEST);
-	if (utf8_strwidth(mesg) < width) {
-		mvwprintw(win, y, x, " %c%c%s", ch_recur, ch_note, mesg);
-	} else {
-		for (i = 0; mesg[i] && width > 0; i++) {
-			if (!UTF8_ISCONT(mesg[i]))
-				width -= utf8_width(&mesg[i]);
-			buf[i] = mesg[i];
-		}
-		if (i)
-			buf[i - 1] = 0;
-		else
-			buf[0] = 0;
-		mvwprintw(win, y, x, " %c%c%s...", ch_recur, ch_note, buf);
-	}
-	if (incolor == 0)
+	mvwprintw(win, y, x, " %c%c%s", ch_recur, ch_note, buf);
+	if (!incolor)
 		custom_remove_attr(win, ATTR_HIGHEST);
 }
 
@@ -503,35 +507,37 @@ void day_popup_item(struct day_item *day)
 }
 
 /*
- * Need to know if there is an item for the current selected day inside
- * calendar. This is used to put the correct colors inside calendar panel.
+ * Check whether there is an item on a given day.
+ *
+ * Returns 2 if the selected day contains a regular event or appointment.
+ * Returns 1 if the selected day does not contain a regular event or
+ * appointment but an occurrence of a recurrent item. Returns 0 otherwise.
  */
 int day_check_if_item(struct date day)
 {
-	const long date = date2sec(day, 0, 0);
+	const time_t t = date2sec(day, 0, 0);
 
-	if (LLIST_FIND_FIRST
-	    (&recur_elist, (long *)&date, recur_event_inday))
+	if (LLIST_FIND_FIRST(&eventlist, (time_t *)&t, event_inday))
+		return 2;
+
+	LLIST_TS_LOCK(&alist_p);
+	if (LLIST_TS_FIND_FIRST(&alist_p, (time_t *)&t, apoint_inday)) {
+		LLIST_TS_UNLOCK(&alist_p);
+		return 2;
+	}
+	LLIST_TS_UNLOCK(&alist_p);
+
+	if (LLIST_FIND_FIRST(&recur_elist, (time_t *)&t, recur_event_inday))
 		return 1;
 
 	LLIST_TS_LOCK(&recur_alist_p);
-	if (LLIST_TS_FIND_FIRST
-	    (&recur_alist_p, (long *)&date, recur_apoint_inday)) {
+	if (LLIST_TS_FIND_FIRST(&recur_alist_p, (time_t *)&t,
+				recur_apoint_inday)) {
 		LLIST_TS_UNLOCK(&recur_alist_p);
 		return 1;
 	}
 	LLIST_TS_UNLOCK(&recur_alist_p);
 
-	if (LLIST_FIND_FIRST(&eventlist, (long *)&date, event_inday))
-		return 1;
-
-	LLIST_TS_LOCK(&alist_p);
-	if (LLIST_TS_FIND_FIRST(&alist_p, (long *)&date, apoint_inday)) {
-		LLIST_TS_UNLOCK(&alist_p);
-		return 1;
-	}
-	LLIST_TS_UNLOCK(&alist_p);
-
 	return 0;
 }
 
@@ -558,28 +564,43 @@ static unsigned fill_slices(int *slices, int slicesno, int first, int last)
  */
 unsigned day_chk_busy_slices(struct date day, int slicesno, int *slices)
 {
+	const time_t t = date2sec(day, 0, 0);
 	llist_item_t *i;
 	int slicelen;
-	const long date = date2sec(day, 0, 0);
 
 	slicelen = DAYINSEC / slicesno;
 
 #define  SLICENUM(tsec)  ((tsec) / slicelen % slicesno)
 
 	LLIST_TS_LOCK(&recur_alist_p);
-	LLIST_TS_FIND_FOREACH(&recur_alist_p, (long *)&date,
+	LLIST_TS_FIND_FOREACH(&recur_alist_p, (time_t *)&t,
 			      recur_apoint_inday, i) {
-		struct apoint *rapt = LLIST_TS_GET_DATA(i);
-		long start = get_item_time(rapt->start);
-		long end = get_item_time(rapt->start + rapt->dur);
+		struct recur_apoint *rapt = LLIST_TS_GET_DATA(i);
+		time_t occurrence;
+		long start, end;
 
-		if (rapt->start < date)
+		if (!recur_apoint_find_occurrence(rapt, t, &occurrence))
+			continue;
+
+		if (occurrence >= t)
+			start = get_item_time(occurrence);
+		else
 			start = 0;
-		if (rapt->start + rapt->dur >= date + DAYINSEC)
+
+		if (occurrence + rapt->dur < t + DAYINSEC)
+			end = get_item_time(occurrence + rapt->dur);
+		else
 			end = DAYINSEC - 1;
 
-		if (!fill_slices
-		    (slices, slicesno, SLICENUM(start), SLICENUM(end))) {
+		/*
+		 * If an item ends on 12:00, we do not want the 12:00 slot to
+		 * be marked busy.
+		 */
+		if (end > start)
+			end--;
+
+		if (!fill_slices(slices, slicesno, SLICENUM(start),
+					SLICENUM(end))) {
 			LLIST_TS_UNLOCK(&recur_alist_p);
 			return 0;
 		}
@@ -587,20 +608,27 @@ unsigned day_chk_busy_slices(struct date day, int slicesno, int *slices)
 	LLIST_TS_UNLOCK(&recur_alist_p);
 
 	LLIST_TS_LOCK(&alist_p);
-	LLIST_TS_FIND_FOREACH(&alist_p, (long *)&date, apoint_inday, i) {
+	LLIST_TS_FIND_FOREACH(&alist_p, (time_t *)&t, apoint_inday, i) {
 		struct apoint *apt = LLIST_TS_GET_DATA(i);
 		long start = get_item_time(apt->start);
 		long end = get_item_time(apt->start + apt->dur);
 
-		if (apt->start >= date + DAYINSEC)
+		if (apt->start >= t + DAYINSEC)
 			break;
-		if (apt->start < date)
+		if (apt->start < t)
 			start = 0;
-		if (apt->start + apt->dur >= date + DAYINSEC)
+		if (apt->start + apt->dur >= t + DAYINSEC)
 			end = DAYINSEC - 1;
 
-		if (!fill_slices
-		    (slices, slicesno, SLICENUM(start), SLICENUM(end))) {
+		/*
+		 * If an item ends on 12:00, we do not want the 12:00 slot to
+		 * be marked busy.
+		 */
+		if (end > start)
+			end--;
+
+		if (!fill_slices(slices, slicesno, SLICENUM(start),
+					SLICENUM(end))) {
 			LLIST_TS_UNLOCK(&alist_p);
 			return 0;
 		}
@@ -666,6 +694,26 @@ int day_paste_item(struct day_item *p, long date)
 	return p->type;
 }
 
+/* Returns the position corresponding to a given item. */
+int day_get_position_by_aptev_ptr(union aptev_ptr aptevp)
+{
+	int n = 0;
+
+	VECTOR_FOREACH(&day_items, n) {
+		struct day_item *p = VECTOR_NTH(&day_items, n);
+		/* Compare pointers. */
+		if (p->item.ev == aptevp.ev)
+			return n;
+	}
+
+	return -1;
+}
+
+int day_get_position(struct day_item *needle)
+{
+	return day_get_position_by_aptev_ptr(needle->item);
+}
+
 /* Returns a structure containing the selected item. */
 struct day_item *day_get_item(int item_number)
 {
