@@ -1,7 +1,7 @@
 /*
  * Calcurse - text-based organizer
  *
- * Copyright (c) 2004-2015 calcurse Development Team <misc@calcurse.org>
+ * Copyright (c) 2004-2016 calcurse Development Team <misc@calcurse.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -40,6 +40,7 @@
 #include <time.h>
 
 #include "calcurse.h"
+#include "sha1.h"
 
 llist_t eventlist;
 
@@ -77,9 +78,14 @@ void event_llist_free(void)
 	LLIST_FREE(&eventlist);
 }
 
-static int event_cmp_day(struct event *a, struct event *b)
+static int event_cmp(struct event *a, struct event *b)
 {
-	return a->day < b->day ? -1 : (a->day == b->day ? 0 : 1);
+	if (a->day < b->day)
+		return -1;
+	if (a->day > b->day)
+		return 1;
+
+	return strcmp(a->mesg, b->mesg);
 }
 
 /* Create a new event */
@@ -93,7 +99,7 @@ struct event *event_new(char *mesg, char *note, long day, int id)
 	ev->id = id;
 	ev->note = (note != NULL) ? mem_strdup(note) : NULL;
 
-	LLIST_ADD_SORTED(&eventlist, ev, event_cmp_day);
+	LLIST_ADD_SORTED(&eventlist, ev, event_cmp);
 
 	return ev;
 }
@@ -101,22 +107,43 @@ struct event *event_new(char *mesg, char *note, long day, int id)
 /* Check if the event belongs to the selected day */
 unsigned event_inday(struct event *i, long *start)
 {
-	return (i->day < *start + DAYINSEC && i->day >= *start);
+	return (date_cmp_day(i->day, *start) == 0);
 }
 
-/* Write to file the event in user-friendly format */
-void event_write(struct event *o, FILE * f)
+char *event_tostr(struct event *o)
 {
+	struct string s;
 	struct tm lt;
 	time_t t;
 
+	string_init(&s);
+
 	t = o->day;
 	localtime_r(&t, &lt);
-	fprintf(f, "%02u/%02u/%04u [%d] ", lt.tm_mon + 1, lt.tm_mday,
+	string_catf(&s, "%02u/%02u/%04u [%d] ", lt.tm_mon + 1, lt.tm_mday,
 		1900 + lt.tm_year, o->id);
 	if (o->note != NULL)
-		fprintf(f, ">%s ", o->note);
-	fprintf(f, "%s\n", o->mesg);
+		string_catf(&s, ">%s ", o->note);
+	string_catf(&s, "%s", o->mesg);
+
+	return string_buf(&s);
+}
+
+char *event_hash(struct event *ev)
+{
+	char *raw = event_tostr(ev);
+	char *sha1 = mem_malloc(SHA1_DIGESTLEN * 2 + 1);
+	sha1_digest(raw, sha1);
+	mem_free(raw);
+
+	return sha1;
+}
+
+void event_write(struct event *o, FILE * f)
+{
+	char *str = event_tostr(o);
+	fprintf(f, "%s\n", str);
+	mem_free(str);
 }
 
 /* Load the events from file */
@@ -124,7 +151,8 @@ struct event *event_scan(FILE * f, struct tm start, int id, char *note,
 			 struct item_filter *filter)
 {
 	char buf[BUFSIZ], *nl;
-	time_t tstart;
+	time_t tstart, tend;
+	struct event *ev;
 
 	EXIT_IF(!check_date(start.tm_year, start.tm_mon, start.tm_mday) ||
 		!check_time(start.tm_hour, start.tm_min),
@@ -147,6 +175,7 @@ struct event *event_scan(FILE * f, struct tm start, int id, char *note,
 
 	tstart = mktime(&start);
 	EXIT_IF(tstart == -1, _("date error in the event\n"));
+	tend = tstart + DAYINSEC - 1;
 
 	/* Filter item. */
 	if (filter) {
@@ -158,9 +187,25 @@ struct event *event_scan(FILE * f, struct tm start, int id, char *note,
 			return NULL;
 		if (filter->start_to >= 0 && tstart > filter->start_to)
 			return NULL;
+		if (filter->end_from >= 0 && tend < filter->end_from)
+			return NULL;
+		if (filter->end_to >= 0 && tend > filter->end_to)
+			return NULL;
 	}
 
-	return event_new(buf, note, tstart, id);
+	ev = event_new(buf, note, tstart, id);
+
+	/* Filter by hash. */
+	if (filter && filter->hash) {
+		char *hash = event_hash(ev);
+		if (!hash_matches(filter->hash, hash)) {
+			event_delete(ev);
+			ev = NULL;
+		}
+		mem_free(hash);
+	}
+
+	return ev;
 }
 
 /* Delete an event from the list. */
@@ -177,5 +222,5 @@ void event_delete(struct event *ev)
 void event_paste_item(struct event *ev, long date)
 {
 	ev->day = date;
-	LLIST_ADD_SORTED(&eventlist, ev, event_cmp_day);
+	LLIST_ADD_SORTED(&eventlist, ev, event_cmp);
 }
