@@ -1,7 +1,7 @@
 /*
  * Calcurse - text-based organizer
  *
- * Copyright (c) 2004-2015 calcurse Development Team <misc@calcurse.org>
+ * Copyright (c) 2004-2016 calcurse Development Team <misc@calcurse.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -45,7 +45,7 @@ static int find_basedir(const char *locale_info[], unsigned n, char **basedir)
 	for (i = 0; i < n; i++) {
 		if (!locale_info[i])
 			continue;
-		locale = strdup(locale_info[i]);
+		locale = mem_strdup(locale_info[i]);
 
 		asprintf(basedir, "%s/%s", DOCDIR, locale);
 		if (io_dir_exists(*basedir)) {
@@ -79,7 +79,7 @@ static int find_basedir(const char *locale_info[], unsigned n, char **basedir)
 
 cleanup:
 	if (locale)
-		free(locale);
+		mem_free(locale);
 	return ret;
 }
 
