@@ -1,7 +1,7 @@
 /*
  * Calcurse - text-based organizer
  *
- * Copyright (c) 2004-2015 calcurse Development Team <misc@calcurse.org>
+ * Copyright (c) 2004-2016 calcurse Development Team <misc@calcurse.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -62,11 +62,11 @@ typedef struct {
 } ical_rpt_t;
 
 static void ical_export_header(FILE *);
-static void ical_export_recur_events(FILE *);
-static void ical_export_events(FILE *);
-static void ical_export_recur_apoints(FILE *);
-static void ical_export_apoints(FILE *);
-static void ical_export_todo(FILE *);
+static void ical_export_recur_events(FILE *, int);
+static void ical_export_events(FILE *, int);
+static void ical_export_recur_apoints(FILE *, int);
+static void ical_export_apoints(FILE *, int);
+static void ical_export_todo(FILE *, int);
 static void ical_export_footer(FILE *);
 
 static const char *ical_recur_type[RECUR_TYPES] =
@@ -98,7 +98,7 @@ static void ical_export_footer(FILE * stream)
 }
 
 /* Export recurrent events. */
-static void ical_export_recur_events(FILE * stream)
+static void ical_export_recur_events(FILE * stream, int export_uid)
 {
 	llist_item_t *i, *j;
 	char ical_date[BUFSIZ];
@@ -126,20 +126,24 @@ static void ical_export_recur_events(FILE * stream)
 				date_sec2date_fmt(exc->st, ICALDATEFMT,
 						  ical_date);
 				fprintf(stream, "%s", ical_date);
-				if (LLIST_NEXT(j))
-					fputc(',', stream);
-				else
-					fputc('\n', stream);
+				fputc(LLIST_NEXT(j) ? ',' : '\n', stream);
 			}
 		}
 
 		fprintf(stream, "SUMMARY:%s\n", rev->mesg);
+
+		if (export_uid) {
+			char *hash = recur_event_hash(rev);
+			fprintf(stream, "UID:%s\n", hash);
+			mem_free(hash);
+		}
+
 		fputs("END:VEVENT\n", stream);
 	}
 }
 
 /* Export events. */
-static void ical_export_events(FILE * stream)
+static void ical_export_events(FILE * stream, int export_uid)
 {
 	llist_item_t *i;
 	char ical_date[BUFSIZ];
@@ -150,12 +154,19 @@ static void ical_export_events(FILE * stream)
 		fputs("BEGIN:VEVENT\n", stream);
 		fprintf(stream, "DTSTART:%s\n", ical_date);
 		fprintf(stream, "SUMMARY:%s\n", ev->mesg);
+
+		if (export_uid) {
+			char *hash = event_hash(ev);
+			fprintf(stream, "UID:%s\n", hash);
+			mem_free(hash);
+		}
+
 		fputs("END:VEVENT\n", stream);
 	}
 }
 
 /* Export recurrent appointments. */
-static void ical_export_recur_apoints(FILE * stream)
+static void ical_export_recur_apoints(FILE * stream, int export_uid)
 {
 	llist_item_t *i, *j;
 	char ical_datetime[BUFSIZ];
@@ -169,7 +180,13 @@ static void ical_export_recur_apoints(FILE * stream)
 				  ical_datetime);
 		fputs("BEGIN:VEVENT\n", stream);
 		fprintf(stream, "DTSTART:%s\n", ical_datetime);
-		fprintf(stream, "DURATION:PT0H0M%ldS\n", rapt->dur);
+		if (rapt->dur > 0) {
+			fprintf(stream, "DURATION:P%ldDT%ldH%ldM%ldS\n",
+				rapt->dur / DAYINSEC,
+				(rapt->dur / HOURINSEC) % DAYINHOURS,
+				(rapt->dur / MININSEC) % HOURINMIN,
+				rapt->dur % MININSEC);
+		}
 		fprintf(stream, "RRULE:FREQ=%s;INTERVAL=%d",
 			ical_recur_type[rapt->rpt->type], rapt->rpt->freq);
 
@@ -188,23 +205,27 @@ static void ical_export_recur_apoints(FILE * stream)
 				date_sec2date_fmt(exc->st, ICALDATEFMT,
 						  ical_date);
 				fprintf(stream, "%s", ical_date);
-				if (LLIST_NEXT(j))
-					fputc(',', stream);
-				else
-					fputc('\n', stream);
+				fputc(LLIST_NEXT(j) ? ',' : '\n', stream);
 			}
 		}
 
 		fprintf(stream, "SUMMARY:%s\n", rapt->mesg);
 		if (rapt->state & APOINT_NOTIFY)
 			ical_export_valarm(stream);
+
+		if (export_uid) {
+			char *hash = recur_apoint_hash(rapt);
+			fprintf(stream, "UID:%s\n", hash);
+			mem_free(hash);
+		}
+
 		fputs("END:VEVENT\n", stream);
 	}
 	LLIST_TS_UNLOCK(&recur_alist_p);
 }
 
 /* Export appointments. */
-static void ical_export_apoints(FILE * stream)
+static void ical_export_apoints(FILE * stream, int export_uid)
 {
 	llist_item_t *i;
 	char ical_datetime[BUFSIZ];
@@ -216,32 +237,48 @@ static void ical_export_apoints(FILE * stream)
 				  ical_datetime);
 		fputs("BEGIN:VEVENT\n", stream);
 		fprintf(stream, "DTSTART:%s\n", ical_datetime);
-		fprintf(stream, "DURATION:P%ldDT%ldH%ldM%ldS\n",
-			apt->dur / DAYINSEC,
-			(apt->dur / HOURINSEC) % DAYINHOURS,
-			(apt->dur / MININSEC) % HOURINMIN,
-			apt->dur % MININSEC);
+		if (apt->dur > 0) {
+			fprintf(stream, "DURATION:P%ldDT%ldH%ldM%ldS\n",
+				apt->dur / DAYINSEC,
+				(apt->dur / HOURINSEC) % DAYINHOURS,
+				(apt->dur / MININSEC) % HOURINMIN,
+				apt->dur % MININSEC);
+		}
 		fprintf(stream, "SUMMARY:%s\n", apt->mesg);
 		if (apt->state & APOINT_NOTIFY)
 			ical_export_valarm(stream);
+
+		if (export_uid) {
+			char *hash = apoint_hash(apt);
+			fprintf(stream, "UID:%s\n", hash);
+			mem_free(hash);
+		}
+
 		fputs("END:VEVENT\n", stream);
 	}
 	LLIST_TS_UNLOCK(&alist_p);
 }
 
 /* Export todo items. */
-static void ical_export_todo(FILE * stream)
+static void ical_export_todo(FILE * stream, int export_uid)
 {
 	llist_item_t *i;
 
 	LLIST_FOREACH(&todolist, i) {
 		struct todo *todo = LLIST_TS_GET_DATA(i);
-		if (todo->id < 0)	/* completed items */
-			continue;
 
 		fputs("BEGIN:VTODO\n", stream);
+		if (todo->completed)
+			fprintf(stream, "STATUS:COMPLETED\n");
 		fprintf(stream, "PRIORITY:%d\n", todo->id);
 		fprintf(stream, "SUMMARY:%s\n", todo->mesg);
+
+		if (export_uid) {
+			char *hash = todo_hash(todo);
+			fprintf(stream, "UID:%s\n", hash);
+			mem_free(hash);
+		}
+
 		fputs("END:VTODO\n", stream);
 	}
 }
@@ -281,32 +318,44 @@ static void ical_log(FILE * log, ical_types_e type, unsigned lineno,
 	const char *typestr[ICAL_TYPES] = { "VEVENT", "VTODO" };
 
 	RETURN_IF(type < 0 || type >= ICAL_TYPES, _("unknown ical type"));
-	if (log)
-		fprintf(log, "%s [%d]: %s\n", typestr[type], lineno, msg);
+	if (!log)
+		return;
+
+	fprintf(log, "%s [%d]: %s\n", typestr[type], lineno, msg);
 }
 
-static void ical_store_todo(int priority, char *mesg, char *note)
+static void ical_store_todo(int priority, int completed, char *mesg,
+			    char *note, const char *fmt_todo)
 {
-	todo_add(mesg, priority, note);
+	struct todo *todo = todo_add(mesg, priority, completed, note);
+	if (fmt_todo)
+		print_todo(fmt_todo, todo);
 	mem_free(mesg);
 	erase_note(&note);
 }
 
 static void
 ical_store_event(char *mesg, char *note, long day, long end,
-		 ical_rpt_t * rpt, llist_t * exc)
+		 ical_rpt_t * rpt, llist_t * exc, const char *fmt_ev,
+		 const char *fmt_rev)
 {
 	const int EVENTID = 1;
+	struct event *ev;
+	struct recur_event *rev;
 
 	if (rpt) {
-		recur_event_new(mesg, note, day, EVENTID, rpt->type,
-				rpt->freq, rpt->until, exc);
+		rev = recur_event_new(mesg, note, day, EVENTID, rpt->type,
+				      rpt->freq, rpt->until, exc);
 		mem_free(rpt);
+		if (fmt_rev)
+			print_recur_event(fmt_rev, day, rev);
 		goto cleanup;
 	}
 
 	if (end == 0 || end - day <= DAYINSEC) {
-		event_new(mesg, note, day, EVENTID);
+		ev = event_new(mesg, note, day, EVENTID);
+		if (fmt_ev)
+			print_event(fmt_ev, day, ev);
 		goto cleanup;
 	}
 
@@ -323,9 +372,11 @@ ical_store_event(char *mesg, char *note, long day, long end,
 	rpt->freq = 1;
 	rpt->count = 0;
 	rpt->until = end;
-	recur_event_new(mesg, note, day, EVENTID, rpt->type,
-			rpt->freq, rpt->until, exc);
+	rev = recur_event_new(mesg, note, day, EVENTID, rpt->type,
+			      rpt->freq, rpt->until, exc);
 	mem_free(rpt);
+	if (fmt_rev)
+		print_recur_event(fmt_rev, day, rev);
 
 cleanup:
 	mem_free(mesg);
@@ -334,18 +385,25 @@ cleanup:
 
 static void
 ical_store_apoint(char *mesg, char *note, long start, long dur,
-		  ical_rpt_t * rpt, llist_t * exc, int has_alarm)
+		  ical_rpt_t * rpt, llist_t * exc, int has_alarm,
+		  const char *fmt_apt, const char *fmt_rapt)
 {
 	char state = 0L;
+	struct apoint *apt;
+	struct recur_apoint *rapt;
 
 	if (has_alarm)
 		state |= APOINT_NOTIFY;
 	if (rpt) {
-		recur_apoint_new(mesg, note, start, dur, state, rpt->type,
-				 rpt->freq, rpt->until, exc);
+		rapt = recur_apoint_new(mesg, note, start, dur, state,
+					rpt->type, rpt->freq, rpt->until, exc);
 		mem_free(rpt);
+		if (fmt_rapt)
+			print_recur_apoint(fmt_rapt, start, rapt->start, rapt);
 	} else {
-		apoint_new(mesg, note, start, dur, state);
+		apt = apoint_new(mesg, note, start, dur, state);
+		if (fmt_apt)
+			print_apoint(fmt_apt, start, apt);
 	}
 	mem_free(mesg);
 	erase_note(&note);
@@ -354,55 +412,43 @@ ical_store_apoint(char *mesg, char *note, long start, long dur,
 /*
  * Returns an allocated string representing the string given in argument once
  * unformatted.
- *
- * Note:
- * Even if the RFC2445 recommends not to have more than 75 octets on one line of
- * text, I prefer not to restrict the parsing to this size, thus I use a buffer
- * of size BUFSIZ.
- *
- * Extract from RFC2445:
- * Lines of text SHOULD NOT be longer than 75 octets, excluding the line
- * break.
  */
 static char *ical_unformat_line(char *line)
 {
-	char *p, uline[BUFSIZ];
-	int len;
+	struct string s;
+	char *p;
 
-	if (strlen(line) >= BUFSIZ)
-		return NULL;
-
-	memset(uline, 0, BUFSIZ);
-	for (len = 0, p = line; *p; p++) {
+	string_init(&s);
+	for (p = line; *p; p++) {
 		switch (*p) {
 		case '\\':
 			switch (*(p + 1)) {
 			case 'n':
-				uline[len++] = '\n';
+				string_catf(&s, "%c", '\n');
 				p++;
 				break;
 			case 't':
-				uline[len++] = '\t';
+				string_catf(&s, "%c", '\t');
 				p++;
 				break;
 			case ';':
 			case ':':
 			case ',':
-				uline[len++] = *(p + 1);
+				string_catf(&s, "%c", *(p + 1));
 				p++;
 				break;
 			default:
-				uline[len++] = *p;
+				string_catf(&s, "%c", *p);
 				break;
 			}
 			break;
 		default:
-			uline[len++] = *p;
+			string_catf(&s, "%c", *p);
 			break;
 		}
 	}
 
-	return mem_strdup(uline);
+	return string_buf(&s);
 }
 
 static void
@@ -455,12 +501,10 @@ static int
 ical_chk_header(FILE * fd, char *buf, char *lstore, unsigned *lineno,
 		int *major, int *minor)
 {
-	const char icalheader[] = "BEGIN:VCALENDAR";
-
 	if (!ical_readline(fd, buf, lstore, lineno))
 		return 0;
 
-	if (strncasecmp(buf, icalheader, sizeof(icalheader) - 1) != 0)
+	if (!starts_with_ci(buf, "BEGIN:VCALENDAR"))
 		return 0;
 
 	while (!sscanf(buf, "VERSION:%d.%d", major, minor)) {
@@ -477,46 +521,47 @@ ical_chk_header(FILE * fd, char *buf, char *lstore, unsigned *lineno,
  * where DATE is 'YYYYMMDD' and TIME is 'HHMMSS'.
  * The time and 'T' separator are optional (in the case of an day-long event).
  *
- * Optionnaly, if the type pointer is given, specify if it is an event
+ * Optionally, if the type pointer is given, specify if it is an event
  * (no time is given, meaning it is an all-day event), or an appointment
  * (time is given).
  *
  * The timezone is not yet handled by calcurse.
  */
-static long ical_datetime2long(char *datestr, ical_vevent_e * type)
+static time_t ical_datetime2time_t(char *datestr, ical_vevent_e * type)
 {
-	const int NOTFOUND = 0, FORMAT_DATE = 3, FORMAT_DATETIME = 5;
+	const int FORMAT_DATE = 3, FORMAT_DATETIME = 6, FORMAT_DATETIMEZ = 7;
 	struct date date;
-	unsigned hour, min;
-	long datelong;
+	unsigned hour, min, sec;
+	char c;
 	int format;
 
-	format = sscanf(datestr, "%04u%02u%02uT%02u%02u",
-			&date.yyyy, &date.mm, &date.dd, &hour, &min);
+	format = sscanf(datestr, "%04u%02u%02uT%02u%02u%02u%c",
+			&date.yyyy, &date.mm, &date.dd, &hour, &min, &sec, &c);
 	if (format == FORMAT_DATE) {
 		if (type)
 			*type = EVENT;
-		datelong = date2sec(date, 0, 0);
-	} else if (format == FORMAT_DATETIME) {
+		return date2sec(date, 0, 0);
+	} else if (format == FORMAT_DATETIME || format == FORMAT_DATETIMEZ) {
 		if (type)
 			*type = APPOINTMENT;
-		datelong = date2sec(date, hour, min);
-	} else {
-		datelong = NOTFOUND;
+		if (format == FORMAT_DATETIMEZ && c == 'Z')
+			return utcdate2sec(date, hour, min);
+		else
+			return date2sec(date, hour, min);
 	}
-	return datelong;
+	return 0;
 }
 
 static long ical_durtime2long(char *timestr)
 {
-	char *p;
+	char *p = timestr;
 	int bytes_read;
 	unsigned hour = 0, min = 0, sec = 0;
 
-	if ((p = strchr(timestr, 'T')) == NULL)
+	if (*p != 'T')
 		return 0;
-
 	p++;
+
 	if (strchr(p, 'H')) {
 		if (sscanf(p, "%uH%n", &hour, &bytes_read) != 1)
 			return 0;
@@ -528,14 +573,11 @@ static long ical_durtime2long(char *timestr)
 		p += bytes_read;
 	}
 	if (strchr(p, 'S')) {
-		if (sscanf(p, "%uM%n", &sec, &bytes_read) != 1)
+		if (sscanf(p, "%uS%n", &sec, &bytes_read) != 1)
 			return 0;
 		p += bytes_read;
 	}
 
-	if (hour == 0 && min == 0 && sec == 0)
-		return 0;
-
 	return hour * HOURINSEC + min * MININSEC + sec;
 }
 
@@ -566,45 +608,40 @@ static long ical_durtime2long(char *timestr)
  */
 static long ical_dur2long(char *durstr)
 {
-	const int NOTFOUND = -1;
-	long durlong;
 	char *p;
+	int bytes_read;
 	struct {
 		unsigned week, day;
 	} date;
 
 	memset(&date, 0, sizeof date);
-	if ((p = strchr(durstr, 'P')) == NULL) {
-		durlong = NOTFOUND;
-	} else {
-		p++;
-		if (*p == '-')
-			return NOTFOUND;
-		else if (*p == '+')
-			p++;
 
-		if (*p == 'T') {	/* dur-time */
-			durlong = ical_durtime2long(p);
-		} else if (strchr(p, 'W')) {	/* dur-week */
-			if (sscanf(p, "%u", &date.week) == 1)
-				durlong =
-				    date.week * WEEKINDAYS * DAYINSEC;
-			else
-				durlong = NOTFOUND;
-		} else {
-			if (strchr(p, 'D')) {	/* dur-date */
-				if (sscanf(p, "%uD", &date.day) == 1) {
-					durlong = date.day * DAYINSEC;
-					durlong += ical_durtime2long(p);
-				} else {
-					durlong = NOTFOUND;
-				}
-			} else {
-				durlong = NOTFOUND;
-			}
+	p = strchr(durstr, 'P');
+	if (!p)
+		return -1;
+	p++;
+
+	if (*p == '-')
+		return -1;
+	if (*p == '+')
+		p++;
+
+	if (*p == 'T') {
+		/* dur-time */
+		return ical_durtime2long(p);
+	} else if (strchr(p, 'W')) {
+		/* dur-week */
+		if (sscanf(p, "%u", &date.week) == 1)
+			return date.week * WEEKINDAYS * DAYINSEC;
+	} else if (strchr(p, 'D')) {
+		/* dur-date */
+		if (sscanf(p, "%uD%n", &date.day, &bytes_read) == 1) {
+			p += bytes_read;
+			return date.day * DAYINSEC + ical_durtime2long(p);
 		}
 	}
-	return durlong;
+
+	return -1;
 }
 
 /*
@@ -617,35 +654,20 @@ static long ical_dur2long(char *durstr)
  */
 static long ical_compute_rpt_until(long start, ical_rpt_t * rpt)
 {
-	long until;
-
 	switch (rpt->type) {
 	case RECUR_DAILY:
-		until =
-		    date_sec_change(start, 0,
-				    rpt->freq * (rpt->count - 1));
-		break;
+		return date_sec_change(start, 0, rpt->freq * (rpt->count - 1));
 	case RECUR_WEEKLY:
-		until = date_sec_change(start, 0,
-					rpt->freq * WEEKINDAYS *
-					(rpt->count - 1));
-		break;
+		return date_sec_change(start, 0,
+				rpt->freq * WEEKINDAYS * (rpt->count - 1));
 	case RECUR_MONTHLY:
-		until =
-		    date_sec_change(start, rpt->freq * (rpt->count - 1),
-				    0);
-		break;
+		return date_sec_change(start, rpt->freq * (rpt->count - 1), 0);
 	case RECUR_YEARLY:
-		until =
-		    date_sec_change(start,
+		return date_sec_change(start,
 				    rpt->freq * 12 * (rpt->count - 1), 0);
-		break;
 	default:
-		until = 0;
-		break;
-		/* NOTREACHED */
+		return 0;
 	}
-	return until;
 }
 
 /*
@@ -688,115 +710,91 @@ static long ical_compute_rpt_until(long start, ical_rpt_t * rpt)
 static ical_rpt_t *ical_read_rrule(FILE * log, char *rrulestr,
 				   unsigned *noskipped, const int itemline)
 {
-	const char daily[] = "DAILY";
-	const char weekly[] = "WEEKLY";
-	const char monthly[] = "MONTHLY";
-	const char yearly[] = "YEARLY";
 	const char count[] = "COUNT=";
 	const char interv[] = "INTERVAL=";
+	char freqstr[BUFSIZ];
 	unsigned interval;
 	ical_rpt_t *rpt;
 	char *p;
 
-	rpt = NULL;
-	if ((p = strchr(rrulestr, ':')) != NULL) {
-		char freqstr[BUFSIZ];
-
-		p++;
-		rpt = mem_malloc(sizeof(ical_rpt_t));
-		memset(rpt, 0, sizeof(ical_rpt_t));
-		if (sscanf(p, "FREQ=%s", freqstr) != 1) {
-			ical_log(log, ICAL_VEVENT, itemline,
-				 _("recurrence frequence not found."));
-			(*noskipped)++;
-			mem_free(rpt);
-			return NULL;
-		} else {
-			if (strncmp(freqstr, daily, sizeof(daily) - 1) ==
-			    0) {
-				rpt->type = RECUR_DAILY;
-			} else
-			    if (strncmp
-				(freqstr, weekly,
-				 sizeof(weekly) - 1) == 0) {
-				rpt->type = RECUR_WEEKLY;
-			} else
-			    if (strncmp
-				(freqstr, monthly,
-				 sizeof(monthly) - 1) == 0) {
-				rpt->type = RECUR_MONTHLY;
-			} else
-			    if (strncmp
-				(freqstr, yearly,
-				 sizeof(yearly) - 1) == 0) {
-				rpt->type = RECUR_YEARLY;
-			} else {
-				ical_log(log, ICAL_VEVENT, itemline,
-					 _("recurrence frequence not recognized."));
-				(*noskipped)++;
-				mem_free(rpt);
-				return NULL;
-			}
-		}
-		/*
-		   The UNTIL rule part defines a date-time value which bounds the
-		   recurrence rule in an inclusive manner.  If not present, and the
-		   COUNT rule part is also not present, the RRULE is considered to
-		   repeat forever.
-
-		   The COUNT rule part defines the number of occurrences at which to
-		   range-bound the recurrence.  The "DTSTART" property value, if
-		   specified, counts as the first occurrence.
-		 */
-		if ((p = strstr(rrulestr, "UNTIL")) != NULL) {
-			char *untilstr;
-
-			untilstr = strchr(p, '=');
-			rpt->until = ical_datetime2long(++untilstr, NULL);
-		} else {
-			unsigned cnt;
-			char *countstr;
-
-			if ((countstr = strstr(rrulestr, count)) != NULL) {
-				countstr += sizeof(count) - 1;
-				if (sscanf(countstr, "%u", &cnt) != 1) {
-					rpt->until = 0;
-					/* endless repetition */
-				} else {
-					rpt->count = cnt;
-				}
-			} else {
-				rpt->until = 0;
-			}
-		}
-
-		if ((p = strstr(rrulestr, interv)) != NULL) {
-			p += sizeof(interv) - 1;
-			if (sscanf(p, "%u", &interval) != 1) {
-				rpt->freq = 1;
-				/* default frequence if none specified */
-			} else {
-				rpt->freq = interval;
-			}
-		} else {
-			rpt->freq = 1;
-		}
-	} else {
+	p = strchr(rrulestr, ':');
+	if (!p) {
 		ical_log(log, ICAL_VEVENT, itemline,
 			 _("recurrence rule malformed."));
 		(*noskipped)++;
+		return NULL;
 	}
+	p++;
+
+	rpt = mem_malloc(sizeof(ical_rpt_t));
+	memset(rpt, 0, sizeof(ical_rpt_t));
+	if (sscanf(p, "FREQ=%s", freqstr) != 1) {
+		ical_log(log, ICAL_VEVENT, itemline,
+			 _("recurrence frequence not found."));
+		(*noskipped)++;
+		mem_free(rpt);
+		return NULL;
+	}
+
+	if (starts_with(freqstr, "DAILY")) {
+		rpt->type = RECUR_DAILY;
+	} else if (starts_with(freqstr, "WEEKLY")) {
+		rpt->type = RECUR_WEEKLY;
+	} else if (starts_with(freqstr, "MONTHLY")) {
+		rpt->type = RECUR_MONTHLY;
+	} else if (starts_with(freqstr, "YEARLY")) {
+		rpt->type = RECUR_YEARLY;
+	} else {
+		ical_log(log, ICAL_VEVENT, itemline,
+			 _("recurrence frequence not recognized."));
+		(*noskipped)++;
+		mem_free(rpt);
+		return NULL;
+	}
+
+	/*
+	 * The UNTIL rule part defines a date-time value which bounds the
+	 * recurrence rule in an inclusive manner.  If not present, and the
+	 * COUNT rule part is also not present, the RRULE is considered to
+	 * repeat forever.
+
+	 * The COUNT rule part defines the number of occurrences at which to
+	 * range-bound the recurrence.  The "DTSTART" property value, if
+	 * specified, counts as the first occurrence.
+	 */
+	if ((p = strstr(rrulestr, "UNTIL")) != NULL) {
+		rpt->until = ical_datetime2time_t(strchr(p, '=') + 1, NULL);
+	} else {
+		unsigned cnt;
+		char *countstr;
+
+		rpt->until = 0;
+		if ((countstr = strstr(rrulestr, count))) {
+			countstr += sizeof(count) - 1;
+			if (sscanf(countstr, "%u", &cnt) == 1)
+				rpt->count = cnt;
+		}
+	}
+
+	rpt->freq = 1;
+	if ((p = strstr(rrulestr, interv))) {
+		p += sizeof(interv) - 1;
+		if (sscanf(p, "%u", &interval) == 1)
+			rpt->freq = interval;
+	}
+
 	return rpt;
 }
 
 static void ical_add_exc(llist_t * exc_head, long date)
 {
-	if (date != 0) {
-		struct excp *exc = mem_malloc(sizeof(struct excp));
-		exc->st = date;
+	if (date == 0)
+		return;
 
-		LLIST_ADD(exc_head, exc);
-	}
+	struct excp *exc = mem_malloc(sizeof(struct excp));
+	exc->st = date;
+
+	LLIST_ADD(exc_head, exc);
 }
 
 /*
@@ -808,27 +806,26 @@ ical_read_exdate(llist_t * exc, FILE * log, char *exstr,
 		 unsigned *noskipped, const int itemline)
 {
 	char *p, *q;
-	long date;
 
-	if ((p = strchr(exstr, ':')) != NULL) {
-		p++;
-		while ((q = strchr(p, ',')) != NULL) {
-			char buf[BUFSIZ];
-			const int buflen = q - p;
-
-			strncpy(buf, p, buflen);
-			buf[buflen] = '\0';
-			date = ical_datetime2long(buf, NULL);
-			ical_add_exc(exc, date);
-			p = ++q;
-		}
-		date = ical_datetime2long(p, NULL);
-		ical_add_exc(exc, date);
-	} else {
+	p = strchr(exstr, ':');
+	if (!p) {
 		ical_log(log, ICAL_VEVENT, itemline,
 			 _("recurrence exception dates malformed."));
 		(*noskipped)++;
+		return;
 	}
+	p++;
+
+	while ((q = strchr(p, ',')) != NULL) {
+		char buf[BUFSIZ];
+		const int buflen = q - p;
+
+		strncpy(buf, p, buflen);
+		buf[buflen] = '\0';
+		ical_add_exc(exc, ical_datetime2time_t(buf, NULL));
+		p = ++q;
+	}
+	ical_add_exc(exc, ical_datetime2time_t(p, NULL));
 }
 
 /* Return an allocated string containing the name of the newly created note. */
@@ -838,28 +835,29 @@ static char *ical_read_note(char *line, unsigned *noskipped,
 {
 	char *p, *notestr, *note;
 
-	if ((p = strchr(line, ':')) != NULL) {
-		p++;
-		notestr = ical_unformat_line(p);
-		if (notestr == NULL) {
-			ical_log(log, item_type, itemline,
-				 _("could not get entire item description."));
-			(*noskipped)++;
-			return NULL;
-		} else if (strlen(notestr) == 0) {
-			mem_free(notestr);
-			return NULL;
-		} else {
-			note = generate_note(notestr);
-			mem_free(notestr);
-			return note;
-		}
-	} else {
+	p = strchr(line, ':');
+	if (!p) {
 		ical_log(log, item_type, itemline,
 			 _("description malformed."));
 		(*noskipped)++;
 		return NULL;
 	}
+	p++;
+
+	notestr = ical_unformat_line(p);
+	if (notestr == NULL) {
+		ical_log(log, item_type, itemline,
+			 _("could not get entire item description."));
+		(*noskipped)++;
+		return NULL;
+	} else if (strlen(notestr) == 0) {
+		mem_free(notestr);
+		return NULL;
+	} else {
+		note = generate_note(notestr);
+		mem_free(notestr);
+		return note;
+	}
 }
 
 /* Returns an allocated string containing the ical item summary. */
@@ -885,19 +883,10 @@ static char *ical_read_summary(char *line)
 static void
 ical_read_event(FILE * fdi, FILE * log, unsigned *noevents,
 		unsigned *noapoints, unsigned *noskipped, char *buf,
-		char *lstore, unsigned *lineno)
+		char *lstore, unsigned *lineno, const char *fmt_ev,
+		const char *fmt_rev, const char *fmt_apt, const char *fmt_rapt)
 {
 	const int ITEMLINE = *lineno;
-	const char endevent[] = "END:VEVENT";
-	const char summary[] = "SUMMARY";
-	const char dtstart[] = "DTSTART";
-	const char dtend[] = "DTEND";
-	const char duration[] = "DURATION";
-	const char rrule[] = "RRULE";
-	const char exdate[] = "EXDATE";
-	const char alarm[] = "BEGIN:VALARM";
-	const char endalarm[] = "END:VALARM";
-	const char desc[] = "DESCRIPTION";
 	ical_vevent_e vevent_type;
 	char *p;
 	struct {
@@ -915,169 +904,122 @@ ical_read_event(FILE * fdi, FILE * log, unsigned *noevents,
 	skip_alarm = 0;
 	while (ical_readline(fdi, buf, lstore, lineno)) {
 		if (skip_alarm) {
-			/* Need to skip VALARM properties because some keywords could
-			   interfere, such as DURATION, SUMMARY,.. */
-			if (strncasecmp
-			    (buf, endalarm, sizeof(endalarm) - 1) == 0)
+			/*
+			 * Need to skip VALARM properties because some keywords
+			 * could interfere, such as DURATION, SUMMARY,..
+			 */
+			if (starts_with_ci(buf, "END:VALARM"))
 				skip_alarm = 0;
 			continue;
 		}
-		if (strncasecmp(buf, endevent, sizeof(endevent) - 1) == 0) {
-			if (vevent.mesg) {
-				if (vevent.rpt && vevent.rpt->count)
-					vevent.rpt->until =
-					    ical_compute_rpt_until(vevent.
-								   start,
-								   vevent.
-								   rpt);
 
-				switch (vevent_type) {
-				case APPOINTMENT:
-					if (vevent.start == 0) {
-						ical_log(log, ICAL_VEVENT,
-							 ITEMLINE,
-							 _("appointment has no start time."));
-						goto cleanup;
-					}
-					if (vevent.dur == 0) {
-						if (vevent.end == 0) {
-							ical_log(log,
-								 ICAL_VEVENT,
-								 ITEMLINE,
-								 _("could not compute duration "
-								  "(no end time)."));
-							goto cleanup;
-						} else if (vevent.start ==
-							   vevent.end) {
-							vevent_type =
-							    EVENT;
-							vevent.end = 0L;
-							ical_store_event
-							    (vevent.mesg,
-							     vevent.note,
-							     vevent.start,
-							     vevent.end,
-							     vevent.rpt,
-							     &vevent.exc);
-							(*noevents)++;
-							return;
-						} else {
-							vevent.dur =
-							    vevent.end -
-							    vevent.start;
-							if (vevent.dur < 0) {
-								ical_log
-								    (log,
-								     ICAL_VEVENT,
-								     ITEMLINE,
-								     _("item has a negative duration."));
-								goto cleanup;
-							}
-						}
-					}
-					ical_store_apoint(vevent.mesg,
-							  vevent.note,
-							  vevent.start,
-							  vevent.dur,
-							  vevent.rpt,
-							  &vevent.exc,
-							  vevent.
-							  has_alarm);
-					(*noapoints)++;
-					break;
-				case EVENT:
-					if (vevent.start == 0) {
-						ical_log(log, ICAL_VEVENT,
-							 ITEMLINE,
-							 _("event date is not defined."));
-						goto cleanup;
-					}
-					ical_store_event(vevent.mesg,
-							 vevent.note,
-							 vevent.start,
-							 vevent.end,
-							 vevent.rpt,
-							 &vevent.exc);
-					(*noevents)++;
-					break;
-				case UNDEFINED:
-					ical_log(log, ICAL_VEVENT,
-						 ITEMLINE,
-						 _("item could not be identified."));
-					goto cleanup;
-					break;
-				}
-			} else {
+		if (starts_with_ci(buf, "END:VEVENT")) {
+			if (!vevent.mesg) {
 				ical_log(log, ICAL_VEVENT, ITEMLINE,
 					 _("could not retrieve item summary."));
 				goto cleanup;
 			}
-			return;
-		} else {
-			if (strncasecmp(buf, dtstart, sizeof(dtstart) - 1)
-			    == 0) {
-				if ((p = strchr(buf, ':')) != NULL)
-					vevent.start =
-					    ical_datetime2long(++p,
-							       &vevent_type);
-				if (!vevent.start) {
-					ical_log(log, ICAL_VEVENT,
-						 ITEMLINE,
-						 _("could not retrieve event start time."));
-					goto cleanup;
-				}
-			} else
-			    if (strncasecmp(buf, dtend, sizeof(dtend) - 1)
-				== 0) {
-				if ((p = strchr(buf, ':')) != NULL)
-					vevent.end =
-					    ical_datetime2long(++p,
-							       &vevent_type);
-				if (!vevent.end) {
-					ical_log(log, ICAL_VEVENT,
-						 ITEMLINE,
-						 _("could not retrieve event end time."));
-					goto cleanup;
-				}
-			} else
-			    if (strncasecmp
-				(buf, duration,
-				 sizeof(duration) - 1) == 0) {
-				if ((vevent.dur = ical_dur2long(buf)) <= 0) {
-					ical_log(log, ICAL_VEVENT,
-						 ITEMLINE,
-						 _("item duration malformed."));
-					goto cleanup;
-				}
-			} else
-			    if (strncasecmp(buf, rrule, sizeof(rrule) - 1)
-				== 0) {
-				vevent.rpt =
-				    ical_read_rrule(log, buf, noskipped,
-						    ITEMLINE);
-			} else
-			    if (strncasecmp
-				(buf, exdate, sizeof(exdate) - 1) == 0) {
-				ical_read_exdate(&vevent.exc, log, buf,
-						 noskipped, ITEMLINE);
-			} else
-			    if (strncasecmp
-				(buf, summary, sizeof(summary) - 1) == 0) {
-				vevent.mesg = ical_read_summary(buf);
-			} else
-			    if (strncasecmp(buf, alarm, sizeof(alarm) - 1)
-				== 0) {
-				skip_alarm = 1;
-				vevent.has_alarm = 1;
-			} else if (strncasecmp(buf, desc, sizeof(desc) - 1)
-				   == 0) {
-				vevent.note =
-				    ical_read_note(buf, noskipped,
-						   ICAL_VEVENT, ITEMLINE,
-						   log);
+			if (vevent.start == 0) {
+				ical_log(log, ICAL_VEVENT, ITEMLINE,
+					 _("item start date is not defined."));
+				goto cleanup;
 			}
+
+			if (vevent_type == APPOINTMENT && vevent.dur == 0) {
+				if (vevent.end != 0) {
+					vevent.dur = vevent.end - vevent.start;
+				}
+
+				if (vevent.dur < 0) {
+					ical_log(log, ICAL_VEVENT, ITEMLINE,
+						_("item has a negative duration."));
+					goto cleanup;
+				}
+			}
+
+			if (vevent.rpt && vevent.rpt->count) {
+				vevent.rpt->until =
+					ical_compute_rpt_until(vevent.start,
+							vevent.rpt);
+			}
+
+			switch (vevent_type) {
+			case APPOINTMENT:
+				ical_store_apoint(vevent.mesg, vevent.note,
+						vevent.start, vevent.dur,
+						vevent.rpt, &vevent.exc,
+						vevent.has_alarm, fmt_apt,
+						fmt_rapt);
+				(*noapoints)++;
+				break;
+			case EVENT:
+				ical_store_event(vevent.mesg, vevent.note,
+						vevent.start, vevent.end,
+						vevent.rpt, &vevent.exc,
+						fmt_ev, fmt_rev);
+				(*noevents)++;
+				break;
+			case UNDEFINED:
+				ical_log(log, ICAL_VEVENT, ITEMLINE,
+					 _("item could not be identified."));
+				goto cleanup;
+				break;
+			}
+
+			return;
+		}
+
+		if (starts_with_ci(buf, "DTSTART")) {
+			p = strchr(buf, ':');
+			if (!p) {
+				ical_log(log, ICAL_VEVENT, ITEMLINE,
+					 _("event start time malformed."));
+				goto cleanup;
+			}
+
+			vevent.start = ical_datetime2time_t(++p, &vevent_type);
+			if (!vevent.start) {
+				ical_log(log, ICAL_VEVENT, ITEMLINE,
+					 _("could not retrieve event start time."));
+				goto cleanup;
+			}
+		} else if (starts_with_ci(buf, "DTEND")) {
+			p = strchr(buf, ':');
+			if (!p) {
+				ical_log(log, ICAL_VEVENT, ITEMLINE,
+					 _("event end time malformed."));
+				goto cleanup;
+			}
+
+			vevent.end = ical_datetime2time_t(++p, &vevent_type);
+			if (!vevent.end) {
+				ical_log(log, ICAL_VEVENT, ITEMLINE,
+					 _("could not retrieve event end time."));
+				goto cleanup;
+			}
+		} else if (starts_with_ci(buf, "DURATION")) {
+			vevent.dur = ical_dur2long(buf);
+			if (vevent.dur <= 0) {
+				ical_log(log, ICAL_VEVENT, ITEMLINE,
+					 _("item duration malformed."));
+				goto cleanup;
+			}
+		} else if (starts_with_ci(buf, "RRULE")) {
+			vevent.rpt = ical_read_rrule(log, buf, noskipped,
+					ITEMLINE);
+		} else if (starts_with_ci(buf, "EXDATE")) {
+			ical_read_exdate(&vevent.exc, log, buf, noskipped,
+					ITEMLINE);
+		} else if (starts_with_ci(buf, "SUMMARY")) {
+			vevent.mesg = ical_read_summary(buf);
+		} else if (starts_with_ci(buf, "BEGIN:VALARM")) {
+			skip_alarm = vevent.has_alarm = 1;
+		} else if (starts_with_ci(buf, "DESCRIPTION")) {
+			vevent.note = ical_read_note(buf, noskipped,
+					ICAL_VEVENT, ITEMLINE, log);
 		}
 	}
+
 	ical_log(log, ICAL_VEVENT, ITEMLINE,
 		 _("The ical file seems to be malformed. "
 		   "The end of item was not found."));
@@ -1095,20 +1037,14 @@ cleanup:
 }
 
 static void
-ical_read_todo(FILE * fdi, FILE * log, unsigned *notodos,
-	       unsigned *noskipped, char *buf, char *lstore,
-	       unsigned *lineno)
+ical_read_todo(FILE * fdi, FILE * log, unsigned *notodos, unsigned *noskipped,
+	       char *buf, char *lstore, unsigned *lineno, const char *fmt_todo)
 {
-	const char endtodo[] = "END:VTODO";
-	const char summary[] = "SUMMARY";
-	const char alarm[] = "BEGIN:VALARM";
-	const char endalarm[] = "END:VALARM";
-	const char desc[] = "DESCRIPTION";
-	const int LOWEST = 9;
 	const int ITEMLINE = *lineno;
 	struct {
 		char *mesg, *note;
-		int has_priority, priority;
+		int priority;
+		int completed;
 	} vtodo;
 	int skip_alarm;
 
@@ -1116,65 +1052,52 @@ ical_read_todo(FILE * fdi, FILE * log, unsigned *notodos,
 	skip_alarm = 0;
 	while (ical_readline(fdi, buf, lstore, lineno)) {
 		if (skip_alarm) {
-			/* Need to skip VALARM properties because some keywords could
-			   interfere, such as DURATION, SUMMARY,.. */
-			if (strncasecmp
-			    (buf, endalarm, sizeof(endalarm) - 1) == 0)
+			/*
+			 * Need to skip VALARM properties because some keywords
+			 * could interfere, such as DURATION, SUMMARY,..
+			 */
+			if (starts_with_ci(buf, "END:VALARM"))
 				skip_alarm = 0;
 			continue;
 		}
-		if (strncasecmp(buf, endtodo, sizeof(endtodo) - 1) == 0) {
-			if (!vtodo.has_priority)
-				vtodo.priority = LOWEST;
-			if (vtodo.mesg) {
-				ical_store_todo(vtodo.priority, vtodo.mesg,
-						vtodo.note);
-				(*notodos)++;
-			} else {
+
+		if (starts_with_ci(buf, "END:VTODO")) {
+			if (!vtodo.mesg) {
 				ical_log(log, ICAL_VTODO, ITEMLINE,
 					 _("could not retrieve item summary."));
 				goto cleanup;
 			}
-			return;
-		} else {
-			int tmpint;
 
-			if (strncasecmp
-			    (buf, "PRIORITY:",
-			     sizeof("PRIORITY:") - 1) == 0) {
-				sscanf(buf, "%d", &tmpint);
-				if (tmpint <= 9 && tmpint >= 1) {
-					vtodo.priority = tmpint;
-					vtodo.has_priority = 1;
-				} else {
-					ical_log(log, ICAL_VTODO, ITEMLINE,
-						 _("item priority is not acceptable "
-						  "(must be between 1 and 9)."));
-					vtodo.priority = LOWEST;
-				}
-			} else
-			    if (strncasecmp
-				(buf, summary, sizeof(summary) - 1) == 0) {
-				vtodo.mesg = ical_read_summary(buf);
-			} else
-			    if (strncasecmp(buf, alarm, sizeof(alarm) - 1)
-				== 0) {
-				skip_alarm = 1;
-			} else if (strncasecmp(buf, desc, sizeof(desc) - 1)
-				   == 0) {
-				vtodo.note =
-				    ical_read_note(buf, noskipped,
-						   ICAL_VTODO, ITEMLINE,
-						   log);
+			ical_store_todo(vtodo.priority, vtodo.completed,
+					vtodo.mesg, vtodo.note, fmt_todo);
+			(*notodos)++;
+			return;
+		}
+
+		if (starts_with_ci(buf, "PRIORITY:")) {
+			sscanf(buf, "%d", &vtodo.priority);
+			if (vtodo.priority < 0 || vtodo.priority > 9) {
+				ical_log(log, ICAL_VTODO, ITEMLINE,
+					 _("item priority is invalid "
+					  "(must be between 0 and 9)."));
 			}
+		} else if (starts_with_ci(buf, "STATUS:COMPLETED")) {
+			vtodo.completed = 1;
+		} else if (starts_with_ci(buf, "SUMMARY")) {
+			vtodo.mesg = ical_read_summary(buf);
+		} else if (starts_with_ci(buf, "BEGIN:VALARM")) {
+			skip_alarm = 1;
+		} else if (starts_with_ci(buf, "DESCRIPTION")) {
+			vtodo.note = ical_read_note(buf, noskipped, ICAL_VTODO,
+					ITEMLINE, log);
 		}
 	}
+
 	ical_log(log, ICAL_VTODO, ITEMLINE,
 		 _("The ical file seems to be malformed. "
 		   "The end of item was not found."));
 
 cleanup:
-
 	if (vtodo.note)
 		mem_free(vtodo.note);
 	if (vtodo.mesg)
@@ -1186,10 +1109,10 @@ cleanup:
 void
 ical_import_data(FILE * stream, FILE * log, unsigned *events,
 		 unsigned *apoints, unsigned *todos, unsigned *lines,
-		 unsigned *skipped)
+		 unsigned *skipped, const char *fmt_ev, const char *fmt_rev,
+		 const char *fmt_apt, const char *fmt_rapt,
+		 const char *fmt_todo)
 {
-	const char vevent[] = "BEGIN:VEVENT";
-	const char vtodo[] = "BEGIN:VTODO";
 	char buf[BUFSIZ], lstore[BUFSIZ];
 	int major, minor;
 
@@ -1203,24 +1126,25 @@ ical_import_data(FILE * stream, FILE * log, unsigned *events,
 
 	while (ical_readline(stream, buf, lstore, lines)) {
 		(*lines)++;
-		if (strncasecmp(buf, vevent, sizeof(vevent) - 1) == 0) {
+		if (starts_with_ci(buf, "BEGIN:VEVENT")) {
 			ical_read_event(stream, log, events, apoints,
-					skipped, buf, lstore, lines);
-		} else if (strncasecmp(buf, vtodo, sizeof(vtodo) - 1) == 0) {
+					skipped, buf, lstore, lines, fmt_ev,
+					fmt_rev, fmt_apt, fmt_rapt);
+		} else if (starts_with_ci(buf, "BEGIN:VTODO")) {
 			ical_read_todo(stream, log, todos, skipped, buf,
-				       lstore, lines);
+				       lstore, lines, fmt_todo);
 		}
 	}
 }
 
 /* Export calcurse data. */
-void ical_export_data(FILE * stream)
+void ical_export_data(FILE * stream, int export_uid)
 {
 	ical_export_header(stream);
-	ical_export_recur_events(stream);
-	ical_export_events(stream);
-	ical_export_recur_apoints(stream);
-	ical_export_apoints(stream);
-	ical_export_todo(stream);
+	ical_export_recur_events(stream, export_uid);
+	ical_export_events(stream, export_uid);
+	ical_export_recur_apoints(stream, export_uid);
+	ical_export_apoints(stream, export_uid);
+	ical_export_todo(stream, export_uid);
 	ical_export_footer(stream);
 }
