@@ -1,7 +1,7 @@
 /*
  * Calcurse - text-based organizer
  *
- * Copyright (c) 2004-2015 calcurse Development Team <misc@calcurse.org>
+ * Copyright (c) 2004-2016 calcurse Development Team <misc@calcurse.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -239,6 +239,7 @@ void io_init(const char *cfile, const char *datadir)
 		snprintf(path_dpid, BUFSIZ, "%s/" DPID_PATH_NAME, home);
 		snprintf(path_dmon_log, BUFSIZ, "%s/" DLOG_PATH_NAME,
 			 home);
+		snprintf(path_hooks, BUFSIZ, "%s/" HOOKS_DIR_NAME, home);
 	} else {
 		home = getenv("HOME");
 		if (home == NULL) {
@@ -252,6 +253,7 @@ void io_init(const char *cfile, const char *datadir)
 		snprintf(path_dpid, BUFSIZ, "%s/" DPID_PATH, home);
 		snprintf(path_dmon_log, BUFSIZ, "%s/" DLOG_PATH, home);
 		snprintf(path_notes, BUFSIZ, "%s/" NOTES_DIR, home);
+		snprintf(path_hooks, BUFSIZ, "%s/" HOOKS_DIR, home);
 	}
 
 	if (cfile == NULL) {
@@ -293,6 +295,37 @@ void io_save_mutex_unlock(void)
 	pthread_mutex_unlock(&io_save_mutex);
 }
 
+/* Print all appointments and events to stdout. */
+void io_dump_apts(const char *fmt_apt, const char *fmt_rapt,
+		  const char *fmt_ev, const char *fmt_rev)
+{
+	llist_item_t *i;
+
+	LLIST_FOREACH(&recur_elist, i) {
+		struct recur_event *rev = LLIST_GET_DATA(i);
+		time_t day = update_time_in_date(rev->day, 0, 0);
+		print_recur_event(fmt_rev, day, rev);
+	}
+
+	LLIST_TS_FOREACH(&recur_alist_p, i) {
+		struct recur_apoint *rapt = LLIST_GET_DATA(i);
+		time_t day = update_time_in_date(rapt->start, 0, 0);
+		print_recur_apoint(fmt_rapt, day, rapt->start, rapt);
+	}
+
+	LLIST_TS_FOREACH(&alist_p, i) {
+		struct apoint *apt = LLIST_TS_GET_DATA(i);
+		time_t day = update_time_in_date(apt->start, 0, 0);
+		print_apoint(fmt_apt, day, apt);
+	}
+
+	LLIST_FOREACH(&eventlist, i) {
+		struct event *ev = LLIST_TS_GET_DATA(i);
+		time_t day = update_time_in_date(ev->day, 0, 0);
+		print_event(fmt_ev, day, ev);
+	}
+}
+
 /*
  * Save the apts data file, which contains the
  * appointments first, and then the events.
@@ -335,6 +368,17 @@ unsigned io_save_apts(const char *aptsfile)
 	return 1;
 }
 
+/* Print all todo items to stdout. */
+void io_dump_todo(const char *fmt_todo)
+{
+	llist_item_t *i;
+
+	LLIST_FOREACH(&todolist, i) {
+		struct todo *todo = LLIST_TS_GET_DATA(i);
+		print_todo(fmt_todo, todo);
+	}
+}
+
 /* Save the todo data file. */
 unsigned io_save_todo(const char *todofile)
 {
@@ -391,6 +435,7 @@ void io_save_cal(enum save_display display)
 	if (read_only)
 		return;
 
+	run_hook("pre-save");
 	pthread_mutex_lock(&io_save_mutex);
 
 	show_bar = 0;
@@ -422,12 +467,13 @@ void io_save_cal(enum save_display display)
 
 	/* Print a message telling data were saved */
 	if (ui_mode == UI_CURSES && display == IO_SAVE_DISPLAY_BAR &&
-	    conf.system_dialogs) {
+	    show_dialogs()) {
 		status_mesg(save_success, enter);
 		wgetch(win[KEY].p);
 	}
 
 	pthread_mutex_unlock(&io_save_mutex);
+	run_hook("post-save");
 }
 
 static void io_load_error(const char *filename, unsigned line,
@@ -626,7 +672,7 @@ void io_load_todo(struct item_filter *filter)
 	FILE *data_file;
 	char *newline;
 	int nb_tod = 0;
-	int c, id;
+	int c, id, completed;
 	char buf[BUFSIZ], e_todo[BUFSIZ], note[MAX_NOTESIZ + 1];
 	unsigned line = 0;
 
@@ -638,7 +684,15 @@ void io_load_todo(struct item_filter *filter)
 		c = getc(data_file);
 		if (c == EOF) {
 			break;
-		} else if (c == '[') {	/* new style with id */
+		} else if (c == '[') {
+			/* new style with id */
+			c = getc(data_file);
+			if (c == '-') {
+				completed = 1;
+			} else {
+				completed = 0;
+				ungetc(c, data_file);
+			}
 			if (fscanf(data_file, " %d ", &id) != 1
 			    || getc(data_file) != ']')
 				io_load_error(path_todo, line,
@@ -647,6 +701,7 @@ void io_load_todo(struct item_filter *filter)
 			ungetc(c, data_file);
 		} else {
 			id = 9;
+			completed = 0;
 			ungetc(c, data_file);
 		}
 		/* Now read the attached note, if any. */
@@ -674,14 +729,26 @@ void io_load_todo(struct item_filter *filter)
 				continue;
 			if (filter->priority && id != filter->priority)
 				continue;
-			if (filter->completed && id > 0)
+			if (filter->completed && !completed)
 				continue;
-			if (filter->uncompleted && id < 0)
+			if (filter->uncompleted && completed)
 				continue;
 		}
 
-		todo_add(e_todo, id, note);
-		++nb_tod;
+		struct todo *todo = todo_add(e_todo, id, completed, note);
+
+		/* Filter by hash. */
+		if (filter && filter->hash) {
+			char *hash = todo_hash(todo);
+			if (!hash_matches(filter->hash, hash)) {
+				todo_delete(todo);
+				todo = NULL;
+			}
+			mem_free(hash);
+		}
+
+		if (todo)
+			++nb_tod;
 	}
 	file_close(data_file, __FILE_POS__);
 }
@@ -689,8 +756,10 @@ void io_load_todo(struct item_filter *filter)
 /* Load appointments and todo items */
 void io_load_data(struct item_filter *filter)
 {
+	run_hook("pre-load");
 	io_load_app(filter);
 	io_load_todo(filter);
+	run_hook("post-load");
 }
 
 void io_reload_data(void)
@@ -728,6 +797,14 @@ void io_reload_data(void)
 			io_save_todo(path_todo_backup);
 			io_save_mutex_unlock();
 
+			/*
+			 * We do not directly write to the data files here;
+			 * however, the external merge tool might incorporate
+			 * changes from the backup file into the main data
+			 * files.
+			 */
+			run_hook("pre-save");
+
 			if (!io_files_equal(path_apts, path_apts_backup)) {
 				const char *arg_apts[] = { conf.mergetool,
 							   path_apts,
@@ -744,8 +821,17 @@ void io_reload_data(void)
 				wins_launch_external(arg_todo);
 			}
 
-			xfree(path_apts_backup);
-			xfree(path_todo_backup);
+			mem_free(path_apts_backup);
+			mem_free(path_todo_backup);
+
+			/*
+			 * We do not directly write to the data files here;
+			 * however, the external merge tool will likely have
+			 * incorporated changes from the backup file into the
+			 * main data files at this point.
+			 */
+			run_hook("post-save");
+
 			break;
 		case 3:
 			/* FALLTHROUGH */
@@ -771,12 +857,20 @@ void io_reload_data(void)
 	recur_event_llist_init();
 	todo_init_list();
 
+	/*
+	 * Temporarily reinitialize the todo list box without any items to make
+	 * sure wins_unprepare_external() does not fail when it is called after
+	 * executing the pre-load hook.
+	 */
+	ui_todo_load_items();
+	ui_todo_sel_reset();
+
 	io_load_data(NULL);
 	io_unset_modified();
 	ui_todo_load_items();
 	ui_todo_sel_reset();
 
-	if (conf.system_dialogs) {
+	if (show_dialogs()) {
 		status_mesg(reload_success, enter);
 		wgetch(win[KEY].p);
 	}
@@ -1066,7 +1160,7 @@ void io_startup_screen(int no_data_file)
 }
 
 /* Export calcurse data. */
-void io_export_data(enum export_type type)
+void io_export_data(enum export_type type, int export_uid)
 {
 	FILE *stream = NULL;
 	const char *success = _("The data were successfully exported");
@@ -1091,11 +1185,11 @@ void io_export_data(enum export_type type)
 		return;
 
 	if (type == IO_EXPORT_ICAL)
-		ical_export_data(stream);
+		ical_export_data(stream, export_uid);
 	else if (type == IO_EXPORT_PCAL)
 		pcal_export_data(stream);
 
-	if (conf.system_dialogs && ui_mode == UI_CURSES) {
+	if (show_dialogs() && ui_mode == UI_CURSES) {
 		status_mesg(success, enter);
 		wgetch(win[KEY].p);
 	}
@@ -1136,7 +1230,10 @@ static FILE *get_import_stream(enum import_type type)
  * A temporary log file is created in /tmp to store the import process report,
  * and is cleared at the end.
  */
-void io_import_data(enum import_type type, const char *stream_name)
+void io_import_data(enum import_type type, const char *stream_name,
+		    const char *fmt_ev, const char *fmt_rev,
+		    const char *fmt_apt, const char *fmt_rapt,
+		    const char *fmt_todo)
 {
 	const char *proc_report =
 	    _("Import process report: %04d lines read");
@@ -1151,7 +1248,10 @@ void io_import_data(enum import_type type, const char *stream_name)
 		|| type >= IO_IMPORT_NBTYPES, _("unknown import type"));
 	switch (ui_mode) {
 	case UI_CMDLINE:
-		stream = fopen(stream_name, "r");
+		if (!strcmp(stream_name, "-"))
+			stream = stdin;
+		else
+			stream = fopen(stream_name, "r");
 		EXIT_IF(stream == NULL,
 			_("FATAL ERROR: the input file cannot be accessed, "
 			 "Aborting..."));
@@ -1179,7 +1279,8 @@ void io_import_data(enum import_type type, const char *stream_name)
 	if (type == IO_IMPORT_ICAL)
 		ical_import_data(stream, log->fd, &stats.events,
 				 &stats.apoints, &stats.todos,
-				 &stats.lines, &stats.skipped);
+				 &stats.lines, &stats.skipped, fmt_ev, fmt_rev,
+				 fmt_apt, fmt_rapt, fmt_todo);
 
 	if (stream != stdin)
 		file_close(stream, __FILE_POS__);
@@ -1193,7 +1294,7 @@ void io_import_data(enum import_type type, const char *stream_name)
 		 stats.todos);
 	asprintf(&stats_str[3], _("%d skipped"), stats.skipped);
 
-	if (ui_mode == UI_CURSES && conf.system_dialogs) {
+	if (ui_mode == UI_CURSES && show_dialogs()) {
 		char *read, *stat;
 
 		asprintf(&read, proc_report, stats.lines);
@@ -1204,7 +1305,7 @@ void io_import_data(enum import_type type, const char *stream_name)
 		mem_free(read);
 		mem_free(stat);
 		wgetch(win[KEY].p);
-	} else if (ui_mode == UI_CMDLINE) {
+	} else if (ui_mode == UI_CMDLINE && show_dialogs()) {
 		printf(proc_report, stats.lines);
 		printf("\n%s / %s / %s / %s\n", stats_str[0], stats_str[1],
 		       stats_str[2], stats_str[3]);
