@@ -1,7 +1,7 @@
 /*
  * Calcurse - text-based organizer
  *
- * Copyright (c) 2004-2015 calcurse Development Team <misc@calcurse.org>
+ * Copyright (c) 2004-2016 calcurse Development Team <misc@calcurse.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -56,8 +56,27 @@ void listbox_delete(struct listbox *lb)
 {
 	EXIT_IF(lb == NULL, "null pointer");
 	wins_scrollwin_delete(&(lb->sw));
-	free(lb->type);
-	free(lb->ch);
+	mem_free(lb->type);
+	mem_free(lb->ch);
+}
+
+static void listbox_fix_visible_region(struct listbox *lb)
+{
+	unsigned last_line = lb->ch[lb->item_count] - 1;
+	int i;
+
+	wins_scrollwin_ensure_visible(&(lb->sw), lb->ch[lb->item_sel]);
+	wins_scrollwin_ensure_visible(&(lb->sw), lb->ch[lb->item_sel + 1] - 1);
+
+	i = lb->item_sel - 1;
+	while (i >= 0 && lb->type[i] != LISTBOX_ROW_TEXT) {
+		wins_scrollwin_ensure_visible(&(lb->sw), lb->ch[i]);
+		wins_scrollwin_ensure_visible(&(lb->sw), lb->ch[i + 1] - 1);
+		i++;
+	}
+
+	if (wins_scrollwin_is_visible(&(lb->sw), last_line))
+		wins_scrollwin_set_lower(&(lb->sw), last_line);
 }
 
 void listbox_resize(struct listbox *lb, int y, int x, int h, int w)
@@ -68,13 +87,7 @@ void listbox_resize(struct listbox *lb, int y, int x, int h, int w)
 	if (lb->item_sel < 0)
 		return;
 
-	unsigned last_line = lb->ch[lb->item_count] - 1;
-
-	if (wins_scrollwin_is_visible(&(lb->sw), last_line))
-		wins_scrollwin_set_lower(&(lb->sw), last_line);
-
-	wins_scrollwin_ensure_visible(&(lb->sw), lb->ch[lb->item_sel]);
-	wins_scrollwin_ensure_visible(&(lb->sw), lb->ch[lb->item_sel + 1] - 1);
+	listbox_fix_visible_region(lb);
 }
 
 void listbox_set_cb_data(struct listbox *lb, void *cb_data)
@@ -95,8 +108,8 @@ void listbox_load_items(struct listbox *lb, int item_count)
 		return;
 	}
 
-	free(lb->type);
-	free(lb->ch);
+	mem_free(lb->type);
+	mem_free(lb->ch);
 	lb->type = mem_malloc(item_count * sizeof(unsigned));
 	lb->ch = mem_malloc((item_count + 1) * sizeof(unsigned));
 	for (i = 0, ch = 0; i < item_count; i++) {
@@ -113,6 +126,7 @@ void listbox_load_items(struct listbox *lb, int item_count)
 	else if (lb->item_sel >= item_count)
 		lb->item_sel = item_count - 1;
 	listbox_fix_sel(lb, 1);
+	listbox_fix_visible_region(lb);
 }
 
 void listbox_draw_deco(struct listbox *lb, int hilt)
@@ -163,21 +177,6 @@ static void listbox_fix_sel(struct listbox *lb, int direction)
 	}
 }
 
-static void listbox_fix_visible_region(struct listbox *lb)
-{
-	int i;
-
-	wins_scrollwin_ensure_visible(&(lb->sw), lb->ch[lb->item_sel]);
-	wins_scrollwin_ensure_visible(&(lb->sw), lb->ch[lb->item_sel + 1] - 1);
-
-	i = lb->item_sel - 1;
-	while (i >= 0 && lb->type[i] != LISTBOX_ROW_TEXT) {
-		wins_scrollwin_ensure_visible(&(lb->sw), lb->ch[i]);
-		wins_scrollwin_ensure_visible(&(lb->sw), lb->ch[i + 1] - 1);
-		i++;
-	}
-}
-
 void listbox_set_sel(struct listbox *lb, unsigned pos)
 {
 	lb->item_sel = pos;
