@@ -1,7 +1,7 @@
 /*
  * Calcurse - text-based organizer
  *
- * Copyright (c) 2004-2015 calcurse Development Team <misc@calcurse.org>
+ * Copyright (c) 2004-2016 calcurse Development Team <misc@calcurse.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -71,18 +71,27 @@ int notify_time_left(void)
 	return left > 0 ? left : 0;
 }
 
+static unsigned notify_trigger(void)
+{
+	int flagged = notify_app.state & APOINT_NOTIFY;
+
+	if (!notify_app.got_app)
+		return 0;
+	if (nbar.notify_all == NOTIFY_ALL)
+		return 1;
+	if (nbar.notify_all == NOTIFY_UNFLAGGED_ONLY)
+		flagged = !flagged;
+	return flagged;
+}
+
 /*
- * Return 1 if the reminder was not sent already for the upcoming
- * appointment.
+ * Return 1 if the reminder was not sent already for the upcoming appointment.
  */
 unsigned notify_needs_reminder(void)
 {
-	if (notify_app.got_app
-	    && (((notify_app.state & APOINT_NOTIFY) && !nbar.notify_all) ||
-		(!(notify_app.state & APOINT_NOTIFY) && nbar.notify_all))
-	    && !(notify_app.state & APOINT_NOTIFIED))
-		return 1;
-	return 0;
+	if (notify_app.state & APOINT_NOTIFIED)
+		return 0;
+	return notify_trigger();
 }
 
 /*
@@ -233,9 +242,8 @@ unsigned notify_launch_cmd(void)
 void notify_update_bar(void)
 {
 	const int space = 3;
-	int file_pos, date_pos, app_pos, txt_max_len, too_long = 0;
+	int file_pos, date_pos, app_pos, txt_max_len;
 	int time_left, blinking;
-	char buf[BUFSIZ];
 
 	date_pos = space;
 	pthread_mutex_lock(&notify.mutex);
@@ -243,7 +251,7 @@ void notify_update_bar(void)
 	file_pos =
 	    strlen(notify.date) + strlen(notify.time) + 7 + 2 * space;
 	app_pos = file_pos + strlen(notify.apts_file) + 2 + space;
-	txt_max_len = col - (app_pos + 12 + space);
+	txt_max_len = MAX(col - (app_pos + 12 + space), 3);
 
 	WINS_NBAR_LOCK;
 	custom_apply_attr(notify.win, ATTR_HIGHEST);
@@ -256,14 +264,12 @@ void notify_update_bar(void)
 
 	pthread_mutex_lock(&notify_app.mutex);
 	if (notify_app.got_app) {
-		if (strlen(notify_app.txt) > txt_max_len) {
-			int shrink_len;
+		char buf[txt_max_len * UTF8_MAXLEN];
+
+		strncpy(buf, notify_app.txt, txt_max_len * UTF8_MAXLEN);
+		buf[sizeof(buf) - 1] = '\0';
+		utf8_chop(buf, txt_max_len);
 
-			too_long = 1;
-			shrink_len = txt_max_len > 3 ? txt_max_len - 3 : 1;
-			strncpy(buf, notify_app.txt, shrink_len);
-			buf[shrink_len] = '\0';
-		}
 		time_left = notify_time_left();
 		if (time_left > 0) {
 			int hours_left, minutes_left;
@@ -274,27 +280,14 @@ void notify_update_bar(void)
 			     hours_left * HOURINSEC) / MININSEC;
 			pthread_mutex_lock(&nbar.mutex);
 
-			if (time_left < nbar.cntdwn &&
-			    (((notify_app.state & APOINT_NOTIFY)
-			      && !nbar.notify_all)
-			     || (!(notify_app.state & APOINT_NOTIFY)
-				 && nbar.notify_all)))
-				blinking = 1;
-			else
-				blinking = 0;
+			blinking = time_left < nbar.cntdwn && notify_trigger();
 
 			WINS_NBAR_LOCK;
 			if (blinking)
 				wattron(notify.win, A_BLINK);
-			if (too_long)
-				mvwprintw(notify.win, 0, app_pos,
-					  "> %02d:%02d :: %s.. <",
-					  hours_left, minutes_left, buf);
-			else
-				mvwprintw(notify.win, 0, app_pos,
-					  "> %02d:%02d :: %s <",
-					  hours_left, minutes_left,
-					  notify_app.txt);
+			mvwprintw(notify.win, 0, app_pos,
+				  "> %02d:%02d :: %s <", hours_left,
+				  minutes_left, buf);
 			if (blinking)
 				wattroff(notify.win, A_BLINK);
 			WINS_NBAR_UNLOCK;
@@ -499,9 +492,8 @@ void notify_check_added(char *mesg, long start, char state)
 /* Check if the newly repeated appointment is to be notified. */
 void notify_check_repeated(struct recur_apoint *i)
 {
-	unsigned real_app_time;
+	time_t current_time, real_app_time;
 	int update_notify = 0;
-	time_t current_time;
 
 	current_time = time(NULL);
 	pthread_mutex_lock(&notify_app.mutex);
@@ -541,7 +533,7 @@ int notify_same_item(long time)
 int notify_same_recur_item(struct recur_apoint *i)
 {
 	int same = 0;
-	unsigned item_start = 0;
+	time_t item_start = 0;
 
 	recur_item_find_occurrence(i->start, i->dur, &i->exc, i->rpt->type,
 				   i->rpt->freq, i->rpt->until,
@@ -557,6 +549,9 @@ int notify_same_recur_item(struct recur_apoint *i)
 /* Launch the notify-bar main thread. */
 void notify_start_main_thread(void)
 {
+	/* Avoid starting the notification bar thread twice. */
+	notify_stop_main_thread();
+
 	pthread_create(&notify_t_main, NULL, notify_main_thread, NULL);
 	notify_check_next_app(0);
 }
@@ -577,19 +572,15 @@ print_option(WINDOW * win, unsigned x, unsigned y, char *name,
 	mvwprintw(win, y, x, "%s", name);
 	erase_window_part(win, x_opt, y, MAXCOL, y);
 	if ((len = strlen(valstr)) != 0) {
-		unsigned maxlen;
+		unsigned maxlen = MAX(MAXCOL - x_opt - 2, 3);
+		char buf[maxlen * UTF8_MAXLEN];
+
+		strncpy(buf, valstr, maxlen * UTF8_MAXLEN);
+		buf[maxlen * UTF8_MAXLEN - 1] = '\0';
+		utf8_chop(buf, maxlen);
 
-		maxlen = MAXCOL - x_opt - 2;
 		custom_apply_attr(win, ATTR_HIGHEST);
-		if (len < maxlen) {
-			mvwaddstr(win, y, x_opt, valstr);
-		} else {
-			char buf[BUFSIZ];
-
-			strncpy(buf, valstr, maxlen - 1);
-			buf[maxlen - 1] = '\0';
-			mvwprintw(win, y, x_opt, "%s...", buf);
-		}
+		mvwaddstr(win, y, x_opt, buf);
 		custom_remove_attr(win, ATTR_HIGHEST);
 	} else {
 		print_bool_option_incolor(win, valbool, y, x_opt);
@@ -600,7 +591,7 @@ print_option(WINDOW * win, unsigned x, unsigned y, char *name,
 /* Print options related to the notify-bar. */
 static void print_config_option(int i, WINDOW *win, int y, int hilt, void *cb_data)
 {
-	enum { SHOW, DATE, CLOCK, WARN, CMD, NOTIFY_ALL, DMON, DMON_LOG,
+	enum { SHOW, DATE, CLOCK, WARN, CMD, NOTIFYALL, DMON, DMON_LOG,
 		    NB_OPT };
 
 	struct opt_s {
@@ -630,8 +621,8 @@ static void print_config_option(int i, WINDOW *win, int y, int hilt, void *cb_da
 	opt[CMD].desc =
 	    _("(Command used to notify user of an upcoming appointment)");
 
-	opt[NOTIFY_ALL].name = "notification.notifyall = ";
-	opt[NOTIFY_ALL].desc =
+	opt[NOTIFYALL].name = "notification.notifyall = ";
+	opt[NOTIFYALL].desc =
 	    _("(Notify all appointments instead of flagged ones only)");
 
 	opt[DMON].name = "daemon.enable = ";
@@ -652,14 +643,21 @@ static void print_config_option(int i, WINDOW *win, int y, int hilt, void *cb_da
 
 	/* Boolean options */
 	opt[SHOW].valnum = nbar.show;
-	opt[NOTIFY_ALL].valnum = nbar.notify_all;
 	pthread_mutex_unlock(&nbar.mutex);
 
 	opt[DMON].valnum = dmon.enable;
 	opt[DMON_LOG].valnum = dmon.log;
 
-	opt[SHOW].valstr[0] = opt[NOTIFY_ALL].valstr[0] =
-	    opt[DMON].valstr[0] = opt[DMON_LOG].valstr[0] = '\0';
+	opt[SHOW].valstr[0] = opt[DMON].valstr[0] =
+		opt[DMON_LOG].valstr[0] = '\0';
+
+	opt[NOTIFYALL].valnum = nbar.notify_all;
+	if (opt[NOTIFYALL].valnum == NOTIFY_FLAGGED_ONLY)
+		strcpy(opt[NOTIFYALL].valstr, "flagged-only");
+	else if (opt[NOTIFYALL].valnum == NOTIFY_UNFLAGGED_ONLY)
+		strcpy(opt[NOTIFYALL].valstr, "unflagged-only");
+	else if (opt[NOTIFYALL].valnum == NOTIFY_ALL)
+		strcpy(opt[NOTIFYALL].valstr, "all");
 
 	if (hilt)
 		custom_apply_attr(win, ATTR_HIGHEST);
@@ -758,7 +756,7 @@ static void config_option_edit(int i)
 		break;
 	case 5:
 		pthread_mutex_lock(&nbar.mutex);
-		nbar.notify_all = !nbar.notify_all;
+		nbar.notify_all = (nbar.notify_all + 1) % 3;
 		pthread_mutex_unlock(&nbar.mutex);
 		notify_check_next_app(1);
 		break;
