@@ -1,7 +1,7 @@
 /*
  * Calcurse - text-based organizer
  *
- * Copyright (c) 2004-2015 calcurse Development Team <misc@calcurse.org>
+ * Copyright (c) 2004-2016 calcurse Development Team <misc@calcurse.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -311,7 +311,7 @@ static void pcal_export_todo(FILE * stream)
 	fputs("#\n# Todos\n#\n", stream);
 	LLIST_FOREACH(&todolist, i) {
 		struct todo *todo = LLIST_TS_GET_DATA(i);
-		if (todo->id < 0)	/* completed items */
+		if (todo->completed)
 			continue;
 
 		fputs("note all  ", stream);
