@@ -1,7 +1,7 @@
 /*
  * Calcurse - text-based organizer
  *
- * Copyright (c) 2004-2015 calcurse Development Team <misc@calcurse.org>
+ * Copyright (c) 2004-2016 calcurse Development Team <misc@calcurse.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -41,6 +41,7 @@
 #include <time.h>
 
 #include "calcurse.h"
+#include "sha1.h"
 
 llist_ts_t recur_alist_p;
 llist_t recur_elist;
@@ -180,15 +181,28 @@ void recur_event_llist_free(void)
 }
 
 static int
-recur_apoint_cmp_start(struct recur_apoint *a, struct recur_apoint *b)
+recur_apoint_cmp(struct recur_apoint *a, struct recur_apoint *b)
 {
-	return a->start < b->start ? -1 : (a->start == b->start ? 0 : 1);
+	if (a->start < b->start)
+		return -1;
+	if (a->start > b->start)
+		return 1;
+	if ((a->state & APOINT_NOTIFY) && !(b->state & APOINT_NOTIFY))
+		return -1;
+	if (!(a->state & APOINT_NOTIFY) && (b->state & APOINT_NOTIFY))
+		return 1;
+
+	return strcmp(a->mesg, b->mesg);
 }
 
-static int recur_event_cmp_day(struct recur_event *a,
-			       struct recur_event *b)
+static int recur_event_cmp(struct recur_event *a, struct recur_event *b)
 {
-	return a->day < b->day ? -1 : (a->day == b->day ? 0 : 1);
+	if (a->day < b->day)
+		return -1;
+	if (a->day > b->day)
+		return 1;
+
+	return strcmp(a->mesg, b->mesg);
 }
 
 /* Insert a new recursive appointment in the general linked list */
@@ -217,7 +231,7 @@ struct recur_apoint *recur_apoint_new(char *mesg, char *note, long start,
 	}
 
 	LLIST_TS_LOCK(&recur_alist_p);
-	LLIST_TS_ADD_SORTED(&recur_alist_p, rapt, recur_apoint_cmp_start);
+	LLIST_TS_ADD_SORTED(&recur_alist_p, rapt, recur_apoint_cmp);
 	LLIST_TS_UNLOCK(&recur_alist_p);
 
 	return rapt;
@@ -245,7 +259,7 @@ struct recur_event *recur_event_new(char *mesg, char *note, long day,
 		LLIST_INIT(&rev->exc);
 	}
 
-	LLIST_ADD_SORTED(&recur_elist, rev, recur_event_cmp_day);
+	LLIST_ADD_SORTED(&recur_elist, rev, recur_event_cmp);
 
 	return rev;
 }
@@ -308,7 +322,7 @@ int recur_char2def(char type)
 }
 
 /* Write days for which recurrent items should not be repeated. */
-static void recur_write_exc(llist_t * lexc, FILE * f)
+static void recur_exc_append(struct string *s, llist_t *lexc)
 {
 	llist_item_t *i;
 	struct tm lt;
@@ -322,7 +336,7 @@ static void recur_write_exc(llist_t * lexc, FILE * f)
 		st_mon = lt.tm_mon + 1;
 		st_day = lt.tm_mday;
 		st_year = lt.tm_year + 1900;
-		fprintf(f, " !%02u/%02u/%04u", st_mon, st_day, st_year);
+		string_catf(s, " !%02u/%02u/%04u", st_mon, st_day, st_year);
 	}
 }
 
@@ -335,6 +349,7 @@ struct recur_apoint *recur_apoint_scan(FILE * f, struct tm start,
 {
 	char buf[BUFSIZ], *nl;
 	time_t tstart, tend, tuntil;
+	struct recur_apoint *rapt;
 
 	EXIT_IF(!check_date(start.tm_year, start.tm_mon, start.tm_mday) ||
 		!check_date(end.tm_year, end.tm_mon, end.tm_mday) ||
@@ -392,8 +407,20 @@ struct recur_apoint *recur_apoint_scan(FILE * f, struct tm start,
 			return NULL;
 	}
 
-	return recur_apoint_new(buf, note, tstart, tend - tstart, state,
+	rapt = recur_apoint_new(buf, note, tstart, tend - tstart, state,
 				recur_char2def(type), freq, tuntil, exc);
+
+	/* Filter by hash. */
+	if (filter && filter->hash) {
+		char *hash = recur_apoint_hash(rapt);
+		if (!hash_matches(filter->hash, hash)) {
+			recur_apoint_erase(rapt);
+			rapt = NULL;
+		}
+		mem_free(hash);
+	}
+
+	return rapt;
 }
 
 /* Load the recursive events from file */
@@ -403,7 +430,8 @@ struct recur_event *recur_event_scan(FILE * f, struct tm start, int id,
 				     struct item_filter *filter)
 {
 	char buf[BUFSIZ], *nl;
-	time_t tstart, tuntil;
+	time_t tstart, tend, tuntil;
+	struct recur_event *rev;
 
 	EXIT_IF(!check_date(start.tm_year, start.tm_mon, start.tm_mday) ||
 		!check_time(start.tm_hour, start.tm_min) ||
@@ -434,6 +462,7 @@ struct recur_event *recur_event_scan(FILE * f, struct tm start, int id,
 	}
 	tstart = mktime(&start);
 	EXIT_IF(tstart == -1 || tuntil == -1, _("date error in event"));
+	tend = tstart + DAYINSEC - 1;
 
 	/* Filter item. */
 	if (filter) {
@@ -445,65 +474,106 @@ struct recur_event *recur_event_scan(FILE * f, struct tm start, int id,
 			return NULL;
 		if (filter->start_to >= 0 && tstart > filter->start_to)
 			return NULL;
+		if (filter->end_from >= 0 && tend < filter->end_from)
+			return NULL;
+		if (filter->end_to >= 0 && tend > filter->end_to)
+			return NULL;
 	}
 
-	return recur_event_new(buf, note, tstart, id, recur_char2def(type),
-			       freq, tuntil, exc);
+	rev = recur_event_new(buf, note, tstart, id, recur_char2def(type),
+			      freq, tuntil, exc);
+
+	/* Filter by hash. */
+	if (filter && filter->hash) {
+		char *hash = recur_event_hash(rev);
+		if (!hash_matches(filter->hash, hash)) {
+			recur_event_erase(rev);
+			rev = NULL;
+		}
+		mem_free(hash);
+	}
+
+	return rev;
 }
 
-/* Writting of a recursive appointment into file. */
-void recur_apoint_write(struct recur_apoint *o, FILE * f)
+char *recur_apoint_tostr(struct recur_apoint *o)
 {
+	struct string s;
 	struct tm lt;
 	time_t t;
 
+	string_init(&s);
+
 	t = o->start;
 	localtime_r(&t, &lt);
-	fprintf(f, "%02u/%02u/%04u @ %02u:%02u", lt.tm_mon + 1, lt.tm_mday,
-		1900 + lt.tm_year, lt.tm_hour, lt.tm_min);
+	string_catf(&s, "%02u/%02u/%04u @ %02u:%02u", lt.tm_mon + 1,
+		lt.tm_mday, 1900 + lt.tm_year, lt.tm_hour, lt.tm_min);
 
 	t = o->start + o->dur;
 	localtime_r(&t, &lt);
-	fprintf(f, " -> %02u/%02u/%04u @ %02u:%02u", lt.tm_mon + 1,
+	string_catf(&s, " -> %02u/%02u/%04u @ %02u:%02u", lt.tm_mon + 1,
 		lt.tm_mday, 1900 + lt.tm_year, lt.tm_hour, lt.tm_min);
 
 	t = o->rpt->until;
-	if (t == 0) {		/* We have an endless recurrent appointment. */
-		fprintf(f, " {%d%c", o->rpt->freq,
+	if (t == 0) {
+		/* We have an endless recurrent appointment. */
+		string_catf(&s, " {%d%c", o->rpt->freq,
 			recur_def2char(o->rpt->type));
 	} else {
 		localtime_r(&t, &lt);
-		fprintf(f, " {%d%c -> %02u/%02u/%04u", o->rpt->freq,
+		string_catf(&s, " {%d%c -> %02u/%02u/%04u", o->rpt->freq,
 			recur_def2char(o->rpt->type), lt.tm_mon + 1,
 			lt.tm_mday, 1900 + lt.tm_year);
 	}
-	recur_write_exc(&o->exc, f);
-	fputs("} ", f);
-	if (o->note != NULL)
-		fprintf(f, ">%s ", o->note);
+	recur_exc_append(&s, &o->exc);
+	string_catf(&s, "} ");
+	if (o->note)
+		string_catf(&s, ">%s ", o->note);
 	if (o->state & APOINT_NOTIFY)
-		fputc('!', f);
+		string_catf(&s, "%c", '!');
 	else
-		fputc('|', f);
-	fprintf(f, "%s\n", o->mesg);
+		string_catf(&s, "%c", '|');
+	string_catf(&s, "%s", o->mesg);
+
+	return string_buf(&s);
 }
 
-/* Writting of a recursive event into file. */
-void recur_event_write(struct recur_event *o, FILE * f)
+char *recur_apoint_hash(struct recur_apoint *rapt)
 {
+	char *raw = recur_apoint_tostr(rapt);
+	char *sha1 = mem_malloc(SHA1_DIGESTLEN * 2 + 1);
+	sha1_digest(raw, sha1);
+	mem_free(raw);
+
+	return sha1;
+}
+
+void recur_apoint_write(struct recur_apoint *o, FILE * f)
+{
+	char *str = recur_apoint_tostr(o);
+	fprintf(f, "%s\n", str);
+	mem_free(str);
+}
+
+char *recur_event_tostr(struct recur_event *o)
+{
+	struct string s;
 	struct tm lt;
 	time_t t;
 	int st_mon, st_day, st_year;
 	int end_mon, end_day, end_year;
 
+	string_init(&s);
+
 	t = o->day;
 	localtime_r(&t, &lt);
 	st_mon = lt.tm_mon + 1;
 	st_day = lt.tm_mday;
 	st_year = lt.tm_year + 1900;
 	t = o->rpt->until;
-	if (t == 0) {		/* We have an endless recurrent event. */
-		fprintf(f, "%02u/%02u/%04u [%d] {%d%c", st_mon, st_day,
+	if (t == 0) {
+		/* We have an endless recurrent event. */
+		string_catf(&s, "%02u/%02u/%04u [%d] {%d%c", st_mon, st_day,
 			st_year, o->id, o->rpt->freq,
 			recur_def2char(o->rpt->type));
 	} else {
@@ -511,16 +581,35 @@ void recur_event_write(struct recur_event *o, FILE * f)
 		end_mon = lt.tm_mon + 1;
 		end_day = lt.tm_mday;
 		end_year = lt.tm_year + 1900;
-		fprintf(f, "%02u/%02u/%04u [%d] {%d%c -> %02u/%02u/%04u",
+		string_catf(&s, "%02u/%02u/%04u [%d] {%d%c -> %02u/%02u/%04u",
 			st_mon, st_day, st_year, o->id, o->rpt->freq,
 			recur_def2char(o->rpt->type), end_mon, end_day,
 			end_year);
 	}
-	recur_write_exc(&o->exc, f);
-	fputs("} ", f);
-	if (o->note != NULL)
-		fprintf(f, ">%s ", o->note);
-	fprintf(f, "%s\n", o->mesg);
+	recur_exc_append(&s, &o->exc);
+	string_catf(&s, "} ");
+	if (o->note)
+		string_catf(&s, ">%s ", o->note);
+	string_catf(&s, "%s", o->mesg);
+
+	return string_buf(&s);
+}
+
+char *recur_event_hash(struct recur_event *rev)
+{
+	char *raw = recur_event_tostr(rev);
+	char *sha1 = mem_malloc(SHA1_DIGESTLEN * 2 + 1);
+	sha1_digest(raw, sha1);
+	mem_free(raw);
+
+	return sha1;
+}
+
+void recur_event_write(struct recur_event *o, FILE * f)
+{
+	char *str = recur_event_tostr(o);
+	fprintf(f, "%s\n", str);
+	mem_free(str);
 }
 
 /* Write recursive items to file. */
@@ -594,7 +683,7 @@ static long diff_years(struct tm lt_start, struct tm lt_end)
 
 static int exc_inday(struct excp *exc, long *day_start)
 {
-	return (exc->st >= *day_start && exc->st < *day_start + DAYINSEC);
+	return (date_cmp_day(exc->st, *day_start) == 0);
 }
 
 /*
@@ -610,14 +699,14 @@ unsigned
 recur_item_find_occurrence(long item_start, long item_dur,
 			   llist_t * item_exc, int rpt_type, int rpt_freq,
 			   long rpt_until, long day_start,
-			   unsigned *occurrence)
+			   time_t *occurrence)
 {
 	struct date start_date;
 	long diff, span;
 	struct tm lt_day, lt_item, lt_item_day;
 	time_t t;
 
-	if (day_start < item_start - DAYINSEC + 1)
+	if (date_cmp_day(day_start, item_start) < 0)
 		return 0;
 
 	if (rpt_until != 0 && day_start >= rpt_until + item_dur)
@@ -696,7 +785,7 @@ recur_item_find_occurrence(long item_start, long item_dur,
 
 unsigned
 recur_apoint_find_occurrence(struct recur_apoint *rapt, long day_start,
-			     unsigned *occurrence)
+			     time_t *occurrence)
 {
 	return recur_item_find_occurrence(rapt->start, rapt->dur,
 					  &rapt->exc, rapt->rpt->type,
@@ -707,7 +796,7 @@ recur_apoint_find_occurrence(struct recur_apoint *rapt, long day_start,
 
 unsigned
 recur_event_find_occurrence(struct recur_event *rev, long day_start,
-			    unsigned *occurrence)
+			    time_t *occurrence)
 {
 	return recur_item_find_occurrence(rev->day, DAYINSEC, &rev->exc,
 					  rev->rpt->type, rev->rpt->freq,
@@ -841,7 +930,7 @@ struct notify_app *recur_apoint_check_next(struct notify_app *app,
 					   long start, long day)
 {
 	llist_item_t *i;
-	unsigned real_recur_start_time;
+	time_t real_recur_start_time;
 
 	LLIST_TS_LOCK(&recur_alist_p);
 	LLIST_TS_FIND_FOREACH(&recur_alist_p, &app->time,
@@ -890,7 +979,7 @@ void recur_event_paste_item(struct recur_event *rev, long date)
 		exc->st += time_shift;
 	}
 
-	LLIST_ADD_SORTED(&recur_elist, rev, recur_event_cmp_day);
+	LLIST_ADD_SORTED(&recur_elist, rev, recur_event_cmp);
 }
 
 void recur_apoint_paste_item(struct recur_apoint *rapt, long date)
@@ -910,7 +999,7 @@ void recur_apoint_paste_item(struct recur_apoint *rapt, long date)
 	}
 
 	LLIST_TS_LOCK(&recur_alist_p);
-	LLIST_TS_ADD_SORTED(&recur_alist_p, rapt, recur_apoint_cmp_start);
+	LLIST_TS_ADD_SORTED(&recur_alist_p, rapt, recur_apoint_cmp);
 	LLIST_TS_UNLOCK(&recur_alist_p);
 
 	if (notify_bar())
