@@ -1,7 +1,7 @@
 /*
  * Calcurse - text-based organizer
  *
- * Copyright (c) 2004-2015 calcurse Development Team <misc@calcurse.org>
+ * Copyright (c) 2004-2016 calcurse Development Team <misc@calcurse.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -39,53 +39,96 @@
 #include <unistd.h>
 
 #include "calcurse.h"
+#include "sha1.h"
 
 llist_t todolist;
 
-/* Returns a structure containing the selected item. */
-struct todo *todo_get_item(int item_number)
+static int todo_is_uncompleted(struct todo *todo, void *cbdata)
 {
-	return LLIST_GET_DATA(LLIST_NTH(&todolist, item_number));
+	return !todo->completed;
 }
 
-static int todo_cmp_id(struct todo *a, struct todo *b)
+/* Returns a structure containing the selected item. */
+struct todo *todo_get_item(int item_number, int skip_completed)
 {
-	/*
-	 * As of version 2.6, todo items can have a negative id, which means they
-	 * were completed. To keep them sorted, we need to consider the absolute id
-	 * value.
-	 */
-	int abs_a = abs(a->id);
-	int abs_b = abs(b->id);
+	llist_item_t *i;
 
-	return abs_a < abs_b ? -1 : (abs_a == abs_b ? 0 : 1);
+	if (skip_completed)
+		i = LLIST_FIND_NTH(&todolist, item_number, NULL,
+				   todo_is_uncompleted);
+	else
+		i = LLIST_NTH(&todolist, item_number);
+
+	if (!i)
+		return NULL;
+
+	return LLIST_GET_DATA(i);
+}
+
+static int todo_cmp(struct todo *a, struct todo *b)
+{
+	if (a->completed && !b->completed)
+		return 1;
+	if (b->completed && !a->completed)
+		return -1;
+	if (a->id > 0 && b->id == 0)
+		return -1;
+	if (b->id > 0 && a->id == 0)
+		return 1;
+	if (a->id == b->id)
+		return strcmp(a->mesg, b->mesg);
+
+	return a->id - b->id;
 }
 
 /*
  * Add an item in the todo linked list.
  */
-struct todo *todo_add(char *mesg, int id, char *note)
+struct todo *todo_add(char *mesg, int id, int completed, char *note)
 {
 	struct todo *todo;
 
 	todo = mem_malloc(sizeof(struct todo));
 	todo->mesg = mem_strdup(mesg);
 	todo->id = id;
+	todo->completed = completed;
 	todo->note = (note != NULL
 		      && note[0] != '\0') ? mem_strdup(note) : NULL;
 
-	LLIST_ADD_SORTED(&todolist, todo, todo_cmp_id);
+	LLIST_ADD_SORTED(&todolist, todo, todo_cmp);
 
 	return todo;
 }
 
+char *todo_tostr(struct todo *todo)
+{
+	char *res;
+	const char *cstr = todo->completed ? "-" : "";
+
+	if (todo->note)
+		asprintf(&res, "[%s%d]>%s %s", cstr, todo->id, todo->note,
+			 todo->mesg);
+	else
+		asprintf(&res, "[%s%d] %s", cstr, todo->id, todo->mesg);
+
+	return res;
+}
+
+char *todo_hash(struct todo *todo)
+{
+	char *raw = todo_tostr(todo);
+	char *sha1 = mem_malloc(SHA1_DIGESTLEN * 2 + 1);
+	sha1_digest(raw, sha1);
+	mem_free(raw);
+
+	return sha1;
+}
+
 void todo_write(struct todo *todo, FILE * f)
 {
-	if (todo->note)
-		fprintf(f, "[%d]>%s %s\n", todo->id, todo->note,
-			todo->mesg);
-	else
-		fprintf(f, "[%d] %s\n", todo->id, todo->mesg);
+	char *str = todo_tostr(todo);
+	fprintf(f, "%s\n", str);
+	mem_free(str);
 }
 
 /* Delete a note previously attached to a todo item. */
@@ -111,33 +154,46 @@ void todo_delete(struct todo *todo)
 }
 
 /*
- * Flag a todo item (for now on, only the 'completed' state is available).
- * Internally, a completed item keeps its priority, but it becomes negative.
- * This way, it is easy to retrive its original priority if the user decides
- * that in fact it was not completed.
+ * Make sure an item is located at the right position within the sorted list.
  */
+void todo_resort(struct todo *t)
+{
+	llist_item_t *i = LLIST_FIND_FIRST(&todolist, t, NULL);
+	LLIST_REMOVE(&todolist, i);
+	LLIST_ADD_SORTED(&todolist, t, todo_cmp);
+}
+
+/* Flag a todo item. */
 void todo_flag(struct todo *t)
 {
-	t->id = -t->id;
+	t->completed = !t->completed;
+	todo_resort(t);
 }
 
 /*
  * Returns the position into the linked list corresponding to the
  * given todo item.
  */
-int todo_get_position(struct todo *needle)
+int todo_get_position(struct todo *needle, int skip_completed)
 {
 	llist_item_t *i;
 	int n = 0;
 
-	LLIST_FOREACH(&todolist, i) {
-		if (LLIST_TS_GET_DATA(i) == needle)
-			return n;
-		n++;
+	if (skip_completed) {
+		LLIST_FIND_FOREACH(&todolist, NULL, todo_is_uncompleted, i) {
+			if (LLIST_TS_GET_DATA(i) == needle)
+				return n;
+			n++;
+		}
+	} else {
+		LLIST_FOREACH(&todolist, i) {
+			if (LLIST_TS_GET_DATA(i) == needle)
+				return n;
+			n++;
+		}
 	}
 
-	EXIT(_("todo not found"));
-	return -1;		/* avoid compiler warnings */
+	return -1;
 }
 
 /* Attach a note to a todo */
