@@ -1,7 +1,7 @@
 /*
  * Calcurse - text-based organizer
  *
- * Copyright (c) 2004-2015 calcurse Development Team <misc@calcurse.org>
+ * Copyright (c) 2004-2016 calcurse Development Team <misc@calcurse.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -87,6 +87,9 @@ void ui_calendar_view_next(void)
 	ui_calendar_view++;
 	if (ui_calendar_view == CAL_VIEWS)
 		ui_calendar_view = 0;
+
+	/* The calendar panel needs to be erased when switching views. */
+	monthly_view_cache_month = 0;
 }
 
 void ui_calendar_view_prev(void)
@@ -94,6 +97,9 @@ void ui_calendar_view_prev(void)
 	if (ui_calendar_view == 0)
 		ui_calendar_view = CAL_VIEWS;
 	ui_calendar_view--;
+
+	/* The calendar panel needs to be erased when switching views. */
+	monthly_view_cache_month = 0;
 }
 
 void ui_calendar_set_view(int view)
@@ -208,7 +214,7 @@ struct date *ui_calendar_get_slctd_day(void)
 }
 
 /* Returned value represents the selected day in calendar (in seconds) */
-long ui_calendar_get_slctd_day_sec(void)
+time_t ui_calendar_get_slctd_day_sec(void)
 {
 	return date2sec(slctd_day, 0, 0);
 }
@@ -250,148 +256,11 @@ static long ymd_to_scalar(unsigned year, unsigned month, unsigned day)
 	return scalar;
 }
 
-/*
- * Used to change date by adding a certain amount of days or weeks.
- * Returns 0 on success, 1 otherwise.
- */
-static int date_change(struct tm *date, int delta_month, int delta_day)
-{
-	struct tm t;
-
-	t = *date;
-	t.tm_mon += delta_month;
-	t.tm_mday += delta_day;
-
-	if (mktime(&t) == -1) {
-		return 1;
-	} else {
-		*date = t;
-		return 0;
-	}
-}
-
 void ui_calendar_monthly_view_cache_set_invalid(void)
 {
 	monthly_view_cache_valid = 0;
 }
 
-/* Draw the monthly view inside calendar panel. */
-static void
-draw_monthly_view(struct scrollwin *sw, struct date *current_day,
-		  unsigned sunday_first)
-{
-	struct date check_day;
-	int c_day, c_day_1, day_1_sav, numdays, j;
-	unsigned yr, mo;
-	int w, ofs_x, ofs_y;
-	int item_this_day = 0;
-
-	mo = slctd_day.mm;
-	yr = slctd_day.yyyy;
-
-	/* offset for centering calendar in window */
-	w = wins_sbar_width() - 2;
-	ofs_y = 0;
-	ofs_x = (w - 27) / 2;
-
-	/* checking the number of days in february */
-	numdays = days[mo - 1];
-	if (2 == mo && ISLEAP(yr))
-		++numdays;
-
-	/*
-	 * the first calendar day will be monday or sunday, depending on
-	 * 'week_begins_on_monday' value
-	 */
-	c_day_1 =
-	    (int)((ymd_to_scalar(yr, mo, 1 + sunday_first) -
-		   (long)1) % 7L);
-
-	/* Write the current month and year on top of the calendar */
-	WINS_CALENDAR_LOCK;
-	if (yr * YEARINMONTHS + mo != monthly_view_cache_month) {
-		/* erase the window if a new month is selected */
-		werase(sw_cal.inner);
-	}
-	custom_apply_attr(sw->inner, ATTR_HIGHEST);
-	mvwprintw(sw->inner, ofs_y,
-		  (w - (strlen(_(monthnames[mo - 1])) + 5)) / 2,
-		  "%s %d", _(monthnames[mo - 1]), slctd_day.yyyy);
-	custom_remove_attr(sw->inner, ATTR_HIGHEST);
-	++ofs_y;
-
-	/* print the days, with regards to the first day of the week */
-	custom_apply_attr(sw->inner, ATTR_HIGHEST);
-	for (j = 0; j < WEEKINDAYS; j++) {
-		mvwaddstr(sw->inner, ofs_y, ofs_x + 4 * j,
-			  _(daynames[1 + j - sunday_first]));
-	}
-	custom_remove_attr(sw->inner, ATTR_HIGHEST);
-	WINS_CALENDAR_UNLOCK;
-
-	day_1_sav = (c_day_1 + 1) * 3 + c_day_1 - 7;
-
-	/* invalidate cache if a new month is selected */
-	if (yr * YEARINMONTHS + mo != monthly_view_cache_month) {
-		monthly_view_cache_month = yr * YEARINMONTHS + mo;
-		monthly_view_cache_valid = 0;
-	}
-
-	for (c_day = 1; c_day <= numdays; ++c_day, ++c_day_1, c_day_1 %= 7) {
-		check_day.dd = c_day;
-		check_day.mm = slctd_day.mm;
-		check_day.yyyy = slctd_day.yyyy;
-
-		/* check if the day contains an event or an appointment */
-		if (monthly_view_cache_valid) {
-			item_this_day = monthly_view_cache[c_day - 1];
-		} else {
-			item_this_day = monthly_view_cache[c_day - 1] =
-			    day_check_if_item(check_day);
-		}
-
-		/* Go to next line, the week is over. */
-		if (!c_day_1 && 1 != c_day) {
-			ofs_y++;
-			ofs_x = (w - 27) / 2 - day_1_sav - 4 * c_day;
-		}
-
-		WINS_CALENDAR_LOCK;
-		if (c_day == current_day->dd
-		    && current_day->mm == slctd_day.mm
-		    && current_day->yyyy == slctd_day.yyyy
-		    && current_day->dd != slctd_day.dd) {
-			/* This is today, so print it in yellow. */
-			custom_apply_attr(sw->inner, ATTR_LOWEST);
-			mvwprintw(sw->inner, ofs_y + 1,
-				  ofs_x + day_1_sav + 4 * c_day + 1, "%2d",
-				  c_day);
-			custom_remove_attr(sw->inner, ATTR_LOWEST);
-		} else if (c_day == slctd_day.dd) {
-			/* This is the selected day, print it according to user's theme. */
-			custom_apply_attr(sw->inner, ATTR_HIGHEST);
-			mvwprintw(sw->inner, ofs_y + 1,
-				  ofs_x + day_1_sav + 4 * c_day + 1, "%2d",
-				  c_day);
-			custom_remove_attr(sw->inner, ATTR_HIGHEST);
-		} else if (item_this_day) {
-			custom_apply_attr(sw->inner, ATTR_LOW);
-			mvwprintw(sw->inner, ofs_y + 1,
-				  ofs_x + day_1_sav + 4 * c_day + 1, "%2d",
-				  c_day);
-			custom_remove_attr(sw->inner, ATTR_LOW);
-		} else {
-			/* otherwise, print normal days in black */
-			mvwprintw(sw->inner, ofs_y + 1,
-				  ofs_x + day_1_sav + 4 * c_day + 1, "%2d",
-				  c_day);
-		}
-		WINS_CALENDAR_UNLOCK;
-	}
-
-	monthly_view_cache_valid = 1;
-}
-
 static int weeknum(const struct tm *t, int firstweekday)
 {
 	int wday, wnum;
@@ -465,6 +334,154 @@ static int ISO8601weeknum(const struct tm *t)
 	return wnum;
 }
 
+static struct tm get_first_weekday(unsigned sunday_first)
+{
+	int c_wday, days_to_remove;
+	struct tm t;
+
+	c_wday = ui_calendar_get_wday(&slctd_day);
+	if (sunday_first)
+		days_to_remove = c_wday;
+	else
+		days_to_remove = c_wday == 0 ? WEEKINDAYS - 1 : c_wday - 1;
+
+	t = date2tm(slctd_day, 0, 0);
+	date_change(&t, 0, -days_to_remove);
+
+	return t;
+}
+
+static void draw_week_number(struct scrollwin *sw, struct tm t)
+{
+	int weeknum = ISO8601weeknum(&t);
+
+	WINS_CALENDAR_LOCK;
+	werase(sw_cal.inner);
+	custom_apply_attr(sw->inner, ATTR_HIGHEST);
+	if (wins_slctd() == CAL)
+		wattron(sw->win, COLOR_PAIR(COLR_CUSTOM));
+	mvwprintw(sw->win, conf.compact_panels ? 0 : 2, sw->w - 9,
+		  "(# %02d)", weeknum);
+	if (wins_slctd() == CAL)
+		wattroff(sw->win, COLOR_PAIR(COLR_CUSTOM));
+	custom_remove_attr(sw->inner, ATTR_HIGHEST);
+	WINS_CALENDAR_UNLOCK;
+}
+
+/* Draw the monthly view inside calendar panel. */
+static void
+draw_monthly_view(struct scrollwin *sw, struct date *current_day,
+		  unsigned sunday_first)
+{
+	struct date check_day;
+	int c_day, c_day_1, day_1_sav, numdays, j;
+	unsigned yr, mo;
+	int w, ofs_x, ofs_y;
+	int item_this_day = 0;
+	struct tm t = get_first_weekday(sunday_first);
+
+	mo = slctd_day.mm;
+	yr = slctd_day.yyyy;
+
+	/* offset for centering calendar in window */
+	w = wins_sbar_width() - 2;
+	ofs_y = 0;
+	ofs_x = (w - 27) / 2;
+
+	/* checking the number of days in february */
+	numdays = days[mo - 1];
+	if (2 == mo && ISLEAP(yr))
+		++numdays;
+
+	/*
+	 * the first calendar day will be monday or sunday, depending on
+	 * 'week_begins_on_monday' value
+	 */
+	c_day_1 =
+	    (int)((ymd_to_scalar(yr, mo, 1 + sunday_first) -
+		   (long)1) % 7L);
+
+	/* Print the week number. */
+	draw_week_number(sw, t);
+
+	/* Write the current month and year on top of the calendar */
+	WINS_CALENDAR_LOCK;
+	if (yr * YEARINMONTHS + mo != monthly_view_cache_month) {
+		/* erase the window if a new month is selected */
+		werase(sw_cal.inner);
+	}
+	custom_apply_attr(sw->inner, ATTR_HIGHEST);
+	mvwprintw(sw->inner, ofs_y,
+		  (w - (strlen(_(monthnames[mo - 1])) + 5)) / 2,
+		  "%s %d", _(monthnames[mo - 1]), slctd_day.yyyy);
+	custom_remove_attr(sw->inner, ATTR_HIGHEST);
+	++ofs_y;
+
+	/* print the days, with regards to the first day of the week */
+	custom_apply_attr(sw->inner, ATTR_HIGHEST);
+	for (j = 0; j < WEEKINDAYS; j++) {
+		mvwaddstr(sw->inner, ofs_y, ofs_x + 4 * j,
+			  _(daynames[1 + j - sunday_first]));
+	}
+	custom_remove_attr(sw->inner, ATTR_HIGHEST);
+	WINS_CALENDAR_UNLOCK;
+
+	day_1_sav = (c_day_1 + 1) * 3 + c_day_1 - 7;
+
+	/* invalidate cache if a new month is selected */
+	if (yr * YEARINMONTHS + mo != monthly_view_cache_month) {
+		monthly_view_cache_month = yr * YEARINMONTHS + mo;
+		monthly_view_cache_valid = 0;
+	}
+
+	for (c_day = 1; c_day <= numdays; ++c_day, ++c_day_1, c_day_1 %= 7) {
+		unsigned attr;
+
+		check_day.dd = c_day;
+		check_day.mm = slctd_day.mm;
+		check_day.yyyy = slctd_day.yyyy;
+
+		/* check if the day contains an event or an appointment */
+		if (monthly_view_cache_valid) {
+			item_this_day = monthly_view_cache[c_day - 1];
+		} else {
+			item_this_day = monthly_view_cache[c_day - 1] =
+			    day_check_if_item(check_day);
+		}
+
+		/* Go to next line, the week is over. */
+		if (!c_day_1 && 1 != c_day) {
+			ofs_y++;
+			ofs_x = (w - 27) / 2 - day_1_sav - 4 * c_day;
+		}
+
+		if (c_day == current_day->dd
+		    && current_day->mm == slctd_day.mm
+		    && current_day->yyyy == slctd_day.yyyy
+		    && current_day->dd != slctd_day.dd)
+			attr = ATTR_LOWEST;
+		else if (c_day == slctd_day.dd)
+			attr = ATTR_HIGHEST;
+		else if (item_this_day == 1)
+			attr = ATTR_LOW;
+		else if (item_this_day == 2)
+			attr = ATTR_TRUE;
+		else
+			attr = 0;
+
+		WINS_CALENDAR_LOCK;
+		if (attr)
+			custom_apply_attr(sw->inner, attr);
+		mvwprintw(sw->inner, ofs_y + 1,
+			  ofs_x + day_1_sav + 4 * c_day + 1, "%2d", c_day);
+		if (attr)
+			custom_remove_attr(sw->inner, attr);
+		WINS_CALENDAR_UNLOCK;
+	}
+
+	monthly_view_cache_valid = 1;
+}
+
 /* Draw the weekly view inside calendar panel. */
 static void
 draw_weekly_view(struct scrollwin *sw, struct date *current_day,
@@ -472,35 +489,14 @@ draw_weekly_view(struct scrollwin *sw, struct date *current_day,
 {
 #define DAYSLICESNO  6
 	const int WCALWIDTH = 28;
-	struct tm t;
-	int OFFY, OFFX, j, c_wday, days_to_remove, weeknum;
+	struct tm t = get_first_weekday(sunday_first);
+	int OFFY, OFFX, j;
 
 	OFFY = 0;
 	OFFX = (wins_sbar_width() - 2 - WCALWIDTH) / 2;
 
-	/* Fill in a tm structure with the first day of the selected week. */
-	c_wday = ui_calendar_get_wday(&slctd_day);
-	if (sunday_first)
-		days_to_remove = c_wday;
-	else
-		days_to_remove = c_wday == 0 ? WEEKINDAYS - 1 : c_wday - 1;
-
-	memset(&t, 0, sizeof(struct tm));
-	t.tm_mday = slctd_day.dd;
-	t.tm_mon = slctd_day.mm - 1;
-	t.tm_year = slctd_day.yyyy - 1900;
-	mktime(&t);
-	date_change(&t, 0, -days_to_remove);
-
 	/* Print the week number. */
-	weeknum = ISO8601weeknum(&t);
-	WINS_CALENDAR_LOCK;
-	werase(sw_cal.inner);
-	custom_apply_attr(sw->inner, ATTR_HIGHEST);
-	mvwprintw(sw->win, conf.compact_panels ? 0 : 2, sw->w - 9,
-		  "(# %02d)", weeknum);
-	custom_remove_attr(sw->inner, ATTR_HIGHEST);
-	WINS_CALENDAR_UNLOCK;
+	draw_week_number(sw, t);
 
 	/* Now draw calendar view. */
 	for (j = 0; j < WEEKINDAYS; j++) {
@@ -525,11 +521,13 @@ draw_weekly_view(struct scrollwin *sw, struct date *current_day,
 		    && current_day->mm == slctd_day.mm
 		    && current_day->yyyy == slctd_day.yyyy
 		    && current_day->dd != slctd_day.dd)
-			attr = ATTR_LOWEST;	/* today, but not selected */
+			attr = ATTR_LOWEST;
 		else if (t.tm_mday == slctd_day.dd)
-			attr = ATTR_HIGHEST;	/* selected day */
-		else if (item_this_day)
+			attr = ATTR_HIGHEST;
+		else if (item_this_day == 1)
 			attr = ATTR_LOW;
+		else if (item_this_day == 2)
+			attr = ATTR_TRUE;
 		else
 			attr = 0;
 
