@@ -1,7 +1,7 @@
 /*
  * Calcurse - text-based organizer
  *
- * Copyright (c) 2004-2015 calcurse Development Team <misc@calcurse.org>
+ * Copyright (c) 2004-2016 calcurse Development Team <misc@calcurse.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -38,29 +38,45 @@
 
 struct day_item day_cut[38] = { {0, 0, {NULL}} };
 
-/* Request the user to enter a new time. */
-static int day_edit_time(int time, unsigned *new_hour,
-			 unsigned *new_minute)
+struct day_item *ui_day_selitem(void)
 {
-	char *timestr = date_sec2date_str(time, "%H:%M");
+	if (day_item_count(0) <= 0)
+		return NULL;
+
+	return day_get_item(listbox_get_sel(&lb_apt));
+}
+
+void ui_day_set_selitem_by_aptev_ptr(union aptev_ptr p)
+{
+	int n = day_get_position_by_aptev_ptr(p);
+	if (n >= 0)
+		listbox_set_sel(&lb_apt, n);
+}
+
+void ui_day_set_selitem(struct day_item *day)
+{
+	ui_day_set_selitem_by_aptev_ptr(day->item);
+}
+
+/* Request the user to enter a new time. */
+static int day_edit_time(int time)
+{
+	char *input = date_sec2date_str(time, "%H:%M");
 	const char *msg_time =
-	    _("Enter start time ([hh:mm] or [hhmm]):");
+	    _("Enter start time ([hh:mm] or [hhmm]) or date:");
 	const char *enter_str = _("Press [Enter] to continue");
 	const char *fmt_msg =
 	    _("You entered an invalid time, should be [hh:mm] or [hhmm]");
+	long ts = time;
 
 	for (;;) {
 		status_mesg(msg_time, "");
-		if (updatestring(win[STA].p, &timestr, 0, 1) !=
-		    GETSTRING_VALID)
+		if (updatestring(win[STA].p, &input, 0, 1) != GETSTRING_VALID)
 			return 0;
-		if (parse_time(timestr, new_hour, new_minute) == 1) {
-			mem_free(timestr);
-			return 1;
-		} else {
-			status_mesg(fmt_msg, enter_str);
-			wgetch(win[KEY].p);
-		}
+		if (parse_datetime(input, &ts))
+			return ts;
+		status_mesg(fmt_msg, enter_str);
+		wgetch(win[KEY].p);
 	}
 }
 
@@ -73,8 +89,7 @@ static int day_edit_duration(int start, int dur, unsigned *new_duration)
 	const char *enter_str = _("Press [Enter] to continue");
 	const char *fmt_msg =
 	    _("You entered an invalid time, should be [hh:mm] or [hhmm]");
-	long newtime;
-	unsigned hr, mn;
+	long end;
 
 	for (;;) {
 		int ret;
@@ -86,21 +101,28 @@ static int day_edit_duration(int start, int dur, unsigned *new_duration)
 		} else if (ret == GETSTRING_RET) {
 			*new_duration = 0;
 			break;
-		} else if (*timestr == '+'
-		    && parse_duration(timestr + 1, new_duration) == 1) {
-			*new_duration *= MININSEC;
-			break;
-		} else if (parse_time(timestr, &hr, &mn) == 1) {
-			newtime = update_time_in_date(start + dur, hr, mn);
-			*new_duration =
-			    (newtime >
-			     start) ? newtime - start : DAYINSEC +
-			    newtime - start;
-			break;
-		} else {
-			status_mesg(fmt_msg, enter_str);
-			wgetch(win[KEY].p);
 		}
+		if (*timestr == '+') {
+			if (parse_duration(timestr + 1, new_duration)) {
+				*new_duration *= MININSEC;
+				break;
+			}
+		}
+		end = start + dur;
+		ret = parse_datetime(timestr, &end);
+		/*
+		 * If the user enters a end time which is smaller than
+		 * the start time, assume that the time belongs to the
+		 * next day.
+		 */
+		if (!(ret & PARSE_DATETIME_HAS_DATE) && end < start)
+			end = date_sec_change(end, 0, 1);
+		if (ret) {
+			*new_duration = end - start;
+			break;
+		}
+		status_mesg(fmt_msg, enter_str);
+		wgetch(win[KEY].p);
 	}
 
 	mem_free(timestr);
@@ -111,20 +133,19 @@ static int day_edit_duration(int start, int dur, unsigned *new_duration)
 static void update_start_time(long *start, long *dur, int update_dur)
 {
 	long newtime;
-	unsigned hr, mn;
 	int valid_date;
 	const char *msg_wrong_time =
 	    _("Invalid time: start time must be before end time!");
 	const char *msg_enter = _("Press [Enter] to continue");
 
 	do {
-		if (!day_edit_time(*start, &hr, &mn))
+		newtime = day_edit_time(*start);
+		if (!newtime)
 			break;
 		if (!update_dur) {
-			*start = update_time_in_date(*start, hr, mn);
+			*start = newtime;
 			return;
 		}
-		newtime = update_time_in_date(*start, hr, mn);
 		if (newtime < *start + *dur) {
 			*dur -= (newtime - *start);
 			*start = newtime;
@@ -155,8 +176,8 @@ static void update_desc(char **desc)
 static void update_rept(struct rpt **rpt, const long start)
 {
 	int newtype, newfreq;
-	long newuntil;
-	char *freqstr = NULL, *timstr, *outstr;
+	time_t newuntil;
+	char *freqstr = NULL, *timstr = NULL, *outstr;
 	const char *msg_rpt_prefix = _("Enter the new repetition type:");
 	const char *msg_rpt_daily = _("(d)aily");
 	const char *msg_rpt_weekly = _("(w)eekly");
@@ -239,8 +260,10 @@ static void update_rept(struct rpt **rpt, const long start)
 		time_t t;
 		struct date new_date;
 		int newmonth, newday, newyear;
+		unsigned days;
 
-		asprintf(&outstr, _("Enter the new ending date: [%s] or '0'"),
+		asprintf(&outstr,
+			 _("Enter end date ([%s]), duration ([+xxwxxd]) or '0':"),
 			 DATEFMT_DESC(conf.input_datefmt));
 		status_mesg(outstr, "");
 		mem_free(outstr);
@@ -255,21 +278,38 @@ static void update_rept(struct rpt **rpt, const long start)
 			newuntil = 0;
 			break;
 		}
-		if (!parse_date
-		    (timstr, conf.input_datefmt, &newyear, &newmonth,
-		     &newday, ui_calendar_get_slctd_day())) {
-			asprintf(&outstr, msg_fmts,
-				 DATEFMT_DESC(conf.input_datefmt));
-			status_mesg(msg_wrong_date, outstr);
-			mem_free(outstr);
-			wgetch(win[KEY].p);
-			continue;
+		if (*timstr == '+') {
+			if (!parse_date_duration(timstr + 1, &days)) {
+				asprintf(&outstr, msg_fmts,
+					 DATEFMT_DESC(conf.input_datefmt));
+				status_mesg(msg_wrong_date, outstr);
+				mem_free(outstr);
+				wgetch(win[KEY].p);
+				continue;
+			}
+			t = start;
+			localtime_r(&t, &lt);
+			date_change(&lt, 0, days);
+			new_date.dd = lt.tm_mday;
+			new_date.mm = lt.tm_mon + 1;
+			new_date.yyyy = lt.tm_year + 1900;
+		} else {
+			if (!parse_date(timstr, conf.input_datefmt, &newyear,
+					&newmonth, &newday,
+					ui_calendar_get_slctd_day())) {
+				asprintf(&outstr, msg_fmts,
+					 DATEFMT_DESC(conf.input_datefmt));
+				status_mesg(msg_wrong_date, outstr);
+				mem_free(outstr);
+				wgetch(win[KEY].p);
+				continue;
+			}
+			t = start;
+			localtime_r(&t, &lt);
+			new_date.dd = newday;
+			new_date.mm = newmonth;
+			new_date.yyyy = newyear;
 		}
-		t = start;
-		localtime_r(&t, &lt);
-		new_date.dd = newday;
-		new_date.mm = newmonth;
-		new_date.yyyy = newyear;
 		newuntil = date2sec(new_date, lt.tm_hour, lt.tm_min);
 		if (newuntil >= start)
 			break;
@@ -300,7 +340,7 @@ void ui_day_item_edit(void)
 	if (day_item_count(0) <= 0)
 		return;
 
-	struct day_item *p = day_get_item(listbox_get_sel(&lb_apt));
+	struct day_item *p = ui_day_selitem();
 
 	switch (p->type) {
 	case RECUR_EVNT:
@@ -426,7 +466,7 @@ void ui_day_item_pipe(void)
 	if (day_item_count(0) <= 0)
 		return;
 
-	struct day_item *p = day_get_item(listbox_get_sel(&lb_apt));
+	struct day_item *p = ui_day_selitem();
 
 	status_mesg(_("Pipe item to external command:"), "");
 	if (getstring(win[STA].p, cmd, BUFSIZ, 0, 1) != GETSTRING_VALID)
@@ -466,7 +506,7 @@ void ui_day_item_pipe(void)
  */
 void ui_day_item_add(void)
 {
-#define LTIME 6
+#define LTIME 16
 #define LDUR 12
 	const char *mesg_1 =
 	    _("Enter start time ([hh:mm] or [hhmm]), leave blank for an all-day event:");
@@ -480,11 +520,11 @@ void ui_day_item_add(void)
 	const char *enter_str = _("Press [Enter] to continue");
 	char item_time[LDUR] = "";
 	char item_mesg[BUFSIZ] = "";
-	long apoint_start;
-	unsigned heures, minutes;
-	unsigned apoint_duration;
-	unsigned end_h, end_m;
+	time_t start = date2sec(*ui_calendar_get_slctd_day(), 0, 0), end;
+	unsigned dur;
 	int is_appointment = 1;
+	int ret;
+	union aptev_ptr item;
 
 	/* Get the starting time */
 	for (;;) {
@@ -496,7 +536,10 @@ void ui_day_item_add(void)
 			is_appointment = 0;
 			break;
 		}
-		if (parse_time(item_time, &heures, &minutes) == 1)
+		ret = parse_datetime(item_time, &start);
+		if (!(ret & PARSE_DATETIME_HAS_TIME))
+			is_appointment = 0;
+		if (ret)
 			break;
 		status_mesg(format_message_1, enter_str);
 		wgetch(win[KEY].p);
@@ -515,27 +558,26 @@ void ui_day_item_add(void)
 			    GETSTRING_ESC)
 				return;
 			if (strlen(item_time) == 0) {
-				apoint_duration = 0;
+				dur = 0;
 				break;
 			}
-			if (*item_time == '+'
-			    && parse_duration(item_time + 1,
-					      &apoint_duration) == 1)
-				break;
-			if (parse_time(item_time, &end_h, &end_m) == 1) {
-				if (end_h < heures
-				    || ((end_h == heures)
-					&& (end_m < minutes))) {
-					apoint_duration =
-					    MININSEC - minutes + end_m +
-					    (24 + end_h -
-					     (heures + 1)) * MININSEC;
-				} else {
-					apoint_duration =
-					    MININSEC - minutes + end_m +
-					    (end_h -
-					     (heures + 1)) * MININSEC;
+			if (*item_time == '+') {
+				if (parse_duration(item_time + 1, &dur)) {
+					dur *= MININSEC;
+					break;
 				}
+			}
+			end = start;
+			ret = parse_datetime(item_time, &end);
+			/*
+			 * If the user enters a end time which is smaller than
+			 * the start time, assume that the time belongs to the
+			 * next day.
+			 */
+			if (!(ret & PARSE_DATETIME_HAS_DATE) && end < start)
+				end = date_sec_change(end, 0, 1);
+			if (ret) {
+				dur = end - start;
 				break;
 			}
 			status_mesg(format_message_2, enter_str);
@@ -547,20 +589,16 @@ void ui_day_item_add(void)
 	if (getstring(win[STA].p, item_mesg, BUFSIZ, 0, 1) ==
 	    GETSTRING_VALID) {
 		if (is_appointment) {
-			apoint_start =
-			    date2sec(*ui_calendar_get_slctd_day(), heures,
-				     minutes);
-			apoint_new(item_mesg, 0L, apoint_start,
-				   min2sec(apoint_duration), 0L);
+			item.apt = apoint_new(item_mesg, 0L, start, dur, 0L);
 			if (notify_bar())
-				notify_check_added(item_mesg, apoint_start,
-						   0L);
+				notify_check_added(item_mesg, start, 0L);
 		} else {
-			event_new(item_mesg, 0L,
-				  date2sec(*ui_calendar_get_slctd_day(), 0,
-					   0), 1);
+			item.ev = event_new(item_mesg, 0L, start, 1);
 		}
 		io_set_modified();
+		day_process_storage(ui_calendar_get_slctd_day(), 0);
+		ui_day_load_items();
+		ui_day_set_selitem_by_aptev_ptr(item);
 	}
 
 	ui_calendar_monthly_view_cache_set_invalid();
@@ -590,7 +628,7 @@ void ui_day_item_delete(unsigned reg)
 	if (day_item_count(0) <= 0)
 		return;
 
-	struct day_item *p = day_get_item(listbox_get_sel(&lb_apt));
+	struct day_item *p = ui_day_selitem();
 
 	if (conf.confirm_delete) {
 		if (status_ask_bool(del_app_str) != 1) {
@@ -661,10 +699,10 @@ void ui_day_item_repeat(void)
 	const char *mesg_wrong_freq =
 	    _("The frequence you entered is not valid.");
 	const char *mesg_until_1 =
-	    _("Enter the ending date: [%s] or '0' for an endless repetition");
+	    _("Enter end date ([%s]), duration ([+xxwxxd]) or '0' for endless repetition:");
 	const char *mesg_wrong_1 = _("The entered date is not valid.");
 	const char *mesg_wrong_2 =
-	    _("Possible formats are [%s] or '0' for an endless repetition");
+	    _("Possible formats are [%s], [+xxwxxd] or '0'.");
 	const char *wrong_type_1 =
 	    _("This item is already a repeated one.");
 	const char *wrong_type_2 = _("Press [ENTER] to continue.");
@@ -680,7 +718,8 @@ void ui_day_item_repeat(void)
 	int item_nb;
 	struct day_item *p;
 	struct recur_apoint *ra;
-	long until, date;
+	time_t until, date;
+	unsigned days;
 
 	if (day_item_count(0) <= 0)
 		goto cleanup;
@@ -731,32 +770,47 @@ void ui_day_item_repeat(void)
 		if (getstring(win[STA].p, user_input, BUFSIZ, 0, 1) !=
 		    GETSTRING_VALID)
 			goto cleanup;
-		if (strlen(user_input) == 1
-		    && strcmp(user_input, "0") == 0) {
+		if (strlen(user_input) == 1 && strcmp(user_input, "0") == 0) {
 			until = 0;
 			break;
 		}
-		if (parse_date(user_input, conf.input_datefmt,
-			       &year, &month, &day,
-			       ui_calendar_get_slctd_day())) {
+		if (*user_input == '+') {
+			if (!parse_date_duration(user_input + 1, &days)) {
+				asprintf(&outstr, mesg_wrong_2,
+					 DATEFMT_DESC(conf.input_datefmt));
+				status_mesg(mesg_wrong_1, outstr);
+				mem_free(outstr);
+				wgetch(win[KEY].p);
+				continue;
+			}
+			t = p->start;
+			localtime_r(&t, &lt);
+			date_change(&lt, 0, days);
+			until_date.dd = lt.tm_mday;
+			until_date.mm = lt.tm_mon + 1;
+			until_date.yyyy = lt.tm_year + 1900;
+		} else if (parse_date(user_input, conf.input_datefmt,
+				      &year, &month, &day,
+				      ui_calendar_get_slctd_day())) {
 			t = p->start;
 			localtime_r(&t, &lt);
 			until_date.dd = day;
 			until_date.mm = month;
 			until_date.yyyy = year;
-			until =
-			    date2sec(until_date, lt.tm_hour, lt.tm_min);
-			if (until >= p->start)
-				break;
-			status_mesg(mesg_older, wrong_type_2);
-			wgetch(win[KEY].p);
 		} else {
 			asprintf(&outstr, mesg_wrong_2,
 				 DATEFMT_DESC(conf.input_datefmt));
 			status_mesg(mesg_wrong_1, outstr);
 			mem_free(outstr);
 			wgetch(win[KEY].p);
+			continue;
 		}
+
+		until = date2sec(until_date, lt.tm_hour, lt.tm_min);
+		if (until >= p->start)
+			break;
+		status_mesg(mesg_older, wrong_type_2);
+		wgetch(win[KEY].p);
 	}
 
 	date = ui_calendar_get_slctd_day_sec();
@@ -820,7 +874,7 @@ void ui_day_item_copy(unsigned reg)
 	if (day_item_count(0) <= 0 || reg == REG_BLACK_HOLE)
 		return;
 
-	struct day_item *item = day_get_item(listbox_get_sel(&lb_apt));
+	struct day_item *item = ui_day_selitem();
 	ui_day_item_cut_free(reg);
 	day_item_fork(item, &day_cut[reg]);
 }
@@ -859,19 +913,19 @@ void ui_day_sel_move(int delta)
 void ui_day_draw(int n, WINDOW *win, int y, int hilt, void *cb_data)
 {
 	struct date slctd_date = *ui_calendar_get_slctd_day();
-	long date = date2sec(slctd_date, 0, 0);
+	time_t date = date2sec(slctd_date, 0, 0);
 	struct day_item *item = day_get_item(n);
-	int width = lb_apt.sw.w;
+	int width = lb_apt.sw.w - 2;
 
 	hilt = hilt && (wins_slctd() == APP);
 
 	if (item->type == EVNT || item->type == RECUR_EVNT) {
-		day_display_item(item, win, !hilt, width, y, 1);
+		day_display_item(item, win, !hilt, width - 1, y, 1);
 	} else if (item->type == APPT || item->type == RECUR_APPT) {
 		day_display_item_date(item, win, !hilt, date, y, 1);
-		day_display_item(item, win, !hilt, width, y + 1, 1);
+		day_display_item(item, win, !hilt, width - 1, y + 1, 1);
 	} else if (item->type == DAY_HEADING) {
-		unsigned x = width - (strlen(_(monthnames[slctd_date.mm - 1])) + 17);
+		unsigned x = width - (strlen(_(monthnames[slctd_date.mm - 1])) + 15);
 		custom_apply_attr(win, ATTR_HIGHEST);
 		mvwprintw(win, y, x, "%s  %s %02d, %04d",
 			  ui_calendar_get_pom(date),
@@ -915,7 +969,7 @@ void ui_day_popup_item(void)
 	if (day_item_count(0) <= 0)
 		return;
 
-	struct day_item *item = day_get_item(listbox_get_sel(&lb_apt));
+	struct day_item *item = ui_day_selitem();
 	day_popup_item(item);
 }
 
@@ -924,7 +978,7 @@ void ui_day_flag(void)
 	if (day_item_count(0) <= 0)
 		return;
 
-	struct day_item *item = day_get_item(listbox_get_sel(&lb_apt));
+	struct day_item *item = ui_day_selitem();
 	day_item_switch_notify(item);
 	io_set_modified();
 }
@@ -934,7 +988,7 @@ void ui_day_view_note(void)
 	if (day_item_count(0) <= 0)
 		return;
 
-	struct day_item *item = day_get_item(listbox_get_sel(&lb_apt));
+	struct day_item *item = ui_day_selitem();
 	day_view_note(item, conf.pager);
 }
 
@@ -943,7 +997,7 @@ void ui_day_edit_note(void)
 	if (day_item_count(0) <= 0)
 		return;
 
-	struct day_item *item = day_get_item(listbox_get_sel(&lb_apt));
+	struct day_item *item = ui_day_selitem();
 	day_edit_note(item, conf.editor);
 	io_set_modified();
 }
