@@ -1,7 +1,7 @@
 /*
  * Calcurse - text-based organizer
  *
- * Copyright (c) 2004-2015 calcurse Development Team <misc@calcurse.org>
+ * Copyright (c) 2004-2016 calcurse Development Team <misc@calcurse.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -36,25 +36,42 @@
 
 #include "calcurse.h"
 
+static unsigned ui_todo_view = 0;
+
+static struct todo *ui_todo_selitem(void)
+{
+	return todo_get_item(listbox_get_sel(&lb_todo),
+			     ui_todo_view == TODO_HIDE_COMPLETED_VIEW);
+}
+
+static void ui_todo_set_selitem(struct todo *todo)
+{
+	int n = todo_get_position(todo,
+				  ui_todo_view == TODO_HIDE_COMPLETED_VIEW);
+	if (n >= 0)
+		listbox_set_sel(&lb_todo, n);
+}
+
 /* Request user to enter a new todo item. */
 void ui_todo_add(void)
 {
 	int ch = 0;
 	const char *mesg = _("Enter the new TODO item:");
 	const char *mesg_id =
-	    _("Enter the TODO priority [1 (highest) - 9 (lowest)]:");
+	    _("Enter the TODO priority [0 (none), 1 (highest) - 9 (lowest)]:");
 	char todo_input[BUFSIZ] = "";
 
 	status_mesg(mesg, "");
 	if (getstring(win[STA].p, todo_input, BUFSIZ, 0, 1) ==
 	    GETSTRING_VALID) {
-		while ((ch < '1') || (ch > '9')) {
+		while ((ch < '0') || (ch > '9')) {
 			status_mesg(mesg_id, "");
 			ch = wgetch(win[KEY].p);
 		}
-		todo_add(todo_input, ch - '0', NULL);
+		struct todo *todo = todo_add(todo_input, ch - '0', 0, NULL);
 		ui_todo_load_items();
 		io_set_modified();
+		ui_todo_set_selitem(todo);
 	}
 }
 
@@ -70,15 +87,14 @@ void ui_todo_delete(void)
 	const int nb_erase_choice = 2;
 	int answer;
 
-	if (!LLIST_FIRST(&todolist) ||
-	    (conf.confirm_delete
-	     && (status_ask_bool(del_todo_str) != 1))) {
+	struct todo *item = ui_todo_selitem();
+
+	if (!item || (conf.confirm_delete &&
+		      (status_ask_bool(del_todo_str) != 1))) {
 		wins_erase_status_bar();
 		return;
 	}
 
-	struct todo *item = todo_get_item(listbox_get_sel(&lb_todo));
-
 	if (item->note)
 		answer = status_ask_choice(erase_warning, erase_choice,
 					   nb_erase_choice);
@@ -104,15 +120,18 @@ void ui_todo_delete(void)
 /* Edit the description of an already existing todo item. */
 void ui_todo_edit(void)
 {
-	if (!LLIST_FIRST(&todolist))
-		return;
-
-	struct todo *item = todo_get_item(listbox_get_sel(&lb_todo));
+	struct todo *item = ui_todo_selitem();
 	const char *mesg = _("Enter the new TODO description:");
 
+	if (!item)
+		return;
+
 	status_mesg(mesg, "");
 	updatestring(win[STA].p, &item->mesg, 0, 1);
+	todo_resort(item);
+	ui_todo_load_items();
 	io_set_modified();
+	ui_todo_set_selitem(item);
 }
 
 /* Pipe a todo item to an external program. */
@@ -124,10 +143,10 @@ void ui_todo_pipe(void)
 	int pid;
 	FILE *fpout;
 
-	if (!LLIST_FIRST(&todolist))
-		return;
+	struct todo *item = ui_todo_selitem();
 
-	struct todo *item = todo_get_item(listbox_get_sel(&lb_todo));
+	if (!item)
+		return;
 
 	status_mesg(_("Pipe item to external command:"), "");
 	if (getstring(win[STA].p, cmd, BUFSIZ, 0, 1) != GETSTRING_VALID)
@@ -149,13 +168,34 @@ void ui_todo_draw(int n, WINDOW *win, int y, int hilt, void *cb_data)
 {
 	llist_item_t *i = *((llist_item_t **)cb_data);
 	struct todo *todo = LLIST_TS_GET_DATA(i);
-	int mark[2];
-	int width = lb_todo.sw.w;
+	char mark[] = { 0, 0, 0, 0 };
+	int width = lb_todo.sw.w - 2;
 	char buf[width * UTF8_MAXLEN];
+	char *mesg;
 	int j;
 
-	mark[0] = todo->id > 0 ? '0' + todo->id : 'X';
-	mark[1] = todo->note ? '>' : '.';
+	if (ui_todo_view == TODO_HIDE_COMPLETED_VIEW) {
+		while (i && todo->completed) {
+			i = i->next;
+			if (i)
+				todo = LLIST_TS_GET_DATA(i);
+		}
+	}
+
+	mark[0] = todo->completed ? 'X' : (todo->id > 0 ? '0' + todo->id : 0);
+	if (todo->note) {
+		if (mark[0] == '\0') {
+			mark[0] = '>';
+			mark[1] = ' ';
+		} else {
+			mark[1] = '>';
+			mark[2] = ' ';
+		}
+	} else if (mark[0] != '\0') {
+		mark[1] = '.';
+		mark[2] = ' ';
+	}
+	width -= strlen(mark);
 
 	hilt = hilt && (wins_slctd() == TOD);
 
@@ -163,20 +203,27 @@ void ui_todo_draw(int n, WINDOW *win, int y, int hilt, void *cb_data)
 		custom_apply_attr(win, ATTR_HIGHEST);
 
 	if (utf8_strwidth(todo->mesg) < width) {
-		mvwprintw(win, y, 0, "%c%c %s", mark[0], mark[1], todo->mesg);
+		mesg = todo->mesg;
 	} else {
+		width -= 3;
 		for (j = 0; todo->mesg[j] && width > 0; j++) {
 			if (!UTF8_ISCONT(todo->mesg[j]))
 				width -= utf8_width(&todo->mesg[j]);
 			buf[j] = todo->mesg[j];
 		}
-		if (j)
-			buf[j - 1] = 0;
-		else
+		if (j) {
+			buf[j - 1] = '.';
+			buf[j] = '.';
+			buf[j + 1] = '.';
+			buf[j + 2] = '\0';
+		} else {
 			buf[0] = 0;
-		mvwprintw(win, y, 0, "%c%c %s...", mark[0], mark[1], buf);
+		}
+		mesg = buf;
 	}
 
+	mvwprintw(win, y, 0, "%s%s", mark, mesg);
+
 	if (hilt)
 		custom_remove_attr(win, ATTR_HIGHEST);
 
@@ -199,8 +246,13 @@ void ui_todo_load_items(void)
 	llist_item_t *i;
 
 	/* TODO: Optimize this by keeping the list size in a variable. */
-	LLIST_FOREACH(&todolist, i)
+	LLIST_FOREACH(&todolist, i) {
+		struct todo *todo = LLIST_TS_GET_DATA(i);
+		if (ui_todo_view == TODO_HIDE_COMPLETED_VIEW &&
+		    todo->completed)
+			continue;
 		n++;
+	}
 
 	listbox_load_items(&lb_todo, n);
 }
@@ -219,7 +271,7 @@ void ui_todo_sel_move(int delta)
 void ui_todo_update_panel(int which_pan)
 {
 	/*
-	 * This is used and modified by display_todo_item() to avoid quadratic
+	 * This is used and modified by ui_todo_draw() to avoid quadratic
 	 * running time.
 	 */
 	llist_item_t *p = LLIST_FIRST(&todolist);
@@ -231,58 +283,94 @@ void ui_todo_update_panel(int which_pan)
 /* Change an item priority by pressing '+' or '-' inside TODO panel. */
 void ui_todo_chg_priority(int diff)
 {
-	if (!LLIST_FIRST(&todolist))
+	struct todo *item = ui_todo_selitem();
+
+	if (!item)
 		return;
 
-	struct todo *item = todo_get_item(listbox_get_sel(&lb_todo));
 	int id = item->id + diff;
 	struct todo *item_new;
 
-	if (id < 1)
-		id = 1;
+	if (id < 0)
+		id = 0;
 	else if (id > 9)
 		id = 9;
 
-	item_new = todo_add(item->mesg, id, item->note);
+	item_new = todo_add(item->mesg, id, item->completed, item->note);
 	todo_delete(item);
 	io_set_modified();
-	listbox_set_sel(&lb_todo, todo_get_position(item_new));
+	ui_todo_set_selitem(item_new);
 }
 
 void ui_todo_popup_item(void)
 {
-	if (!LLIST_FIRST(&todolist))
+	struct todo *item = ui_todo_selitem();
+
+	if (!item)
 		return;
 
-	struct todo *item = todo_get_item(listbox_get_sel(&lb_todo));
 	item_in_popup(NULL, NULL, item->mesg, _("TODO:"));
 }
 
 void ui_todo_flag(void)
 {
-	if (!LLIST_FIRST(&todolist))
+	struct todo *item = ui_todo_selitem();
+
+	if (!item)
 		return;
 
-	struct todo *item = todo_get_item(listbox_get_sel(&lb_todo));
 	todo_flag(item);
+	ui_todo_load_items();
 	io_set_modified();
+	ui_todo_set_selitem(item);
 }
 
 void ui_todo_view_note(void)
 {
-	if (!LLIST_FIRST(&todolist))
+	struct todo *item = ui_todo_selitem();
+
+	if (!item)
 		return;
 
-	struct todo *item = todo_get_item(listbox_get_sel(&lb_todo));
 	todo_view_note(item, conf.pager);
 }
 
 void ui_todo_edit_note(void)
 {
-	if (!LLIST_FIRST(&todolist))
+	struct todo *item = ui_todo_selitem();
+
+	if (!item)
 		return;
 
-	struct todo *item = todo_get_item(listbox_get_sel(&lb_todo));
 	todo_edit_note(item, conf.editor);
 	io_set_modified();
 }
+
+/* Switch to next todo view. */
+void ui_todo_view_next(void)
+{
+	ui_todo_view++;
+	if (ui_todo_view == TODO_VIEWS)
+		ui_todo_view = 0;
+	ui_todo_load_items();
+}
+
+/* Switch to previous todo view. */
+void ui_todo_view_prev(void)
+{
+	if (ui_todo_view == 0)
+		ui_todo_view = TODO_VIEWS;
+	ui_todo_view--;
+	ui_todo_load_items();
+}
+
+void ui_todo_set_view(int view)
+{
+	ui_todo_view = (view < 0 || view >= TODO_VIEWS) ?
+		       TODO_SHOW_COMPLETED_VIEW : view;
+}
+
+int ui_todo_get_view(void)
+{
+	return (int)ui_todo_view;
+}
