@@ -1,7 +1,7 @@
 /*
  * Calcurse - text-based organizer
  *
- * Copyright (c) 2004-2015 calcurse Development Team <misc@calcurse.org>
+ * Copyright (c) 2004-2016 calcurse Development Team <misc@calcurse.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -48,6 +48,7 @@
 #include <termios.h>
 
 #include "calcurse.h"
+#include "sha1.h"
 
 #define ISLEAP(y) ((((y) % 4) == 0 && ((y) % 100) != 0) || ((y) % 400) == 0)
 
@@ -62,6 +63,8 @@ enum format_specifier {
 	FS_NOTE,
 	FS_NOTEFILE,
 	FS_PRIORITY,
+	FS_RAW,
+	FS_HASH,
 	FS_PSIGN,
 	FS_EOF,
 	FS_UNKNOWN
@@ -72,6 +75,9 @@ void exit_calcurse(int status)
 {
 	int was_interactive;
 
+	ui_calendar_stop_date_thread();
+	io_stop_psave_thread();
+
 	if (ui_mode == UI_CURSES) {
 		notify_stop_main_thread();
 		clear();
@@ -83,11 +89,10 @@ void exit_calcurse(int status)
 		was_interactive = 0;
 	}
 
-	ui_calendar_stop_date_thread();
-	io_stop_psave_thread();
 	free_user_data();
 	keys_free();
 	mem_stats();
+
 	if (was_interactive) {
 		if (unlink(path_cpid) != 0)
 			EXIT(_("Could not remove calcurse lock file: %s\n"),
@@ -197,19 +202,17 @@ void status_mesg(const char *msg1, const char *msg2)
 int status_ask_choice(const char *message, const char choice[],
 		      int nb_choice)
 {
-	int i, ch;
-	char *tmp;
 	/* "[4/2/f/t/w/.../Z] " */
 	char avail_choice[2 * nb_choice + 3];
+	int i, ch;
 
 	avail_choice[0] = '[';
-	avail_choice[1] = '\0';
-
 	for (i = 1; i <= nb_choice; i++) {
-		asprintf(&tmp, (i == nb_choice) ? "%c] " : "%c/", choice[i]);
-		strcat(avail_choice, tmp);
-		mem_free(tmp);
+		avail_choice[i * 2 - 1] = choice[i];
+		avail_choice[i * 2] = '/';
 	}
+	avail_choice[nb_choice * 2] = ']';
+	avail_choice[nb_choice * 2 + 1] = '\0';
 
 	status_mesg(message, avail_choice);
 
@@ -365,7 +368,7 @@ int get_item_min(long date)
 	return lt.tm_min;
 }
 
-long date2sec(struct date day, unsigned hour, unsigned min)
+struct tm date2tm(struct date day, unsigned hour, unsigned min)
 {
 	time_t t = now();
 	struct tm start;
@@ -380,12 +383,67 @@ long date2sec(struct date day, unsigned hour, unsigned min)
 	start.tm_sec = 0;
 	start.tm_isdst = -1;
 
-	t = mktime(&start);
+	return start;
+}
+
+time_t date2sec(struct date day, unsigned hour, unsigned min)
+{
+	struct tm start = date2tm(day, hour, min);
+	time_t t = mktime(&start);
+
 	EXIT_IF(t == -1, _("failure in mktime"));
 
 	return t;
 }
 
+time_t utcdate2sec(struct date day, unsigned hour, unsigned min)
+{
+	char *tz;
+	time_t t;
+
+	tz = getenv("TZ");
+	if (tz)
+		tz = mem_strdup(tz);
+	setenv("TZ", "", 1);
+	tzset();
+
+	t = date2sec(day, hour, min);
+
+	if (tz) {
+		setenv("TZ", tz, 1);
+		mem_free(tz);
+	} else {
+	    unsetenv("TZ");
+	}
+	tzset();
+
+	return t;
+}
+
+/* Compare two dates (without comparing times). */
+int date_cmp_day(time_t d1, time_t d2)
+{
+	struct tm lt1, lt2;
+
+	localtime_r((time_t *)&d1, &lt1);
+	localtime_r((time_t *)&d2, &lt2);
+
+	if (lt1.tm_year < lt2.tm_year)
+		return -1;
+	if (lt1.tm_year > lt2.tm_year)
+		return 1;
+	if (lt1.tm_mon < lt2.tm_mon)
+		return -1;
+	if (lt1.tm_mon > lt2.tm_mon)
+		return 1;
+	if (lt1.tm_mday < lt2.tm_mday)
+		return -1;
+	if (lt1.tm_mday > lt2.tm_mday)
+		return 1;
+
+	return 0;
+}
+
 /* Return a string containing the date, given a date in seconds. */
 char *date_sec2date_str(long sec, const char *datefmt)
 {
@@ -421,6 +479,26 @@ void date_sec2date_fmt(long sec, const char *fmt, char *datef)
 #endif
 }
 
+/*
+ * Used to change date by adding a certain amount of days or weeks.
+ * Returns 0 on success, 1 otherwise.
+ */
+int date_change(struct tm *date, int delta_month, int delta_day)
+{
+	struct tm t;
+
+	t = *date;
+	t.tm_mon += delta_month;
+	t.tm_mday += delta_day;
+
+	if (mktime(&t) == -1) {
+		return 1;
+	} else {
+		*date = t;
+		return 0;
+	}
+}
+
 /*
  * Used to change date by adding a certain amount of days or weeks.
  */
@@ -453,6 +531,7 @@ long update_time_in_date(long date, unsigned hr, unsigned mn)
 	localtime_r(&t, &lt);
 	lt.tm_hour = hr;
 	lt.tm_min = mn;
+	lt.tm_sec = 0;
 	new_date = mktime(&lt);
 	EXIT_IF(new_date == -1, _("error in mktime"));
 
@@ -463,11 +542,10 @@ long update_time_in_date(long date, unsigned hr, unsigned mn)
  * Returns the date in seconds from year 1900.
  * If no date is entered, current date is chosen.
  */
-long get_sec_date(struct date date)
+time_t get_sec_date(struct date date)
 {
 	struct tm ptrtime;
 	time_t timer;
-	long long_date;
 	char current_day[] = "dd ";
 	char current_month[] = "mm ";
 	char current_year[] = "yyyy ";
@@ -476,16 +554,14 @@ long get_sec_date(struct date date)
 		timer = time(NULL);
 		localtime_r(&timer, &ptrtime);
 		strftime(current_day, strlen(current_day), "%d", &ptrtime);
-		strftime(current_month, strlen(current_month), "%m",
-			 &ptrtime);
-		strftime(current_year, strlen(current_year), "%Y",
-			 &ptrtime);
+		strftime(current_month, strlen(current_month), "%m", &ptrtime);
+		strftime(current_year, strlen(current_year), "%Y", &ptrtime);
 		date.mm = atoi(current_month);
 		date.dd = atoi(current_day);
 		date.yyyy = atoi(current_year);
 	}
-	long_date = date2sec(date, 0, 0);
-	return long_date;
+
+	return date2sec(date, 0, 0);
 }
 
 long min2sec(unsigned minutes)
@@ -541,11 +617,10 @@ item_in_popup(const char *a_start, const char *a_end, const char *msg,
 }
 
 /* Returns the beginning of current day in seconds from 1900. */
-long get_today(void)
+time_t get_today(void)
 {
 	struct tm lt;
 	time_t current_time;
-	long current_day;
 	struct date day;
 
 	current_time = time(NULL);
@@ -553,9 +628,8 @@ long get_today(void)
 	day.mm = lt.tm_mon + 1;
 	day.dd = lt.tm_mday;
 	day.yyyy = lt.tm_year + 1900;
-	current_day = date2sec(day, 0, 0);
 
-	return current_day;
+	return date2sec(day, 0, 0);
 }
 
 /* Returns the current time in seconds. */
@@ -811,6 +885,77 @@ parse_date(const char *date_string, enum datefmt datefmt, int *year,
 	return 1;
 }
 
+/*
+ * Convert a date duration string into a number of days.
+ *
+ * Returns 1 on success and 0 on failure.
+ */
+int parse_date_duration(const char *string, unsigned *days)
+{
+	enum {
+		STATE_INITIAL,
+		STATE_WWDD_DD,
+		STATE_DONE
+	} state = STATE_INITIAL;
+
+	const char *p;
+	unsigned in = 0, frac = 0, denom = 1;
+	unsigned dur = 0;
+
+	if (!string || *string == '\0')
+		return 0;
+
+	/* parse string using a simple state machine */
+	for (p = string; *p; p++) {
+		if (state == STATE_DONE) {
+			return 0;
+		} else if ((*p >= '0') && (*p <= '9')) {
+			in = in * 10 + (int)(*p - '0');
+			if (frac)
+				denom *= 10;
+		} else if (*p == '.') {
+			if (frac)
+				return 0;
+			frac++;
+		} else {
+			switch (state) {
+			case STATE_INITIAL:
+				if (*p == 'w') {
+					dur += in * WEEKINDAYS / denom;
+					state = STATE_WWDD_DD;
+				} else if (*p == 'd') {
+					dur += in / denom;
+					state = STATE_DONE;
+				} else {
+					return 0;
+				}
+				break;
+			case STATE_WWDD_DD:
+				if (*p == 'd') {
+					dur += in / denom;
+					state = STATE_DONE;
+				} else {
+					return 0;
+				}
+				break;
+			default:
+				break;
+			}
+
+			in = frac = 0;
+			denom = 1;
+		}
+	}
+
+	if (state == STATE_DONE && in > 0)
+		return 0;
+
+	dur += in;
+	*days = dur;
+
+	return 1;
+}
+
 /*
  * Check if time is valid.
  */
@@ -879,7 +1024,7 @@ int parse_duration(const char *string, unsigned *duration)
 	} state = STATE_INITIAL;
 
 	const char *p;
-	unsigned in = 0;
+	unsigned in = 0, frac = 0, denom = 1;
 	unsigned dur = 0;
 
 	if (!string || *string == '\0')
@@ -891,20 +1036,26 @@ int parse_duration(const char *string, unsigned *duration)
 			return 0;
 		} else if ((*p >= '0') && (*p <= '9')) {
 			in = in * 10 + (int)(*p - '0');
+			if (frac)
+				denom *= 10;
+		} else if (*p == '.') {
+			if (frac)
+				return 0;
+			frac++;
 		} else {
 			switch (state) {
 			case STATE_INITIAL:
 				if (*p == ':') {
-					dur += in * HOURINMIN;
+					dur += in * HOURINMIN / denom;
 					state = STATE_HHMM_MM;
 				} else if (*p == 'd') {
-					dur += in * DAYINMIN;
+					dur += in * DAYINMIN / denom;
 					state = STATE_DDHHMM_HH;
 				} else if (*p == 'h') {
-					dur += in * HOURINMIN;
+					dur += in * HOURINMIN / denom;
 					state = STATE_DDHHMM_MM;
 				} else if (*p == 'm') {
-					dur += in;
+					dur += in / denom;
 					state = STATE_DONE;
 				} else {
 					return 0;
@@ -912,10 +1063,10 @@ int parse_duration(const char *string, unsigned *duration)
 				break;
 			case STATE_DDHHMM_HH:
 				if (*p == 'h') {
-					dur += in * HOURINMIN;
+					dur += in * HOURINMIN / denom;
 					state = STATE_DDHHMM_MM;
 				} else if (*p == 'm') {
-					dur += in;
+					dur += in / denom;
 					state = STATE_DONE;
 				} else {
 					return 0;
@@ -923,7 +1074,7 @@ int parse_duration(const char *string, unsigned *duration)
 				break;
 			case STATE_DDHHMM_MM:
 				if (*p == 'm') {
-					dur += in;
+					dur += in / denom;
 					state = STATE_DONE;
 				} else {
 					return 0;
@@ -936,7 +1087,8 @@ int parse_duration(const char *string, unsigned *duration)
 				break;
 			}
 
-			in = 0;
+			in = frac = 0;
+			denom = 1;
 		}
 	}
 
@@ -951,6 +1103,49 @@ int parse_duration(const char *string, unsigned *duration)
 	return 1;
 }
 
+/*
+ * Converts a string containing a date or a time into a time stamp.
+ *
+ * Takes a date/time string and a time stamp. If the string only contains a
+ * date, the date of the time stamp is updated while the time remains
+ * untouched. If the string only contains a time, the time of the time stamp is
+ * updated and the date remains the same. If the string contains both a date
+ * and a time, the time stamp is updated to match the given string.
+ *
+ * Returns a positive value on success and 0 on failure. The least-significant
+ * bit is set if the date was updated. Bit 1 is set if the time was updated.
+ */
+int parse_datetime(const char *string, long *ts)
+{
+	char *t = mem_strdup(string);
+	char *p = strtok(t, " ");
+
+	unsigned int hour, minute;
+	int year, month, day;
+	struct date new_date;
+	int ret = 0;
+
+	while (p) {
+		if (parse_date(p, conf.input_datefmt, &year, &month, &day,
+			       ui_calendar_get_slctd_day())) {
+			new_date.dd = day;
+			new_date.mm = month;
+			new_date.yyyy = year;
+			*ts = date2sec(new_date, 0, 0) + get_item_time(*ts);
+			ret |= PARSE_DATETIME_HAS_DATE;
+		} else if (parse_time(p, &hour, &minute) == 1) {
+			*ts = update_time_in_date(*ts, hour, minute);
+			ret |= PARSE_DATETIME_HAS_TIME;
+		} else {
+			return 0;
+		}
+		p = strtok(NULL, " ");
+	}
+
+	mem_free(t);
+	return ret;
+}
+
 void file_close(FILE * f, const char *pos)
 {
 	EXIT_IF((fclose(f)) != 0, _("Error when closing file at %s"), pos);
@@ -1269,6 +1464,10 @@ static enum format_specifier parse_fs(const char **s, char *extformat)
 			return FS_NOTEFILE;
 		else if (!strcmp(buf, "priority"))
 			return FS_PRIORITY;
+		else if (!strcmp(buf, "raw"))
+			return FS_RAW;
+		else if (!strcmp(buf, "hash"))
+			return FS_HASH;
 		else
 			return FS_UNKNOWN;
 	case '%':
@@ -1381,12 +1580,12 @@ static void print_datediff(long difference, const char *extformat)
 }
 
 /* Print a formatted appointment to stdout. */
-void print_apoint(const char *format, long day, struct apoint *apt)
+static void print_apoint_helper(const char *format, long day,
+				struct apoint *apt, struct recur_apoint *rapt)
 {
 	const char *p;
 	char extformat[FS_EXT_MAXLEN];
 
-
 	for (p = format; *p; p++) {
 		if (*p == '%') {
 			p++;
@@ -1418,6 +1617,18 @@ void print_apoint(const char *format, long day, struct apoint *apt)
 			case FS_NOTEFILE:
 				print_notefile(stdout, apt->note, 1);
 				break;
+			case FS_RAW:
+				if (rapt)
+					recur_apoint_write(rapt, stdout);
+				else
+					apoint_write(apt, stdout);
+				break;
+			case FS_HASH:
+				if (rapt)
+					printf("%s", recur_apoint_hash(rapt));
+				else
+					printf("%s", apoint_hash(apt));
+				break;
 			case FS_PSIGN:
 				putchar('%');
 				break;
@@ -1437,7 +1648,8 @@ void print_apoint(const char *format, long day, struct apoint *apt)
 }
 
 /* Print a formatted event to stdout. */
-void print_event(const char *format, long day, struct event *ev)
+static void print_event_helper(const char *format, long day, struct event *ev,
+			       struct recur_event *rev)
 {
 	const char *p;
 	char extformat[FS_EXT_MAXLEN];
@@ -1458,6 +1670,18 @@ void print_event(const char *format, long day, struct event *ev)
 			case FS_PSIGN:
 				putchar('%');
 				break;
+			case FS_RAW:
+				if (rev)
+					recur_event_write(rev, stdout);
+				else
+					event_write(ev, stdout);
+				break;
+			case FS_HASH:
+				if (rev)
+					printf("%s", recur_event_hash(rev));
+				else
+					printf("%s", event_hash(ev));
+				break;
 			case FS_EOF:
 				return;
 				break;
@@ -1473,9 +1697,21 @@ void print_event(const char *format, long day, struct event *ev)
 	}
 }
 
+/* Print a formatted appointment to stdout. */
+void print_apoint(const char *format, long day, struct apoint *apt)
+{
+	print_apoint_helper(format, day, apt, NULL);
+}
+
+/* Print a formatted event to stdout. */
+void print_event(const char *format, long day, struct event *ev)
+{
+	print_event_helper(format, day, ev, NULL);
+}
+
 /* Print a formatted recurrent appointment to stdout. */
 void
-print_recur_apoint(const char *format, long day, unsigned occurrence,
+print_recur_apoint(const char *format, long day, time_t occurrence,
 		   struct recur_apoint *rapt)
 {
 	struct apoint apt;
@@ -1485,7 +1721,7 @@ print_recur_apoint(const char *format, long day, unsigned occurrence,
 	apt.mesg = rapt->mesg;
 	apt.note = rapt->note;
 
-	print_apoint(format, day, &apt);
+	print_apoint_helper(format, day, &apt, rapt);
 }
 
 /* Print a formatted recurrent event to stdout. */
@@ -1497,7 +1733,7 @@ void print_recur_event(const char *format, long day,
 	ev.mesg = rev->mesg;
 	ev.note = rev->note;
 
-	print_event(format, day, &ev);
+	print_event_helper(format, day, &ev, rev);
 }
 
 /* Print a formatted todo item to stdout. */
@@ -1522,6 +1758,12 @@ void print_todo(const char *format, struct todo *todo)
 			case FS_NOTEFILE:
 				print_notefile(stdout, todo->note, 1);
 				break;
+			case FS_RAW:
+				todo_write(todo, stdout);
+				break;
+			case FS_HASH:
+				printf("%s", todo_hash(todo));
+				break;
 			case FS_PSIGN:
 				putchar('%');
 				break;
@@ -1540,40 +1782,47 @@ void print_todo(const char *format, struct todo *todo)
 	}
 }
 
-#define VASPRINTF_INITIAL_BUFSIZE 128
-
-int
-vasprintf(char **str, const char *format, va_list ap)
-{
-	va_list ap2;
-	int n;
-
-	va_copy(ap2, ap);
-
-	*str = mem_malloc(VASPRINTF_INITIAL_BUFSIZE);
-	n = vsnprintf(*str, VASPRINTF_INITIAL_BUFSIZE, format, ap);
-	if (n >= VASPRINTF_INITIAL_BUFSIZE) {
-		*str = mem_realloc(*str, 1, n + 1);
-		n = vsnprintf(*str, n + 1, format, ap2);
-	}
-
-	if (n < 0) {
-		mem_free(*str);
-		*str = NULL;
-	}
-
-	return n;
-}
-
 int
 asprintf(char **str, const char *format, ...)
 {
+	struct string s;
 	va_list	ap;
 	int n;
 
 	va_start(ap, format);
-	n = vasprintf(str, format, ap);
+	string_init(&s);
+	n = string_vcatf(&s, format, ap);
+	*str = string_buf(&s);
 	va_end(ap);
 
 	return n;
 }
+
+int starts_with(const char *s, const char *p)
+{
+	for (; *p && *p == *s; s++, p++);
+	return (*p == '\0');
+}
+
+int starts_with_ci(const char *s, const char *p)
+{
+	for (; *p && tolower(*p) == tolower(*s); s++, p++);
+	return (*p == '\0');
+}
+
+int hash_matches(const char *pattern, const char *hash)
+{
+	int invert = 0;
+
+	if (pattern[0] == '!') {
+		invert = 1;
+		pattern++;
+	}
+
+	return (starts_with(hash, pattern) != invert);
+}
+
+int show_dialogs(void)
+{
+	return (!quiet) && conf.system_dialogs;
+}
