@@ -1,7 +1,7 @@
 /*
  * Calcurse - text-based organizer
  *
- * Copyright (c) 2004-2015 calcurse Development Team <misc@calcurse.org>
+ * Copyright (c) 2004-2016 calcurse Development Team <misc@calcurse.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -53,7 +53,7 @@ void vector_free(vector_t *v)
 {
 	v->count = 0;
 	v->size = 0;
-	free(v->data);
+	mem_free(v->data);
 	v->data = NULL;
 }
 
@@ -100,7 +100,7 @@ void vector_add(vector_t *v, void *data)
 {
 	if (v->count >= v->size) {
 		v->size *= 2;
-		v->data = realloc(v->data, v->size * sizeof(void *));
+		v->data = mem_realloc(v->data, v->size, sizeof(void *));
 	}
 
 	v->data[v->count] = data;
