@@ -1,7 +1,7 @@
 /*
  * Calcurse - text-based organizer
  *
- * Copyright (c) 2004-2015 calcurse Development Team <misc@calcurse.org>
+ * Copyright (c) 2004-2016 calcurse Development Team <misc@calcurse.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -665,6 +665,7 @@ void wins_update_bindings(void)
 	static int bindings_todo[] = {
 		KEY_GENERIC_HELP, KEY_GENERIC_QUIT, KEY_GENERIC_SAVE,
 		KEY_GENERIC_RELOAD, KEY_GENERIC_CHANGE_VIEW,
+		KEY_GENERIC_SCROLL_DOWN, KEY_GENERIC_SCROLL_UP,
 		KEY_GENERIC_IMPORT, KEY_GENERIC_EXPORT, KEY_ADD_ITEM,
 		KEY_DEL_ITEM, KEY_EDIT_ITEM, KEY_VIEW_ITEM, KEY_PIPE_ITEM,
 		KEY_FLAG_ITEM, KEY_RAISE_PRIORITY, KEY_LOWER_PRIORITY,
@@ -716,11 +717,24 @@ void wins_erase_status_bar(void)
 /* Update the status bar page number to display other commands. */
 void wins_other_status_page(int panel)
 {
-	int max_page = bindings_size / (KEYS_CMDS_PER_LINE * 2 - 1) + 1;
+	/*
+	 * Determine the number of completely filled pages of key bindings.
+	 * There are two lines of bindings and KEYS_CMDS_PER_LINE bindings per
+	 * line. On each page (other than the last page), one slot is reserved
+	 * for OtherCmd.
+	 */
+	const int slots_per_page = KEYS_CMDS_PER_LINE * 2 - 1;
+	int max_page = bindings_size / slots_per_page;
 
-	/* There is no "OtherCmd" on the last page. */
-	if (bindings_size % (KEYS_CMDS_PER_LINE * 2 - 1) == 1)
-		max_page--;
+	/*
+	 * The result of the previous computation might have been rounded down.
+	 * In this case, there are some bindings left. If there is exactly one
+	 * binding left, it can be squashed onto the last page in place of the
+	 * OtherCmd binding. If there are at least two bindings left, we need
+	 * to add another page.
+	 */
+	if (bindings_size % slots_per_page > 1)
+		max_page++;
 
 	status_page = (status_page % max_page) + 1;
 }
